@@ -48,9 +48,13 @@ namespace MonoDevelop.Core.Gui
 		
 		public void LoadCommands (string addinPath)
 		{
-			object[] commandCodons = Runtime.AddInService.GetTreeItems (addinPath);
-			foreach (Command cmd in commandCodons)
-				manager.RegisterCommand (cmd, null);
+			Runtime.AddInService.RegisterExtensionItemListener (addinPath, OnExtensionChange);
+		}
+		
+		void OnExtensionChange (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add)
+				manager.RegisterCommand (item as Command, null);
 		}
 		
 		public void EnableUpdate ()
@@ -110,6 +114,14 @@ namespace MonoDevelop.Core.Gui
 		
 		public void ShowContextMenu (Gtk.Menu menu)
 		{
+			ShowContextMenu (menu, null);
+		}
+		
+		public void ShowContextMenu (Gtk.Menu menu, object initialCommandTarget)
+		{
+			if (menu is CommandMenu) {
+				((CommandMenu)menu).InitialCommandTarget = initialCommandTarget;
+			}
 			menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
 		}
 		
@@ -125,7 +137,12 @@ namespace MonoDevelop.Core.Gui
 		
 		public void ShowContextMenu (CommandEntrySet cset)
 		{
-			manager.ShowContextMenu (cset);
+			ShowContextMenu (cset, null);
+		}
+		
+		public void ShowContextMenu (CommandEntrySet cset, object initialTarget)
+		{
+			manager.ShowContextMenu (cset, initialTarget);
 		}
 		
 		public void ShowContextMenu (string addinPath)
