@@ -28,6 +28,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Xml;
 using System.Xml.Serialization;
@@ -49,25 +50,29 @@ namespace MonoDevelop.Core.AddIns.Setup
 		ArrayList addinSetupInfos;
 		AddInStatus addinStatus;
 		AddinSystemConfiguration config;
-		string[] addInDirs;
+		SystemConfiguration systemConfig;
+		
+		ArrayList addInDirs = new ArrayList ();
 		FileUtilityService fileUtilityService = (FileUtilityService)ServiceManager.GetService(typeof(FileUtilityService));
 		
 		internal void Initialize (string[] addInDirs, bool ignoreDefaultPath)
 		{
-			if (ignoreDefaultPath && addInDirs == null)
-				this.addInDirs = new string [0];
-			else if (ignoreDefaultPath)
-				this.addInDirs = addInDirs;
+			if (ignoreDefaultPath && addInDirs == null) {
+				// nothing
+			} else if (ignoreDefaultPath)
+				this.addInDirs.AddRange (addInDirs);
 			else if (addInDirs != null) {
-				this.addInDirs = new string [addInDirs.Length + 1];
-				this.addInDirs [0] = RootAddinPath;
-				addInDirs.CopyTo (this.addInDirs, 1);
+				this.addInDirs.Add (RootAddinPath);
+				this.addInDirs.AddRange (addInDirs);
 			} else {
-				this.addInDirs = new string [] { RootAddinPath, UserAddinPath };
+				this.addInDirs.Add (RootAddinPath);
+				this.addInDirs.Add (UserAddinPath);
 			}
+			foreach (string s in SystemConfiguration.AddinPaths)
+				this.addInDirs.Add (s);
 		}
 		
-		string[] AddinDirectories {
+		ArrayList AddinDirectories {
 			get { return addInDirs; }
 		}
 		
@@ -87,7 +92,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 			get { return Path.Combine (BinPath, "../AddIns"); }
 		}
 		
-		string RootConfigFile {
+		string UserConfigFile {
 			get { return Path.Combine (UserConfigPath, "addins.config"); }
 		}
 		
@@ -95,6 +100,44 @@ namespace MonoDevelop.Core.AddIns.Setup
 			get { return Path.Combine (UserConfigPath, "addins"); }
 		}
 		
+		string SystemConfigPath {
+			get {
+				if (Environment.OSVersion.Platform == PlatformID.Unix) {
+					if (BinPath.StartsWith ("/usr/lib"))
+						return "/etc/monodevelop";
+					else
+						return BinPath + "/../../../etc/monodevelop";
+				} else {
+					return BinPath;
+				}
+			}
+		}
+		
+		string SystemConfigFile {
+			get { return Path.Combine (SystemConfigPath, "monodevelop.config"); }
+		}
+		
+		public void AddAddinPath (string path)
+		{
+			if (!SystemConfiguration.AddinPaths.Contains (path)) {
+				SystemConfiguration.AddinPaths.Add (path);
+				SaveSystemConfiguration ();
+			}
+		}
+		
+		public void RemoveAddinPath (string path)
+		{
+			SystemConfiguration.AddinPaths.Remove (path);
+			SaveSystemConfiguration ();
+		}
+		
+		public string[] GetAddinPaths ()
+		{
+			string[] dirs = new string [SystemConfiguration.AddinPaths.Count];
+			SystemConfiguration.AddinPaths.CopyTo (dirs, 0);
+			return dirs;
+		}
+		
 		public bool Install (IProgressMonitor monitor, params string[] files)
 		{
 			Package[] packages = new Package [files.Length];
@@ -620,6 +663,49 @@ namespace MonoDevelop.Core.AddIns.Setup
 			return addinSetupInfos;
 		}
 		
+		internal bool IsAddinEnabled (string id)
+		{
+			return !Configuration.DisabledAddins.Contains (id);
+		}
+		
+		internal void EnableAddin (string id)
+		{
+			if (IsAddinEnabled (id))
+				return;
+			
+			Configuration.DisabledAddins.Remove (id);
+			SaveConfiguration ();
+
+			// Enable required add-ins
+			
+			AddinSetupInfo ainfo = GetInstalledAddin (id);
+			
+			foreach (PackageDependency dep in ainfo.Addin.Dependencies) {
+				if (dep is AddinDependency)
+					EnableAddin (((AddinDependency)dep).AddinId);
+			}
+		}
+		
+		internal void DisableAddin (string id)
+		{
+			if (!IsAddinEnabled (id))
+				return;
+
+			Configuration.DisabledAddins.Add (id);
+			SaveConfiguration ();
+			
+			// Disable all add-ins which depend on it
+			
+			foreach (AddinSetupInfo ainfo in InternalGetInstalledAddins ()) {
+				foreach (PackageDependency dep in ainfo.Addin.Dependencies) {
+					if (dep is AddinDependency && ((AddinDependency)dep).AddinId == id) {
+						DisableAddin (ainfo.Addin.Id);
+						break;
+					}
+				}
+			}
+		}
+		
 		internal AddInStatus GetAddInStatus ()
 		{
 			if (addinStatus != null) return addinStatus;
@@ -629,17 +715,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 			
 			foreach (AddinSetupInfo ia in InternalGetInstalledAddins ()) {
 				AddinConfiguration conf = ia.GetConfiguration ();
-				foreach (XmlElement elem in conf.Content.SelectNodes ("AddIn/Extension")) {
-					string path = elem.GetAttribute ("path");
-					AddChildExtensions (ia.Addin.Id, hash, path, elem);
-				}
-				foreach (XmlElement elem in conf.Content.SelectNodes ("AddIn/Extension[@path='/Workspace/Applications']/Class")) {
-					ApplicationRecord arec = new ApplicationRecord ();
-					arec.Id = elem.GetAttribute ("id");
-					arec.Description = elem.GetAttribute ("description");
-					arec.AddIn = ia.Addin.Id;
-					apps.Add (arec);
-				}
+				CollectExtensionData (conf.Content.DocumentElement, ia, hash, apps);
 			}
 			addinStatus = new AddInStatus ();
 			ExtensionRelation[] rels = new ExtensionRelation [hash.Count];
@@ -656,6 +732,24 @@ namespace MonoDevelop.Core.AddIns.Setup
 			return addinStatus;
 		}
 		
+		void CollectExtensionData (XmlElement rootElem, AddinSetupInfo ia, Hashtable hash, ArrayList apps)
+		{
+			foreach (XmlElement elem in rootElem.SelectNodes ("Extension")) {
+				string path = elem.GetAttribute ("path");
+				AddChildExtensions (ia.Addin.Id, hash, path, elem);
+			}
+			foreach (XmlElement elem in rootElem.SelectNodes ("Extension[@path='/Workspace/Applications']/Class")) {
+				ApplicationRecord arec = new ApplicationRecord ();
+				arec.Id = elem.GetAttribute ("id");
+				arec.Description = elem.GetAttribute ("description");
+				arec.AddIn = ia.Addin.Id;
+				apps.Add (arec);
+			}
+			foreach (XmlElement elem in rootElem.SelectNodes ("Module")) {
+				CollectExtensionData (elem, ia, hash, apps);
+			}
+		}
+		
 		void AddChildExtensions (string addin, Hashtable hash, string path, XmlElement elem)
 		{
 			ArrayList list = (ArrayList) hash [path];
@@ -731,7 +825,11 @@ namespace MonoDevelop.Core.AddIns.Setup
 			string name = Path.Combine (UserConfigPath, "repository-cache");
 			if (!Directory.Exists (name))
 				Directory.CreateDirectory (name);
-			name = Path.Combine (name, new Uri (url).Host);
+				
+			string host = new Uri (url).Host;
+			if (host == null || host.Length == 0)
+				host = "local";
+			name = Path.Combine (name, host);
 			rr.File = name + "_" + Configuration.RepositoryIdCount + ".mrep";
 			
 			rr.Id = "rep" + Configuration.RepositoryIdCount;
@@ -959,7 +1057,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 			ZipOutputStream s = new ZipOutputStream (File.Create (outFilePath));
 			s.SetLevel(5);
 			
-			ArrayList list = new ArrayList ();
+			List<string> list = new List<string> ();
 			list.Add (Path.GetFileName (filePath));
 			list.AddRange (conf.AllFiles);
 			
@@ -1147,7 +1245,7 @@ namespace MonoDevelop.Core.AddIns.Setup
 		AddinSystemConfiguration Configuration {
 			get {
 				if (config == null) {
-					config = (AddinSystemConfiguration) ReadObject (RootConfigFile, typeof(AddinSystemConfiguration));
+					config = (AddinSystemConfiguration) ReadObject (UserConfigFile, typeof(AddinSystemConfiguration));
 					if (config == null) {
 						config = new AddinSystemConfiguration ();
 						RegisterRepository ("http://go-mono.com/md/main.mrep", false);
@@ -1157,10 +1255,29 @@ namespace MonoDevelop.Core.AddIns.Setup
 			}
 		}
 		
+		SystemConfiguration SystemConfiguration {
+			get {
+				if (systemConfig == null) {
+					systemConfig = (SystemConfiguration) ReadObject (SystemConfigFile, typeof(SystemConfiguration));
+					if (systemConfig == null) {
+						systemConfig = new SystemConfiguration ();
+					}
+				}
+				return systemConfig;
+			}
+		}
+		
 		void SaveConfiguration ()
 		{
 			if (config != null) {
-				WriteObject (RootConfigFile, config); 
+				WriteObject (UserConfigFile, config); 
+			}
+		}
+		
+		void SaveSystemConfiguration ()
+		{
+			if (systemConfig != null) {
+				WriteObject (SystemConfigFile, systemConfig); 
 			}
 		}
 		
