@@ -34,6 +34,7 @@ namespace MonoDevelop.Core.AddIns
 		static int internalIdCount;
 		
 		AddIn  addIn = null;
+		AddIn.Extension extension;
 		
 		/// <summary>
 		/// Returns the AddIn in which the codon is defined.
@@ -123,6 +124,12 @@ namespace MonoDevelop.Core.AddIns
 			}
 		}
 		
+		public AddIn.Extension Extension {
+			get { return extension; }
+			internal set { extension = value; }
+		}
+		
+		
 		/// <summary>
 		/// Creates an item with the specified sub items and the current
 		/// Conditions for this item.
