@@ -17,7 +17,7 @@ namespace MonoDevelop.Core.Execution
 	public class ProcessService : AbstractService
 	{
 		ProcessHostController externalProcess;
-		ExecutionHandlerCodon[] executionHandlers;
+		ArrayList executionHandlers;
 		string remotingChannel = "unix";
 		string unixRemotingFile;
 		
@@ -62,7 +62,7 @@ namespace MonoDevelop.Core.Execution
 			if (exited != null)
 				p.Exited += exited;
 				
-			if(arguments == null || arguments.Length == 0)
+			if(String.IsNullOrEmpty (arguments))
 				p.StartInfo = new ProcessStartInfo (command);
 			else
 				p.StartInfo = new ProcessStartInfo (command, arguments);
@@ -112,14 +112,22 @@ namespace MonoDevelop.Core.Execution
 		
 		public IExecutionHandler GetDefaultExecutionHandler (string platformId)
 		{
-			if (executionHandlers == null)
-				executionHandlers = (ExecutionHandlerCodon[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/ExecutionHandlers", typeof(ExecutionHandlerCodon));
+			if (executionHandlers == null) {
+				executionHandlers = new ArrayList ();
+				Runtime.AddInService.RegisterExtensionItemListener ("/SharpDevelop/Workbench/ExecutionHandlers", OnExtensionChange);
+			}
 			
 			foreach (ExecutionHandlerCodon codon in executionHandlers)
 				if (codon.Platform == platformId) return codon.ExecutionHandler;
 			return null;
 		}
 		
+		void OnExtensionChange (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add)
+				executionHandlers.Add (item);
+		}
+		
 		ProcessHostController GetHost (string id, bool shared)
 		{
 			if (!shared)
@@ -140,12 +148,24 @@ namespace MonoDevelop.Core.Execution
 		
 		public RemoteProcessObject CreateExternalProcessObject (Type type, bool shared)
 		{
-			return GetHost (type.ToString(), shared).CreateInstance (type.Assembly.Location, type.FullName);
+			return GetHost (type.ToString(), shared).CreateInstance (type.Assembly.Location, type.FullName, GetRequiredAddins (type));
 		}
 		
-		public RemoteProcessObject CreateExternalProcessObject (string assemblyPath, string typeName, bool shared)
+		public RemoteProcessObject CreateExternalProcessObject (string assemblyPath, string typeName, bool shared, params string[] requiredAddins)
 		{
-			return GetHost (typeName, shared).CreateInstance (assemblyPath, typeName);
+			return GetHost (typeName, shared).CreateInstance (assemblyPath, typeName, requiredAddins);
+		}
+		
+		string[] GetRequiredAddins (Type type)
+		{
+			if (type.IsDefined (typeof(AddinDependencyAttribute), true)) {
+				object[] ats = type.GetCustomAttributes (typeof(AddinDependencyAttribute), true);
+				string[] addins = new string [ats.Length];
+				for (int n=0; n<ats.Length; n++)
+					addins [n] = ((AddinDependencyAttribute)ats [n]).Addin;
+				return addins;
+			} else
+				return null;
 		}
 		
 		public string ExternalProcessRemotingChannel {
