@@ -128,12 +128,12 @@ namespace MonoDevelop.Core
 		{
 			// Fixme: 260 is the hardcoded maximal length for a path on my Windows XP system
 			//        I can't find a .NET property or method for determining this variable.
-			if (fileName == null || fileName.Length == 0 || fileName.Length >= 260) {
+			if (String.IsNullOrEmpty (fileName) || fileName.Length >= 260) {
 				return false;
 			}
 			
-			// platform independend : check for invalid path chars
-			foreach (char invalidChar in Path.InvalidPathChars) {
+			// platform independent : check for invalid path chars
+			foreach (char invalidChar in Path.GetInvalidPathChars()) {
 				if (fileName.IndexOf(invalidChar) >= 0) {
 					return false;
 				}
