@@ -41,7 +41,7 @@ namespace MonoDevelop.Core
 			}
 
 			configDirectory = Environment.GetEnvironmentVariable ("XDG_CONFIG_HOME");
-			if (configDirectory == null || configDirectory == "")
+			if (String.IsNullOrEmpty (configDirectory))
 				configDirectory = System.IO.Path.Combine (Environment.GetEnvironmentVariable ("HOME"), ".config");
 
 			configDirectory = System.IO.Path.Combine (configDirectory, "MonoDevelop");
@@ -103,7 +103,7 @@ namespace MonoDevelop.Core
 					return false;
 				}
 				SetValueFromXmlElement(doc.DocumentElement["Properties"]);
-			} catch (Exception e) {
+			} catch {
 				//Console.WriteLine("Exception while load properties from stream :\n " + e.ToString());
 				return false;
 			}
