@@ -54,7 +54,10 @@ namespace Gdl
 		
 		public event DockItemMotionHandler DockItemMotion;
 		public event DockItemDragBeginHandler DockItemDragBegin;
-		public event DockItemDragEndHandler DockItemDragEnd;		
+		public event DockItemDragEndHandler DockItemDragEnd;
+		
+		public event EventHandler DockItemShown;
+		public event EventHandler DockItemHidden;
 				
 		static DockItem ()
 		{
@@ -365,8 +368,8 @@ namespace Gdl
 				
 				childAlloc.X = bw + Style.XThickness;
 				childAlloc.Y = bw + Style.YThickness;
-				childAlloc.Width = allocation.Width - 2 * (bw + Style.XThickness);
-				childAlloc.Height = allocation.Height - 2 * (bw + Style.YThickness);
+				childAlloc.Width = Math.Max (0, allocation.Width - 2 * (bw + Style.XThickness));
+				childAlloc.Height = Math.Max (0, allocation.Height - 2 * (bw + Style.YThickness));
 				
 				if (GripShown) {
 					Gdk.Rectangle gripAlloc = childAlloc;
@@ -374,11 +377,11 @@ namespace Gdl
 					
 					if (Orientation == Orientation.Horizontal) {
 						childAlloc.X += gripReq.Width;
-						childAlloc.Width -= gripReq.Width;
+						childAlloc.Width = Math.Max (0, childAlloc.Width - gripReq.Width);
 						gripAlloc.Width = gripReq.Width;
 					} else {
 						childAlloc.Y += gripReq.Height;
-						childAlloc.Height -= gripReq.Height;
+						childAlloc.Height = Math.Max (0, childAlloc.Height - gripReq.Height);
 						gripAlloc.Height = gripReq.Height;
 					}
 					
@@ -471,6 +474,7 @@ namespace Gdl
 				grip = null;
 			}
 			if (dockPlaceHolder != null) {
+				dockPlaceHolder.Dispose ();
 				dockPlaceHolder = null;
 			}
 			base.OnDestroyed ();
@@ -688,7 +692,7 @@ namespace Gdl
 		{
 			DockObject parent = ParentObject;
 			DockObject newParent = null;
-			bool addOurselvesFirst;
+			bool addOurselvesFirst = false;
 			
 			switch (position) {
 			case DockPlacement.Top:
@@ -702,8 +706,12 @@ namespace Gdl
 				addOurselvesFirst = (position == DockPlacement.Right);
 				break;
 			case DockPlacement.Center:
-				newParent = new DockNotebook ();
-				addOurselvesFirst = true;
+				// If the parent is already a DockNotebook, we don't need
+				// to create a new one.
+				if (!(parent is DockNotebook)) {
+					newParent = new DockNotebook ();
+					addOurselvesFirst = true;
+				}
 				break;
 			default:
 				Console.WriteLine ("Unsupported docking strategy");
@@ -713,33 +721,44 @@ namespace Gdl
 			if (parent != null)
 				parent.Freeze ();
 
-			DockObjectFlags |= DockObjectFlags.InReflow;
-			Detach (false);
-			newParent.Freeze ();
-			newParent.Bind (Master);
+			if (newParent != null) {
+				DockObjectFlags |= DockObjectFlags.InReflow;
+				Detach (false);
+				newParent.Freeze ();
+				newParent.Bind (Master);
+				
+				if (addOurselvesFirst) {
+					newParent.Add (this);
+					newParent.Add (requestor);
+				} else {
+					newParent.Add (requestor);
+					newParent.Add (this);
+				}
+				
+				if (parent != null)
+					parent.Add (newParent);
 			
-			if (addOurselvesFirst) {
-				newParent.Add (this);
-				newParent.Add (requestor);
+				if (Visible)
+					newParent.Show ();
+			
+				DockObjectFlags &= ~(DockObjectFlags.InReflow);
+			
+				newParent.Thaw ();
 			} else {
-				newParent.Add (requestor);
-				newParent.Add (this);
+				parent.Add (requestor);
 			}
-			
-			if (parent != null)
-				parent.Add (newParent);
-			
-			if (Visible)
-				newParent.Show ();
-			
+
 			if (position != DockPlacement.Center && data != null && data is System.Int32) {
 				if (newParent is DockPaned)
 					((DockPaned) newParent).Position = (int) data;
 			}
 			
-			DockObjectFlags &= ~(DockObjectFlags.InReflow);
-
-			newParent.Thaw ();
+			if (requestor.Parent is Notebook) {
+				// Activate the page we just added
+				Notebook notb = (Notebook) requestor.Parent;
+				notb.Page = notb.PageNum (requestor);
+			}
+			
 			if (parent != null)
 				parent.Thaw ();
 		}
@@ -866,6 +885,8 @@ namespace Gdl
 			} else {
 				target.Dock (this, position, null);
 			}
+			if (DockItemShown != null)
+				DockItemShown (this, EventArgs.Empty);
 		}
 		
 		public virtual void SetOrientation (Orientation orientation)
@@ -907,8 +928,11 @@ namespace Gdl
 			
 			/* if the object is manual, create a new placeholder to be
 			   able to restore the position later */
-			if (!IsAutomatic)
+			if (!IsAutomatic) {
+				if (dockPlaceHolder != null)
+					dockPlaceHolder.Dispose ();
 				dockPlaceHolder = new DockPlaceholder (this, false);
+			}
 			
 			Freeze ();
 
@@ -918,6 +942,9 @@ namespace Gdl
 			
 			Detach (true);
 			Thaw ();
+			
+			if (DockItemHidden != null)
+				DockItemHidden (this, EventArgs.Empty);
 		}
 		
 		private void HideChildItem (Widget widget)
@@ -941,16 +968,21 @@ namespace Gdl
 			
 			if (dockPlaceHolder != null) {
 				dockPlaceHolder.Add (this);
+				dockPlaceHolder.Dispose ();
 				dockPlaceHolder = null;
 			} else if (IsBound) {
 				if (Master.Controller != null) {
 					Master.Controller.Dock (this, DockPlacement.Floating, null);
 				}
 			}
+			if (DockItemShown != null)
+				DockItemShown (this, EventArgs.Empty);
 		}
 		
 		public virtual void SetDefaultPosition (DockObject reference)
 		{
+			if (dockPlaceHolder != null)
+				dockPlaceHolder.Dispose ();
 			dockPlaceHolder = null;
 			
 			if (reference != null && reference.IsAttached) {
