@@ -7,13 +7,16 @@
 
 using System;
 using System.IO;
+using System.Threading;
 using System.Diagnostics;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
+using MonoDevelop.Core.Gui.ProgressMonitoring;
 using MonoDevelop.Projects;
 using MonoDevelop.Components;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Components.Commands;
+using MonoDevelop.Projects.Deployment;
 
 namespace MonoDevelop.Ide.Commands
 {
@@ -49,7 +52,9 @@ namespace MonoDevelop.Ide.Commands
 		Stop,
 		Clean,
 		CleanSolution,
-		LocalCopyReference
+		LocalCopyReference,
+		DeployTargetList,
+		ConfigureDeployTargets
 	}
 	
 	internal class RunHandler: CommandHandler
@@ -359,16 +364,95 @@ namespace MonoDevelop.Ide.Commands
 		}
 	}
 	
-	internal class GenerateMakefilesHandler: CommandHandler
+	internal class DeployHandler: CommandHandler
 	{
 		protected override void Run ()
 		{
-			if (IdeApp.ProjectOperations.CurrentOpenCombine != null) {
-				IdeApp.ProjectOperations.CurrentOpenCombine.GenerateMakefiles ();
+			CombineEntry e = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			IAsyncOperation op = IdeApp.ProjectOperations.Build (e);
+			op.Completed += delegate {
+				if (op.Success)
+					IdeApp.ProjectOperations.Deploy (e, e.DefaultDeployTarget);
+			};
+		}
+		
+		protected override void Update (CommandInfo info)
+		{
+			CombineEntry e = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			if (e != null && e.DefaultDeployTarget != null) {
+				info.Enabled = true;
+			} else
+				info.Enabled = false;
+		}
+	}
+	
+	internal class DeployTargetListHandler: CommandHandler
+	{
+		protected override void Run (object dt)
+		{
+			CombineEntry ce = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			IAsyncOperation op = IdeApp.ProjectOperations.Build (ce);
+			op.Completed += delegate {
+				if (op.Success)
+					IdeApp.ProjectOperations.Deploy (ce, (DeployTarget)dt);
+			};
+		}
+		
+		protected override void Update (CommandArrayInfo info)
+		{
+			CombineEntry ce = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			if (ce != null) {
+				if (ce.DeployTargets.Count > 0) {
+					foreach (DeployTarget dt in ce.DeployTargets) {
+						if (dt is UnknownDeployTarget)
+							continue;
+						CommandInfo cinfo = new CommandInfo ();
+						cinfo.Text = dt.Name;
+						if (ce.DefaultDeployTarget == dt)
+							cinfo.Text += " " + GettextCatalog.GetString ("(default)");
+						cinfo.Icon = dt.DeployHandler.Icon;
+						info.Add (cinfo, dt);
+					}
+				} else {
+					CommandInfo cinfo = new CommandInfo ();
+					cinfo.Text = GettextCatalog.GetString ("No deploy targets available");
+					cinfo.Enabled = false;
+					info.Add (cinfo, null);
+				}
 			}
 		}
 	}
-
+	
+	internal class ConfigureDeployTargetsHandler: CommandHandler
+	{
+		protected override void Run ()
+		{
+			CombineEntry ce = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			if (ce != null)
+				IdeApp.ProjectOperations.ShowOptions (ce, "Deployment");
+		}
+		
+		protected override void Update (CommandInfo info)
+		{
+			info.Enabled = IdeApp.ProjectOperations.CurrentSelectedCombineEntry != null;
+		}
+	}
+	
+	internal class CombineEntryOptionsHandler: CommandHandler
+	{
+		protected override void Run ()
+		{
+			CombineEntry ce = IdeApp.ProjectOperations.CurrentSelectedCombineEntry;
+			if (ce != null)
+				IdeApp.ProjectOperations.ShowOptions (ce);
+		}
+		
+		protected override void Update (CommandInfo info)
+		{
+			info.Enabled = IdeApp.ProjectOperations.CurrentSelectedCombineEntry != null;
+		}
+	}
+	
 	internal class GenerateProjectDocumentation : CommandHandler
 	{
 		protected override void Run ()
