@@ -55,7 +55,9 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			"Paco Martinez",
 			"Lluis Sanchez Gual",
 			"Christian Hergert",
-			"Jacob Ilsø Christensen"
+			"Jacob Ilsø Christensen",
+			"Scott Ellington",
+			"Michael Hutchinson"
 		};
 		
 		public ScrollBox ()
