@@ -33,14 +33,13 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		ArrayList alltemplates = new ArrayList ();
 		ArrayList categories   = new ArrayList ();
 		Hashtable icons        = new Hashtable ();
+		ArrayList projectLangs = new ArrayList ();
 
 		PixbufList cat_imglist;
 
 		TreeStore catStore;
-		TreeStore templateStore;
 		Gtk.TreeView catView;
-		Gtk.TreeView templateView;
-		IconView TemplateView;
+		IconView iconView;
 		Button okButton;
 		Button cancelButton;
 		Label infoLabel;
@@ -50,7 +49,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		string basePath;
 		
 		string currentProjectType = string.Empty;
-		string currentLanguage = string.Empty;
 
 		public NewFileDialog (Project parentProject, string basePath) : base ()
 		{
@@ -61,17 +59,26 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			this.BorderWidth = 6;
 			this.HasSeparator = false;
 			
-			if (parentProject != null) {
-				currentProjectType = parentProject.ProjectType;
-				DotNetProject netproject = parentProject as DotNetProject;
-				if (netproject != null)
-					currentLanguage = netproject.LanguageName;
+			//if there's a parent project, check whether it wants to filter languages, else use defaults
+			if ((parentProject != null) && (parentProject.SupportedLanguages != null) && (parentProject.SupportedLanguages.Length > 0)) {
+				projectLangs.AddRange (parentProject.SupportedLanguages);
+			} else {
+				projectLangs.Add ("");	//match all non-filtered templates
+				projectLangs.Add ("*");	//match all .NET langs with CodeDom
 			}
+			ExpandLanguageWildcards (projectLangs);
 			
 			InitializeTemplates ();
+			InitializeComponents ();
 			nameEntry.GrabFocus ();
 		}
 		
+		public override void Dispose ()
+		{
+			Destroy ();
+			base.Dispose ();
+		}
+		
 		void InitializeView()
 		{
 			PixbufList smalllist  = new PixbufList();
@@ -133,7 +140,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				foreach (TemplateItem item in (ArrayList)(catStore.GetValue (iter, 2))) {
 					if (item.Template.Id == id) {
 						catView.Selection.SelectIter (iter);
-						TemplateView.CurrentlySelected = item;
+						iconView.CurrentlySelected = item;
 						return true;
 					}
 				}
@@ -178,26 +185,50 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 				// Ignore templates not supported by this project
 				if (template.ProjectType != "" && template.ProjectType != currentProjectType)
 					continue;
-					
-				// Ignore templates not supported by the current language
-				if (template.LanguageName != "" && template.LanguageName != "*" && currentLanguage != "" && template.LanguageName != currentLanguage)
-					continue;
-					
-				if (template.LanguageName == "*") {
-					ILanguageBinding[] langs = MonoDevelop.Projects.Services.Languages.GetLanguageBindings ();
-					foreach (ILanguageBinding lang in langs) {
-						IDotNetLanguageBinding dnlang = lang as IDotNetLanguageBinding; 
-						if (dnlang != null && dnlang.GetCodeDomProvider () != null) {
-							TemplateItem titem = new TemplateItem (template, dnlang.Language);
-							AddTemplate (titem, dnlang.Language);
-						}
-					}
-				} else {
-					TemplateItem titem = new TemplateItem (template, template.LanguageName);
-					AddTemplate (titem, template.LanguageName);
+				
+				//Find the languages that the template supports
+				ArrayList templateLangs = new ArrayList ();
+				foreach (string s in template.LanguageName.Split (','))
+					templateLangs.Add (s.Trim ());
+				ExpandLanguageWildcards (templateLangs);
+				
+				//Find all matches between the language strings of project and template
+				ArrayList langMatches = new ArrayList ();
+				
+				foreach (string templLang in templateLangs)
+					foreach (string projLang in projectLangs)
+						if (templLang == projLang)
+							langMatches.Add (projLang);
+				
+				//Eliminate duplicates
+				int pos = 0;
+				while (pos < langMatches.Count) {
+					int next = langMatches.IndexOf (langMatches [pos], pos +1);
+					if (next != -1)
+						langMatches.RemoveAt (next);
+					else
+						pos++;
+				}
+				
+				//Add all the possible templates
+				foreach (string match in langMatches) {
+					AddTemplate (new TemplateItem (template, match), match);	
+				}
+			}
+		}
+		
+		void ExpandLanguageWildcards (ArrayList list)
+		{
+			//Template can match all CodeDom .NET languages with a "*"
+			if (list.Contains ("*")) {
+				ILanguageBinding [] bindings = MonoDevelop.Projects.Services.Languages.GetLanguageBindings ();
+				foreach (ILanguageBinding lb in bindings) {
+					IDotNetLanguageBinding dnlang = lb as IDotNetLanguageBinding;
+					if (dnlang != null && dnlang.GetCodeDomProvider () != null)
+						list.Add (dnlang.Language);
+				list.Remove ("*");
 				}
 			}
-			InitializeComponents ();
 		}
 		
 		void AddTemplate (TemplateItem titem, string templateLanguage)
@@ -205,17 +236,11 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			Category cat = null;
 			
 			if (parentProject != null) {
-				if (templateLanguage != "") {
-					if (currentLanguage != "") {
-						// The template requires a language, and there is a language set, so only show it if they match 
-						if (currentLanguage != templateLanguage)
-							return;
-						cat = GetCategory (titem.Template.Category);
-					} else {
-						// The template requires a language, but there is no current language set, so create a category for it 
-						cat = GetCategory (templateLanguage);
-						cat = GetCategory (cat.Categories, titem.Template.Category);
-					}
+				if ((templateLanguage != "") && (projectLangs.Count != 2) ) {
+					// The template requires a language, but the project does not have a single fixed
+					// language type (plus empty match), so create a language category
+					cat = GetCategory (templateLanguage);
+					cat = GetCategory (cat.Categories, titem.Template.Category);
 				} else {
 					cat = GetCategory (titem.Template.Category);
 				}
@@ -268,9 +293,9 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		
 		void FillCategoryTemplates (TreeIter iter)
 		{
-			TemplateView.Clear ();
+			iconView.Clear ();
 			foreach (TemplateItem item in (ArrayList)(catStore.GetValue (iter, 2))) {
-				TemplateView.AddIcon (new Gtk.Image (Services.Resources.GetBitmap (item.Template.Icon, Gtk.IconSize.Dnd)), item.Name, item);
+				iconView.AddIcon (new Gtk.Image (Services.Resources.GetBitmap (item.Template.Icon, Gtk.IconSize.Dnd)), item.Name, item);
 			}
 		}
 		
@@ -287,17 +312,22 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 		
 		void UpdateOkStatus ()
 		{
-			TemplateItem sel = (TemplateItem) TemplateView.CurrentlySelected;
-			if (sel == null)
-				return;
-			
-			FileTemplate item = sel.Template;
+			try {
+				TemplateItem sel = (TemplateItem) iconView.CurrentlySelected;
+				if (sel == null)
+					return;
+				
+				FileTemplate item = sel.Template;
+				
 
-			if (item != null) {
-				infoLabel.Text = item.Description;
-				okButton.Sensitive = nameEntry.Text.Length > 0;
-			} else {
-				okButton.Sensitive = false;
+				if (item != null) {
+					infoLabel.Text = item.Description;
+					okButton.Sensitive = item.IsValidName (nameEntry.Text, sel.Language);
+				} else {
+					okButton.Sensitive = false;
+				}
+			} catch (Exception ex) {
+				Runtime.LoggingService.Error (ex);
 			}
 		}
 		
@@ -317,9 +347,8 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			//propertyService.SetProperty("Dialogs.NewProjectDialog.LargeImages", ((RadioButton)ControlDictionary["largeIconsRadioButton"]).Checked);
 			//propertyService.SetProperty("Dialogs.NewFileDialog.LastSelectedCategory", ((TreeView)ControlDictionary["categoryTreeView"]).SelectedNode.Text);
 			
-			//if (templateView.Selection.GetSelected (out mdl, out iter)) {
-			if (TemplateView.CurrentlySelected != null && nameEntry.Text.Length > 0) {
-				TemplateItem titem = (TemplateItem) TemplateView.CurrentlySelected;
+			if (iconView.CurrentlySelected != null && nameEntry.Text.Length > 0) {
+				TemplateItem titem = (TemplateItem) iconView.CurrentlySelected;
 				FileTemplate item = titem.Template;
 				
 				try {
@@ -409,12 +438,9 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 
 		void InitializeComponents()
 		{
-			
 			catStore = new Gtk.TreeStore (typeof(string), typeof(ArrayList), typeof(ArrayList), typeof(Gdk.Pixbuf));
 			catStore.SetSortColumnId (0, SortType.Ascending);
 			
-			templateStore = new Gtk.TreeStore (typeof(string), typeof(FileTemplate));
-
 			ScrolledWindow swindow1 = new ScrolledWindow();
 			swindow1.VscrollbarPolicy = PolicyType.Automatic;
 			swindow1.HscrollbarPolicy = PolicyType.Automatic;
@@ -422,8 +448,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			catView = new Gtk.TreeView (catStore);
 			catView.WidthRequest = 160;
 			catView.HeadersVisible = false;
-			templateView = new Gtk.TreeView (templateStore);
-			TemplateView = new IconView();
+			iconView = new IconView();
 
 			TreeViewColumn catColumn = new TreeViewColumn ();
 			catColumn.Title = "categories";
@@ -434,13 +459,6 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 
 			catView.AppendColumn (catColumn);
 
-			TreeViewColumn templateColumn = new TreeViewColumn ();
-			templateColumn.Title = "template";
-			CellRendererText tmpl_text_render = new CellRendererText ();
-			templateColumn.PackStart (tmpl_text_render, true);
-			templateColumn.AddAttribute (tmpl_text_render, "text", 0);
-			templateView.AppendColumn (templateColumn);
-
 			okButton = new Button (Gtk.Stock.New);
 			okButton.Clicked += new EventHandler (OpenEvent);
 
@@ -454,7 +472,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			HBox viewbox = new HBox (false, 6);
 			swindow1.Add(catView);
 			viewbox.PackStart (swindow1,false,true,0);
-			viewbox.PackStart(TemplateView, true, true,0);
+			viewbox.PackStart(iconView, true, true,0);
 
 			this.AddActionWidget (cancelButton, (int)Gtk.ResponseType.Cancel);
 			this.AddActionWidget (okButton, (int)Gtk.ResponseType.Ok);
@@ -474,8 +492,8 @@ namespace MonoDevelop.Ide.Gui.Dialogs
 			cat_imglist.Add(Services.Resources.GetBitmap("md-closed-folder"));
 			catView.Selection.Changed += new EventHandler (CategoryChange);
 			catView.RowActivated += new RowActivatedHandler (CategoryActivated);
-			TemplateView.IconSelected += new EventHandler(SelectedIndexChange);
-			TemplateView.IconDoubleClicked += new EventHandler(OpenEvent);
+			iconView.IconSelected += new EventHandler(SelectedIndexChange);
+			iconView.IconDoubleClicked += new EventHandler(OpenEvent);
 			InitializeView ();
 		}
 	}
