@@ -209,7 +209,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 			StringBuilder sb = new StringBuilder ();
 			for (int n=0; n<name.Length; n++) {
 				char c = name [n];
-				if (Array.IndexOf (Path.InvalidPathChars, c) != -1)
+				if (Array.IndexOf (Path.GetInvalidPathChars(), c) != -1)
 					continue;
 				if (c == Path.DirectorySeparatorChar || c == Path.AltDirectorySeparatorChar || c == Path.VolumeSeparatorChar)
 					continue;
@@ -241,7 +241,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 			}
 		}
 		
-		void SolutionCheckChanged (object sender, EventArgs e)
+		protected void SolutionCheckChanged (object sender, EventArgs e)
 		{
 			if (CreateSolutionDirectory && txt_subdirectory.Text == "")
 				txt_subdirectory.Text = txt_name.Text;
@@ -249,7 +249,7 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 			PathChanged (null, null);
 		}
 		
-		void NameChanged (object sender, EventArgs e)
+		protected void NameChanged (object sender, EventArgs e)
 		{
 			if (CreateSolutionDirectory && txt_subdirectory.Text == lastName)
 				txt_subdirectory.Text = txt_name.Text;
@@ -332,12 +332,12 @@ namespace MonoDevelop.Ide.Gui.Dialogs {
 				{
 					System.IO.Directory.CreateDirectory (ProjectLocation);
 				}
-				catch (IOException ioException)
+				catch (IOException)
 				{
 					Services.MessageService.ShowError (dialog, String.Format (GettextCatalog.GetString ("Could not create directory {0}. File already exists."), ProjectLocation));
 					return;
 				}
-				catch (UnauthorizedAccessException accessException)
+				catch (UnauthorizedAccessException)
 				{
 					Services.MessageService.ShowError (dialog, String.Format (GettextCatalog.GetString ("You do not have permission to create to {0}"), ProjectLocation));
 					return;
