@@ -36,6 +36,7 @@ using MonoDevelop.Ide.Commands;
 using MonoDevelop.Ide.Gui;
 using MonoDevelop.Components.Commands;
 using MonoDevelop.Core.Gui;
+using MonoDevelop.Ide.Codons;
 
 namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 {
@@ -174,23 +175,41 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		[CommandUpdateHandler (ProjectCommands.IncludeInDeploy)]
 		public void OnUpdateIncludeInDeploy (CommandInfo info)
 		{
-			Project project = (Project) CurrentNode.GetParentDataItem (typeof(Project), false);
 			ProjectFile finfo = CurrentNode.DataItem as ProjectFile;
-			info.Checked = !project.DeployInformation.IsFileExcluded (finfo.Name);
+			info.Checked = finfo.BuildAction == BuildAction.FileCopy;
 		}
 		
 		[CommandHandler (ProjectCommands.IncludeInDeploy)]
 		public void OnIncludeInDeploy ()
 		{
 			ProjectFile finfo = CurrentNode.DataItem as ProjectFile;
-			Project project = (Project) CurrentNode.GetParentDataItem (typeof(Project), false);
 
-			if (project.DeployInformation.IsFileExcluded (finfo.Name)) {
-				project.DeployInformation.RemoveExcludedFile (finfo.Name);
+			if (finfo.BuildAction == BuildAction.FileCopy) {
+				finfo.BuildAction = BuildAction.Nothing;
 			} else {
-				project.DeployInformation.AddExcludedFile (finfo.Name);
+				finfo.BuildAction = BuildAction.FileCopy;
 			}
 			IdeApp.ProjectOperations.SaveProject (finfo.Project);
 		}
+		
+		[CommandHandler (ViewCommands.OpenWithList)]
+		public void OnOpenWith (object ob)
+		{
+			ProjectFile finfo = CurrentNode.DataItem as ProjectFile;
+			((FileViewer)ob).OpenFile (finfo.Name);
+		}
+		
+		[CommandUpdateHandler (ViewCommands.OpenWithList)]
+		public void OnOpenWithUpdate (CommandArrayInfo info)
+		{
+			ProjectFile finfo = CurrentNode.DataItem as ProjectFile;
+			FileViewer prev = null; 
+			foreach (FileViewer fv in IdeApp.Workbench.GetFileViewers (finfo.Name)) {
+				if (prev != null && fv.IsExternal != prev.IsExternal)
+					info.AddSeparator ();
+				info.Add (fv.Title, fv);
+				prev = fv;
+			}
+		}
 	}
 }
