@@ -113,8 +113,19 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 			base.BuildNode (treeBuilder, dataObject, ref label, ref icon, ref closedIcon);
 
 			Project p = dataObject as Project;
-			label = p.Name;
-			string iconName = Services.Icons.GetImageForProjectType (p.ProjectType);
+			
+			string iconName;
+			if (p is DotNetProject && ((DotNetProject)p).LanguageBinding == null) {
+				iconName = Gtk.Stock.DialogError;
+				label = GettextCatalog.GetString ("{0} <span foreground='red' size='small'>(Unknown language '{1}')</span>", p.Name, ((DotNetProject)p).LanguageName);
+			} else {
+				iconName = Services.Icons.GetImageForProjectType (p.ProjectType);
+				if (p.ParentCombine != null && p.ParentCombine.StartupEntry == p)
+					label = "<b>" + p.Name + "</b>";
+				else
+					label = p.Name;
+			}
+			
 			icon = Context.GetIcon (iconName);
 		}
 
@@ -273,20 +284,6 @@ namespace MonoDevelop.Ide.Gui.Pads.ProjectPad
 		{
 		}
 		
-		[CommandHandler (ProjectCommands.Options)]
-		public void OnProjectOptions ()
-		{
-			Project selectedProject = CurrentNode.DataItem as Project;
-			IdeApp.ProjectOperations.ShowOptions (selectedProject);
-		}
-		
-		[CommandHandler (ProjectCommands.Deploy)]
-		public void OnProjectDeploy ()
-		{
-			Project selectedProject = CurrentNode.DataItem as Project;
-			IdeApp.ProjectOperations.Deploy (selectedProject);
-		}
-		
 		[CommandHandler (ProjectCommands.SetAsStartupProject)]
 		public void SetAsStartupProject ()
 		{
