@@ -63,8 +63,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 		internal const int ClosedIconColumn = 2;
 		internal const int DataItemColumn = 3;
 		internal const int BuilderChainColumn = 4;
-		internal const int WeightColumn = 5;
-		internal const int FilledColumn = 6;
+		internal const int FilledColumn = 5;
 		
 		NodeBuilder[] builders;
 		Hashtable builderChains = new Hashtable ();
@@ -143,46 +142,20 @@ namespace MonoDevelop.Ide.Gui.Pads
 		
 		public virtual void Initialize (NodeBuilder[] builders, TreePadOption[] options)
 		{
-			// Create default options
-			
-			this.options = options;
-			globalOptions = new TreeOptions ();
-			foreach (TreePadOption op in options)
-				globalOptions [op.Id] = op.DefaultValue;
-				
-			globalOptions.Pad = this;
-			
-			// Check that there is only one node builder per type
-			
-			Hashtable bc = new Hashtable ();
-			foreach (NodeBuilder nb in builders) {
-				TypeNodeBuilder tnb = nb as TypeNodeBuilder;
-				if (tnb != null) {
-					TypeNodeBuilder other = (TypeNodeBuilder) bc [tnb.NodeDataType];
-					if (other != null)
-						throw new ApplicationException (string.Format ("The type node builder {0} can't be used in this context because the type {1} is already handled by {2}", nb.GetType(), tnb.NodeDataType, other.GetType()));
-					bc [tnb.NodeDataType] = tnb;
-				}
-				else if (!(nb is NodeBuilderExtension))
-					throw new InvalidOperationException (string.Format ("Invalid NodeBuilder type: {0}. NodeBuilders must inherit either from TypeNodeBuilder or NodeBuilderExtension", nb.GetType()));
-			}
-			
-			NodeBuilders = builders;
-
 			builderContext = new TreeBuilderContext (this);
-			
 			tree = new Gtk.TreeView ();
 			
+			SetBuilders (builders, options);
+			
 			/*
 			0 -- Text
 			1 -- Icon (Open)
 			2 -- Icon (Closed)
 			3 -- Node Data
 			4 -- Builder chain
-			5 -- Pango weight
-			6 -- Expanded
+			5 -- Expanded
 			*/
-			store = new Gtk.TreeStore (typeof (string), typeof (Gdk.Pixbuf), typeof (Gdk.Pixbuf), typeof (object), typeof (object), typeof(int), typeof(bool));
+			store = new Gtk.TreeStore (typeof (string), typeof (Gdk.Pixbuf), typeof (Gdk.Pixbuf), typeof (object), typeof (object), typeof(bool));
 			tree.Model = store;
 
 			tree.EnableModelDragDest (target_table, Gdk.DragAction.Copy | Gdk.DragAction.Move);
@@ -208,8 +181,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			text_render.EditingCanceled += new EventHandler (HandleOnEditCancelled);
 			
 			complete_column.PackStart (text_render, true);
-			complete_column.AddAttribute (text_render, "text", TextColumn);
-			complete_column.AddAttribute (text_render, "weight", WeightColumn);
+			complete_column.AddAttribute (text_render, "markup", TextColumn);
 	
 			tree.AppendColumn (complete_column);
 			
@@ -224,9 +196,6 @@ namespace MonoDevelop.Ide.Gui.Pads
 			contentPanel.ButtonReleaseEvent += new Gtk.ButtonReleaseEventHandler(OnButtonRelease);
 			contentPanel.PopupMenu += new Gtk.PopupMenuHandler (OnPopupMenu);
 			
-			foreach (NodeBuilder nb in builders)
-				nb.SetContext (builderContext);
-			
 			workNode = new TreeNodeNavigator (this);
 			compareNode1 = new TreeNodeNavigator (this);
 			compareNode2 = new TreeNodeNavigator (this);
@@ -241,6 +210,62 @@ namespace MonoDevelop.Ide.Gui.Pads
 			
 			contentPanel.ShowAll ();
 		}
+		
+		public void UpdateBuilders (NodeBuilder[] builders, TreePadOption[] options)
+		{
+			// Save the current state
+			ITreeNavigator root = GetRootNode ();
+			NodeState state = root != null ? root.SaveState () : null;
+			object obj = root != null ? root.DataItem : null;
+			
+			Clear ();
+			
+			// Clean cached builder chains
+			builderChains.Clear ();
+			
+			// Update the builders
+			SetBuilders (builders, options);
+
+			// Restore the tree
+			if (obj != null)
+				LoadTree (obj);
+			
+			root = GetRootNode ();
+			if (root != null && state != null)
+				root.RestoreState (state);
+		}
+		
+		void SetBuilders (NodeBuilder[] builders, TreePadOption[] options)
+		{
+			// Create default options
+			
+			this.options = options;
+			globalOptions = new TreeOptions ();
+			foreach (TreePadOption op in options)
+				globalOptions [op.Id] = op.DefaultValue;
+				
+			globalOptions.Pad = this;
+			
+			// Check that there is only one TypeNodeBuilder per type
+			
+			Hashtable bc = new Hashtable ();
+			foreach (NodeBuilder nb in builders) {
+				TypeNodeBuilder tnb = nb as TypeNodeBuilder;
+				if (tnb != null) {
+					TypeNodeBuilder other = (TypeNodeBuilder) bc [tnb.NodeDataType];
+					if (other != null)
+						throw new ApplicationException (string.Format ("The type node builder {0} can't be used in this context because the type {1} is already handled by {2}", nb.GetType(), tnb.NodeDataType, other.GetType()));
+					bc [tnb.NodeDataType] = tnb;
+				}
+				else if (!(nb is NodeBuilderExtension))
+					throw new InvalidOperationException (string.Format ("Invalid NodeBuilder type: {0}. NodeBuilders must inherit either from TypeNodeBuilder or NodeBuilderExtension", nb.GetType()));
+			}
+			
+			NodeBuilders = builders;
+			
+			foreach (NodeBuilder nb in builders)
+				nb.SetContext (builderContext);
+		}
 
 		void OnDragBegin (object o, Gtk.DragBeginArgs arg)
 		{
@@ -597,13 +622,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 			}
 		}
 
-		/// <summary>
-		/// If you want to edit a node label. Select the node you want to edit and then
-		/// call this method, instead of using the LabelEdit Property and the BeginEdit
-		/// Method directly.
-		/// </summary>
-		[CommandHandler (EditCommands.Rename)]
-		public void StartLabelEdit()
+		public void StartLabelEditInternal()
 		{
 			if (editingText)
 				return;
@@ -696,8 +715,10 @@ namespace MonoDevelop.Ide.Gui.Pads
 		{
 			ITreeNavigator nav = GetRootNode ();
 			XmlElement rootNode = parent ["Node"];
-			NodeState state = NodeState.FromXml (rootNode);
-			nav.RestoreState (state);
+			if (rootNode != null) {
+				NodeState state = NodeState.FromXml (rootNode);
+				nav.RestoreState (state);
+			}
 		}
 		
 		TypeNodeBuilder GetTypeNodeBuilder (Type type)
@@ -962,14 +983,21 @@ namespace MonoDevelop.Ide.Gui.Pads
 			return false;
 		}
 
-		public void StealFocus ()
+		/// <summary>
+		/// If you want to edit a node label. Select the node you want to edit and then
+		/// call this method, instead of using the LabelEdit Property and the BeginEdit
+		/// Method directly.
+		/// </summary>
+		[CommandHandler (EditCommands.Rename)]
+		public void StartLabelEdit ()
 		{
 			GLib.Timeout.Add (20, new GLib.TimeoutHandler (wantFocus));
 		}
+		
 		bool wantFocus ()
 		{
 			tree.GrabFocus ();
-			StartLabelEdit ();
+			StartLabelEditInternal ();
 			return false;
 		}
 
@@ -993,7 +1021,7 @@ namespace MonoDevelop.Ide.Gui.Pads
 					opset.AddItem (ViewCommands.TreeDisplayOptionList);
 					opset.AddItem (Command.Separator);
 					opset.AddItem (ViewCommands.ResetTreeDisplayOptions);
-					IdeApp.CommandService.ShowContextMenu (opset);
+					IdeApp.CommandService.ShowContextMenu (opset, this);
 				}
 			} else {
 				CommandEntrySet eset = IdeApp.CommandService.CreateCommandEntrySet (nb.ContextMenuAddinPath);
@@ -1002,7 +1030,8 @@ namespace MonoDevelop.Ide.Gui.Pads
 				opset.AddItem (ViewCommands.TreeDisplayOptionList);
 				opset.AddItem (Command.Separator);
 				opset.AddItem (ViewCommands.ResetTreeDisplayOptions);
-				IdeApp.CommandService.ShowContextMenu (eset);
+				opset.AddItem (ViewCommands.RefreshTree);
+				IdeApp.CommandService.ShowContextMenu (eset, this);
 			}
 		}
 		
@@ -1043,6 +1072,17 @@ namespace MonoDevelop.Ide.Gui.Pads
 			tb.UpdateAll ();
 		}
 
+		[CommandHandler (ViewCommands.RefreshTree)]
+		protected void RefreshTree ()
+		{
+			Gtk.TreeModel foo;
+			Gtk.TreeIter iter;
+			if (!tree.Selection.GetSelected (out foo, out iter))
+				return;
+			TreeBuilder tb = new TreeBuilder (this, iter);
+			tb.UpdateAll ();
+		}
+		
 		void OnPopupMenu (object o, Gtk.PopupMenuArgs args)
 		{
 			if (GetSelectedNode () != null)
