@@ -76,6 +76,12 @@ namespace MonoDevelop.Ide.Gui
 				isViewOnly = value;
 			}
 		}
+
+		public virtual string StockIconId {
+			get {
+				return null;
+			}
+		}
 		
 		public virtual void Save()
 		{
@@ -102,6 +108,11 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
+		public override bool CanReuseView (string fileName)
+		{
+			return (ContentName == fileName);
+		}
+		
 		public string PathRelativeToProject
 		{
 			get
