@@ -9,6 +9,7 @@ using System;
 using System.Xml;
 using System.IO;
 using System.Collections;
+using System.Collections.Generic;
 using System.Reflection;
 using System.CodeDom.Compiler;
 
@@ -28,7 +29,7 @@ namespace MonoDevelop.Ide.Gui
 	public class DisplayBindingService : AbstractService
 	{
 		readonly static string displayBindingPath = "/SharpDevelop/Workbench/DisplayBindings";
-		DisplayBindingCodon[] bindings = null;
+		List<DisplayBindingCodon> bindings = null;
 
 		public IDisplayBinding LastBinding {
 			get {
@@ -52,6 +53,17 @@ namespace MonoDevelop.Ide.Gui
 			return null;
 		}
 		
+		public IDisplayBinding[] GetBindingsForMimeType (string mimeType)
+		{
+			ArrayList list = new ArrayList ();
+			foreach (DisplayBindingCodon binding in bindings) {
+				if (binding.DisplayBinding != null && binding.DisplayBinding.CanCreateContentForMimeType (mimeType)) {
+					list.Add (binding.DisplayBinding);
+				}
+			}
+			return (IDisplayBinding[]) list.ToArray (typeof(IDisplayBinding));
+		}
+		
 		internal DisplayBindingCodon GetCodonPerFileName(string filename)
 		{
 			string vfsname = filename;
@@ -86,7 +98,14 @@ namespace MonoDevelop.Ide.Gui
 		
 		public override void InitializeService ()
 		{
-			bindings = (DisplayBindingCodon[]) Runtime.AddInService.GetTreeItems (displayBindingPath, typeof(DisplayBindingCodon));
+			bindings = new List<DisplayBindingCodon> ();
+			Runtime.AddInService.RegisterExtensionItemListener (displayBindingPath, OnExtensionChanged);
+		}
+		
+		void OnExtensionChanged (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add)
+				bindings.Add ((DisplayBindingCodon)item);
 		}
 	}
 }
