@@ -36,8 +36,10 @@ using Gtk;
 using MonoDevelop.Core;
 using MonoDevelop.Components;
 using MonoDevelop.Projects;
+using MonoDevelop.Projects.Text;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Ide.Gui.Dialogs;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -56,7 +58,8 @@ namespace MonoDevelop.Ide.Gui
 		internal Document (IWorkbenchWindow window)
 		{
 			this.window = window;
-			window.Closed += new EventHandler (OnClosed);
+			window.Closed += OnClosed;
+			window.ActiveViewContentChanged += OnActiveViewContentChanged;
 		}
 		
 		public string FileName {
@@ -86,6 +89,10 @@ namespace MonoDevelop.Ide.Gui
 			window.SelectWindow ();
 		}
 		
+		public object ActiveView {
+			get { return window.ActiveViewContent; }
+		}
+		
 /*		public void JumpTo (int line, int column)
 		{
 			IViewContent content = Window.ViewContent;
@@ -96,7 +103,7 @@ namespace MonoDevelop.Ide.Gui
 */
 		public virtual void Save ()
 		{
-			if (!Window.ViewContent.IsDirty)
+			if (Window.ViewContent.IsViewOnly || !Window.ViewContent.IsDirty)
 				return;
 
 			if (Window.ViewContent.ContentName == null) {
@@ -127,17 +134,32 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void SaveAs (string filename)
 		{
+			if (Window.ViewContent.IsViewOnly)
+				return;
+
 			if (Window.ViewContent is ICustomizedCommands) {
 				if (((ICustomizedCommands)window.ViewContent).SaveAsCommand()) {
 					return;
 				}
 			}
 			
+			string encoding = null;
+			
+			IEncodedTextContent tbuffer = Window.ViewContent as IEncodedTextContent;
+			if (tbuffer != null) {
+				encoding = tbuffer.SourceEncoding;
+				if (encoding == null)
+					encoding = TextEncoding.DefaultEncoding;
+			}
+				
 			if (filename == null) {
-				FileSelector fdiag = new FileSelector (GettextCatalog.GetString ("Save as..."), Gtk.FileChooserAction.Save);
+				FileSelectorDialog fdiag = new FileSelectorDialog (GettextCatalog.GetString ("Save as..."), Gtk.FileChooserAction.Save);
 				fdiag.CurrentName = Window.ViewContent.UntitledName;
+				fdiag.Encoding = encoding;
+				fdiag.ShowEncodingSelector = (tbuffer != null);
 				int response = fdiag.Run ();
 				filename = fdiag.Filename;
+				encoding = fdiag.Encoding;
 				fdiag.Hide ();
 				if (response != (int)Gtk.ResponseType.Ok)
 					return;
@@ -153,13 +175,21 @@ namespace MonoDevelop.Ide.Gui
 					return;
 				}
 			}
+			
 			// save backup first
 			if((bool) Runtime.Properties.GetProperty ("SharpDevelop.CreateBackupCopy", false)) {
-				Window.ViewContent.Save (filename + "~");
+				if (tbuffer != null && encoding != null)
+					tbuffer.Save (filename + "~", encoding);
+				else
+					Window.ViewContent.Save (filename + "~");
 			}
 			
 			// do actual save
-			Window.ViewContent.Save (filename);
+			if (tbuffer != null && encoding != null)
+				tbuffer.Save (filename, encoding);
+			else
+				Window.ViewContent.Save (filename);
+
 			IdeApp.Services.FileService.NotifyFileChange (filename);
 			IdeApp.Workbench.RecentOpen.AddLastFile (filename, null);
 			
@@ -214,17 +244,31 @@ namespace MonoDevelop.Ide.Gui
 		
 		void OnClosed (object s, EventArgs a)
 		{
+			window.Closed -= OnClosed;
+			window.ActiveViewContentChanged -= OnActiveViewContentChanged;
 			OnClosed (a);
 		}
 		
+		void OnActiveViewContentChanged (object s, EventArgs args)
+		{
+			OnViewChanged (args);
+		}
+		
 		protected virtual void OnClosed (EventArgs args)
 		{
 			if (Closed != null)
 				Closed (this, args);
 		}
 		
+		protected virtual void OnViewChanged (EventArgs args)
+		{
+			if (ViewChanged != null)
+				ViewChanged (this, args);
+		}
+		
 		public event EventHandler Closed;
 		public event EventHandler Saved;
+		public event EventHandler ViewChanged;
 	}
 }
 
