@@ -64,6 +64,10 @@ namespace MonoDevelop.Ide.Gui
 		bool IsViewOnly {
 			get;
 		}
+
+		string StockIconId {
+			get;
+		}
 		
 		/// <summary>
 		/// Saves this content to the last load/save location.
@@ -80,10 +84,6 @@ namespace MonoDevelop.Ide.Gui
 		/// </summary>
 		void Load(string fileName);
 		
-		string TabPageLabel {
-			get;
-		}
-		
 		/// <summary>
 		/// The name of the project the content is attached to
 		/// </summary>
