@@ -54,11 +54,11 @@ namespace MonoDevelop.Ide.Gui
 {
 	public abstract class IdeApp
 	{
-		static Workbench workbench = new Workbench ();
-		static ProjectOperations projectOperations = new ProjectOperations ();
-		static HelpOperations helpOperations = new HelpOperations ();
-		static CommandService commandService = new CommandService ();
-		static IdeServices ideServices = new IdeServices ();
+		static Workbench workbench;
+		static ProjectOperations projectOperations;
+		static HelpOperations helpOperations;
+		static CommandService commandService;
+		static IdeServices ideServices;
 		
 		IdeApp ()
 		{
@@ -86,6 +86,12 @@ namespace MonoDevelop.Ide.Gui
 		
 		public static void Initialize (IProgressMonitor monitor)
 		{
+			workbench = new Workbench ();
+			projectOperations = new ProjectOperations ();
+			helpOperations = new HelpOperations ();
+			commandService = new CommandService ();
+			ideServices = new IdeServices ();
+		
 			monitor.BeginTask (GettextCatalog.GetString("Loading Workbench"), 6);
 			
 			commandService.LoadCommands ("/SharpDevelop/Commands");
@@ -134,14 +140,19 @@ namespace MonoDevelop.Ide.Gui
 		
 			commandService.EnableUpdate ();
 			
-			foreach (CommandHandler handler in Runtime.AddInService.GetTreeItems ("/MonoDevelop/IDE/StartupHandlers", typeof(CommandHandler))) {
+			Runtime.AddInService.RegisterExtensionItemListener ("/MonoDevelop/IDE/StartupHandlers", OnExtensionChanged);
+			monitor.EndTask ();
+		}
+		
+		static void OnExtensionChanged (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add) {
 				try {
-					typeof(CommandHandler).GetMethod ("Run", System.Reflection.BindingFlags.NonPublic|System.Reflection.BindingFlags.Instance, null, Type.EmptyTypes, null).Invoke (handler, null);
+					typeof(CommandHandler).GetMethod ("Run", System.Reflection.BindingFlags.NonPublic|System.Reflection.BindingFlags.Instance, null, Type.EmptyTypes, null).Invoke (item, null);
 				} catch (Exception ex) {
 					Runtime.LoggingService.Error (ex);
 				}
 			}
-			monitor.EndTask ();
 		}
 			
 		public static void Run ()
