@@ -38,6 +38,7 @@ using MonoDevelop.Projects;
 using MonoDevelop.Projects.Gui.Dialogs;
 using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects.CodeGeneration;
+using MonoDevelop.Projects.Deployment;
 using MonoDevelop.Components;
 using MonoDevelop.Core;
 using MonoDevelop.Core.AddIns;
@@ -177,6 +178,30 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 		
+		public void JumpToDeclaration (ILanguageItem item)
+		{
+			if (item is IMember) {
+				IMember mem = (IMember) item;
+				if (mem.DeclaringType != null && mem.DeclaringType.Region != null && mem.Region != null)
+					IdeApp.Workbench.OpenDocument (mem.DeclaringType.Region.FileName, mem.Region.BeginLine, mem.Region.BeginColumn, true);
+			} else if (item is IClass) {
+				IClass cls = (IClass) item;
+				if (cls.Region != null)
+					IdeApp.Workbench.OpenDocument (cls.Region.FileName, cls.Region.BeginLine, cls.Region.BeginColumn, true);
+			} else if (item is LocalVariable) {
+				LocalVariable cls = (LocalVariable) item;
+				if (cls.Region != null)
+					IdeApp.Workbench.OpenDocument (cls.Region.FileName, cls.Region.BeginLine, cls.Region.BeginColumn, true);
+			}
+		}
+		
+		public bool CanJumpToDeclaration (ILanguageItem item)
+		{
+			return ((item is IMember && ((IMember)item).Region != null) || 
+			    (item is IClass && ((IClass)item).Region != null) || 
+			    (item is LocalVariable && ((LocalVariable)item).Region != null)); 
+		}
+		
 		public void SaveCombinePreferences ()
 		{
 			if (CurrentOpenCombine != null)
@@ -371,7 +396,7 @@ namespace MonoDevelop.Ide.Gui
 		public void MarkFileDirty (string filename)
 		{
 			if (openCombine != null) {
-				Project entry = openCombine.GetProjectEntryContaining (filename);
+				Project entry = openCombine.GetProjectContainingFile (filename);
 				if (entry != null) {
 					entry.NeedsBuilding = true;
 				}
@@ -390,16 +415,12 @@ namespace MonoDevelop.Ide.Gui
 				openCombine.RenameFileInProjects (e.SourceFile, e.TargetFile);
 		}
 		
-		public void Deploy (Project project)
-		{
-			foreach (Document doc in IdeApp.Workbench.Documents) {
-				if (doc.IsDirty)
-					doc.Save();
-			}
-			DeployInformation.Deploy (project);
-		}
-
 		public void ShowOptions (CombineEntry entry)
+		{
+			ShowOptions (entry, null);
+		}
+		
+		public void ShowOptions (CombineEntry entry, string panelId)
 		{
 			if (entry is Project) {
 				Project selectedProject = (Project) entry;
@@ -408,6 +429,9 @@ namespace MonoDevelop.Ide.Gui
 				IAddInTreeNode configurationPropertiesNode = Runtime.AddInService.GetTreeNode("/SharpDevelop/Workbench/ProjectOptions/ConfigurationProperties");
 				
 				ProjectOptionsDialog optionsDialog = new ProjectOptionsDialog (IdeApp.Workbench.RootWindow, selectedProject, generalOptionsNode, configurationPropertiesNode);
+				if (panelId != null)
+					optionsDialog.SelectPanel (panelId);
+					
 				if (optionsDialog.Run() == (int)Gtk.ResponseType.Ok) {
 					selectedProject.NeedsBuilding = true;
 				}
@@ -418,6 +442,8 @@ namespace MonoDevelop.Ide.Gui
 				IAddInTreeNode configurationPropertiesNode = Runtime.AddInService.GetTreeNode("/SharpDevelop/Workbench/CombineOptions/ConfigurationProperties");
 				
 				CombineOptionsDialog optionsDialog = new CombineOptionsDialog (IdeApp.Workbench.RootWindow, combine, generalOptionsNode, configurationPropertiesNode);
+				if (panelId != null)
+					optionsDialog.SelectPanel (panelId);
 				optionsDialog.Run ();
 			}
 			
@@ -465,6 +491,7 @@ namespace MonoDevelop.Ide.Gui
 			CombineEntry res = null;
 			
 			using (FileSelector fdiag = new FileSelector (GettextCatalog.GetString ("Add to Solution"))) {
+				fdiag.SetCurrentFolder (parentCombine.BaseDirectory);
 				fdiag.SelectMultiple = false;
 				if (fdiag.Run () == (int) Gtk.ResponseType.Ok) {
 					try {
@@ -645,6 +672,26 @@ namespace MonoDevelop.Ide.Gui
 				Runtime.LoggingService.Warn("Could not save solution preferences: " + fileToSave as object, e);
 			}
 		}
+		
+		public IAsyncOperation Deploy (CombineEntry entry, DeployTarget target)
+		{
+			IProgressMonitor mon = IdeApp.Workbench.ProgressMonitors.GetOutputProgressMonitor (GettextCatalog.GetString ("Deploy Output"), MonoDevelop.Core.Gui.Stock.RunProgramIcon, true, true);
+
+			// Run the deploy command in a background thread to avoid
+			// deadlocks with the gui thread
+			
+			System.Threading.Thread t = new System.Threading.Thread (
+				delegate () {
+					using (mon) {
+						target.Deploy (mon);
+					}
+				}
+			);
+			t.IsBackground = true;
+			t.Start ();
+
+			return mon.AsyncOperation;
+		}
 
 		public IAsyncOperation Execute (CombineEntry entry)
 		{
@@ -1175,7 +1222,7 @@ namespace MonoDevelop.Ide.Gui
 	{
 		public IProgressMonitor CreateProgressMonitor ()
 		{
-			return IdeApp.Workbench.ProgressMonitors.GetBackgroundProgressMonitor ("Code Completion Database Generation", "gtk-execute");
+			return IdeApp.Workbench.ProgressMonitors.GetBackgroundProgressMonitor ("Code Completion Database Generation", "md-parser");
 		}
 	}
 }
