@@ -54,10 +54,12 @@ namespace MonoDevelop.Ide.Gui
 		EventHandler contextChangedHandler;
 		Hashtable padWindows = new Hashtable ();
 		
-		WorkbenchContextCodon[] contextCodons;
 		bool initialized;
 		IWorkbenchWindow lastActive;
 		bool ignorePageSwitch;
+		
+		Gtk.Toolbar[] toolBars;
+		Gtk.MenuBar menubar;
 
 		public SdiWorkbenchLayout () {
 			contextChangedHandler = new EventHandler (OnContextChanged);
@@ -68,7 +70,7 @@ namespace MonoDevelop.Ide.Gui
 				if (tabControl == null || tabControl.CurrentPage < 0 || tabControl.CurrentPage >= tabControl.NPages)  {
 					return null;
 				}
-				return (IWorkbenchWindow)workbench.ViewContentCollection[tabControl.CurrentPage].WorkbenchWindow;
+				return (IWorkbenchWindow) tabControl.CurrentPageWidget;
 			}
 		}
 		
@@ -93,6 +95,9 @@ namespace MonoDevelop.Ide.Gui
 				}
 			}
 			
+			toolBars = workbench.ToolBars;
+			menubar = workbench.TopMenu;
+			
 			// Create the docking widget and add it to the window.
 			dock = new Dock ();
 			DockBar dockBar = new DockBar (dock);
@@ -217,6 +222,13 @@ namespace MonoDevelop.Ide.Gui
 				return currentLayout.Substring (currentLayout.IndexOf (".") + 1);
 			}
 			set {
+				// Store a list of pads being shown
+				ArrayList visible = new ArrayList ();
+				foreach (IPadContent content in activePadCollection) {
+					if (IsVisible (content))
+						visible.Add (content);
+				}
+				
 				// save previous layout first
 				if (currentLayout != "")
 					dockLayout.SaveLayout (currentLayout);
@@ -226,6 +238,15 @@ namespace MonoDevelop.Ide.Gui
 				if (layouts.Contains (value))
 				{
 					dockLayout.LoadLayout (newLayout);
+					
+					DockItem ot = dock.GetItemByName ("Documents");
+					if (!ot.IsAttached) {
+						// Something went wrong. The documents item should always be visible.
+						// The cause may be a corrupt configuration file.
+						// This will reset this layout:
+						dockLayout.LoadLayout (null);
+						dockLayout.SaveLayout (newLayout);
+					}
 				}
 				else
 				{
@@ -247,6 +268,21 @@ namespace MonoDevelop.Ide.Gui
 				// persist the selected layout for the current context
 				Runtime.Properties.SetProperty ("MonoDevelop.Core.Gui.SdiWorkbenchLayout." +
 				                             workbenchContext.Id, value);
+
+				// Notify hide/show events
+				foreach (IPadContent content in activePadCollection) {
+					if (IsVisible (content)) {
+						if (!visible.Contains (content)) {
+							PadWindow win = (PadWindow) padWindows [content];
+							win.NotifyShown ();
+						}
+					} else {
+						if (visible.Contains (content)) {
+							PadWindow win = (PadWindow) padWindows [content];
+							win.NotifyHidden ();
+						}
+					}
+				}
 			}
 		}
 
@@ -290,22 +326,7 @@ namespace MonoDevelop.Ide.Gui
 		
 		void CreateDefaultLayout()
 		{
-			contextCodons = (WorkbenchContextCodon[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/Contexts", typeof(WorkbenchContextCodon));
-			PadContentCollection collection;
-			
-			// Set the pads specific of each context
-			foreach (WorkbenchContextCodon codon in contextCodons)
-			{
-				collection = new PadContentCollection ();
-				WorkbenchContext ctx = WorkbenchContext.GetContext (codon.ID);
-				padCollections [ctx] = collection;
-
-				foreach (ContextPadCodon padCodon in codon.Pads) {
-					IPadContent pad = workbench.PadContentCollection [padCodon.ID];
-					if (pad != null)
-						collection.Add (pad);
-				}
-			}
+			Runtime.AddInService.RegisterExtensionItemListener ("/SharpDevelop/Workbench/Contexts", OnExtensionChanged);
 			
 			Runtime.LoggingService.Debug ("Default Layout created.");
 			dockLayout = new DockLayout (dock);
@@ -316,6 +337,22 @@ namespace MonoDevelop.Ide.Gui
 			}
 		}
 
+		void OnExtensionChanged (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add) {
+				WorkbenchContextCodon codon = (WorkbenchContextCodon) item;
+				PadContentCollection collection = new PadContentCollection ();
+				WorkbenchContext ctx = WorkbenchContext.GetContext (codon.ID);
+				padCollections [ctx] = collection;
+
+				foreach (ContextPadCodon padCodon in codon.Pads) {
+					IPadContent pad = workbench.PadContentCollection [padCodon.ID];
+					if (pad != null)
+						collection.Add (pad);
+				}
+			}
+		}
+		
 		public void Detach()
 		{
 			workbench.ContextChanged -= contextChangedHandler;
@@ -388,16 +425,29 @@ namespace MonoDevelop.Ide.Gui
 								 window.Icon,
 								 DockItemBehavior.Normal);
 
+			item.DockItemShown += delegate (object s, EventArgs a) {
+				window.NotifyShown ();
+			};
+
+			item.DockItemHidden += delegate (object s, EventArgs a) {
+				window.NotifyHidden ();
+			};
+
 			Gtk.Label label = item.TabLabel as Gtk.Label;
 			label.UseMarkup = true;
 
+			Gtk.Widget pcontent;
 			if (content is Widget)
-				item.Add (content.Control);
+				pcontent = content.Control;
 			else {
 				CommandRouterContainer crc = new CommandRouterContainer (content.Control, content, true);
 				crc.Show ();
-				item.Add (crc);
+				pcontent = crc;
 			}
+			
+			CommandRouterContainer router = new CommandRouterContainer (pcontent, toolbarFrame, false);
+			router.Show ();
+			item.Add (router);
 				
 			item.Show ();
 			item.HideItem ();
@@ -417,9 +467,7 @@ namespace MonoDevelop.Ide.Gui
 				GetPlacement (placement, out dockPlacement, out ot);
 				
 			if (item.Iconified)
-			{
 				item.Iconified = false;
-			}
 			
 			if (dockPlacement != DockPlacement.None && dockPlacement != DockPlacement.Floating) {
 				if (ot != null) {
@@ -451,13 +499,13 @@ namespace MonoDevelop.Ide.Gui
 		{
 			DockItem item = GetDockItem (content);
 			if (item != null) {
-			
-				// TODO: ShowItem is not working properly in the
-				// managed Gdl.
-/*				if (item.DefaultPosition != null)
+				if (item.IsAttached)
+					return;
+
+				if (item.DefaultPosition != null)
 					item.ShowItem();
 				else
-*/					DockPad (item, content.DefaultPlacement);
+					DockPad (item, content.DefaultPlacement);
 			}
 			else
 				AddPad (content, content.DefaultPlacement);
@@ -474,8 +522,9 @@ namespace MonoDevelop.Ide.Gui
 		public void HidePad (IPadContent padContent)
 		{
 			DockItem item = GetDockItem (padContent);
-			if (item != null)
+			if (item != null) {
 				item.HideItem();
+			}
 		}
 		
 		public void ActivatePad (IPadContent padContent)
@@ -492,6 +541,36 @@ namespace MonoDevelop.Ide.Gui
 				if (item != null)
 					item.LongName = GetPadWindow (content).Title;
 			}
+			
+			// If the toolbar or menubar has changed, replace it in the layout
+			
+			DefaultWorkbench wb = (DefaultWorkbench) workbench;
+			if (wb.ToolBars != toolBars) {
+				string cl = toolbarFrame.CurrentLayout;
+				DockToolbarFrameStatus mem = toolbarFrame.GetStatus ();
+				toolBars = wb.ToolBars;
+				toolbarFrame.ClearToolbars ();
+				if (toolBars != null) {
+					foreach (DockToolbar tb in toolBars) {
+						tb.ShowAll ();
+						toolbarFrame.AddBar (tb);
+					}
+				}
+				toolbarFrame.SetStatus (mem);
+				toolbarFrame.CurrentLayout = cl;
+			}
+
+			if (wb.TopMenu != menubar) {
+				Gtk.Box parent = (Gtk.Box) menubar.Parent;
+				int pos = ((Gtk.Box.BoxChild) parent [menubar]).Position;
+				
+				parent.PackStart (wb.TopMenu, false, false, 0);
+				((Gtk.Box.BoxChild) parent [wb.TopMenu]).Position = pos;
+				wb.TopMenu.ShowAll ();
+				
+				parent.Remove (menubar);
+				menubar = wb.TopMenu;
+			}
 		}
 		
 		public IPadWindow GetPadWindow (IPadContent content)
@@ -501,25 +580,31 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void CloseWindowEvent(object sender, EventArgs e)
 		{
-			// FIXME: GTKize
-
 			SdiWorkspaceWindow f = (SdiWorkspaceWindow)sender;
+			
+			// Unsubscribe events to avoid memory leaks
+			f.TabLabel.Button.Clicked -= new EventHandler (closeClicked);
+			f.TabLabel.Button.StateChanged -= new StateChangedHandler (stateChanged);
+
 			if (f.ViewContent != null) {
 				((IWorkbench)wbWindow).CloseContent(f.ViewContent);
 				ActiveMdiChanged(this, null);
 			}
-
 		}
 		
 		public IWorkbenchWindow ShowView(IViewContent content)
 		{	
 			Gtk.Image mimeimage = null;
-			if (content.IsUntitled && content.UntitledName == null) {
+			
+			if (content.StockIconId != null ) {
+				mimeimage = new Gtk.Image ( content.StockIconId, IconSize.Menu );
+			}
+			else if (content.IsUntitled && content.UntitledName == null) {
 				mimeimage = new Gtk.Image (FileIconLoader.GetPixbufForType ("gnome-fs-regular", 16));
 			} else {
 				mimeimage = new Gtk.Image (FileIconLoader.GetPixbufForFile (content.ContentName, 16));
-			}
-			
+			}			
+
 			TabLabel tabLabel = new TabLabel (new Label (), mimeimage != null ? mimeimage : new Gtk.Image (""));
 			tabLabel.Button.Clicked += new EventHandler (closeClicked);
 			tabLabel.Button.StateChanged += new StateChangedHandler (stateChanged);
@@ -541,17 +626,14 @@ namespace MonoDevelop.Ide.Gui
 
 		void closeClicked (object o, EventArgs e)
 		{
-			int pageNum = -1;
 			Widget parent = ((Widget)o).Parent;
 			foreach (Widget child in tabControl.Children) {
 				if (tabControl.GetTabLabel (child) == parent) {
-					pageNum = tabControl.PageNum (child);
+					int pageNum = tabControl.PageNum (child);
+					((SdiWorkspaceWindow)child).CloseWindow (false, false, pageNum);
 					break;
 				}
 			}
-			if (pageNum != -1) {
-				workbench.ViewContentCollection [pageNum].WorkbenchWindow.CloseWindow (false, false, pageNum);
-			}
 		}
 
 		public void RemoveTab (int pageNum) {
