@@ -90,7 +90,7 @@ namespace MonoDevelop.Ide.Gui
 			
 			IViewContent content = window.ActiveViewContent as IViewContent;
 			if (content != null)
-				info.Enabled = content.IsDirty;
+				info.Enabled = !content.IsViewOnly && content.IsDirty;
 			else
 				info.Enabled = false;
 		}
@@ -101,6 +101,16 @@ namespace MonoDevelop.Ide.Gui
 			IdeApp.Workbench.FindDocument (window).SaveAs ();
 		}
 		
+		[CommandUpdateHandler (FileCommands.SaveAs)]
+		protected void OnUpdateSaveFileAs (CommandInfo info)
+		{
+			IViewContent content = window.ActiveViewContent as IViewContent;
+			if (content != null)
+				info.Enabled = !content.IsViewOnly;
+			else
+				info.Enabled = false;
+		}
+		
 		[CommandHandler (FileCommands.ReloadFile)]
 		protected void OnReloadFile ()
 		{
@@ -292,5 +302,63 @@ namespace MonoDevelop.Ide.Gui
 		{
 			info.Enabled = window.ActiveViewContent is ICodeStyleOperations;
 		}
+		
+		[CommandHandler (EditCommands.UppercaseSelection)]
+		public void OnUppercaseSelection ()
+		{
+			IEditableTextBuffer buffer = window.ActiveViewContent as IEditableTextBuffer;
+			if (buffer != null)
+			{
+				if (buffer.SelectedText == String.Empty)
+				{
+					int pos = buffer.CursorPosition;
+					string ch = buffer.GetText (pos, pos + 1);
+					buffer.DeleteText (pos, 1);
+					buffer.InsertText (pos, ch.ToUpper ());
+					buffer.CursorPosition = pos + 1;
+				} else
+				{
+					string newText = buffer.SelectedText.ToUpper ();
+					int startPos = buffer.SelectionStartPosition;
+					buffer.DeleteText (startPos, buffer.SelectedText.Length);
+					buffer.InsertText (startPos, newText);
+				}
+			}
+		}
+		
+		[CommandUpdateHandler (EditCommands.UnIndentSelection)]
+		protected void OnUppercaseSelection (CommandInfo info)
+		{
+			info.Enabled = window.ActiveViewContent is IEditableTextBuffer;
+		}
+		
+		[CommandHandler (EditCommands.LowercaseSelection)]
+		public void OnLowercaseSelection ()
+		{
+			IEditableTextBuffer buffer = window.ActiveViewContent as IEditableTextBuffer;
+			if (buffer != null)
+			{
+				if (buffer.SelectedText == String.Empty)
+				{
+					int pos = buffer.CursorPosition;
+					string ch = buffer.GetText (pos, pos + 1);
+					buffer.DeleteText (pos, 1);
+					buffer.InsertText (pos, ch.ToLower ());
+					buffer.CursorPosition = pos + 1;
+				} else
+				{
+					string newText = buffer.SelectedText.ToLower ();
+					int startPos = buffer.SelectionStartPosition;
+					buffer.DeleteText (startPos, buffer.SelectedText.Length);
+					buffer.InsertText (startPos, newText);
+				}
+			}
+		}
+		
+		[CommandUpdateHandler (EditCommands.LowercaseSelection)]
+		protected void OnLowercaseSelection (CommandInfo info)
+		{
+			info.Enabled = window.ActiveViewContent is IEditableTextBuffer;
+		}
 	}
 }
