@@ -38,6 +38,7 @@ using MonoDevelop.Core.Properties;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Ide.Codons;
 using MonoDevelop.Ide.Gui.Content;
+using MonoDevelop.Core.Gui.Utils;
 
 namespace MonoDevelop.Ide.Gui
 {
@@ -164,7 +165,10 @@ namespace MonoDevelop.Ide.Gui
 		
 		public string CurrentLayout {
 			get { return workbench.WorkbenchLayout != null ? workbench.WorkbenchLayout.CurrentLayout : ""; }
-			set { workbench.WorkbenchLayout.CurrentLayout = value; }
+			set {
+				if (value != workbench.WorkbenchLayout.CurrentLayout)
+					workbench.WorkbenchLayout.CurrentLayout = value; 
+			}
 		}
 
 		public string[] Layouts {
@@ -220,25 +224,81 @@ namespace MonoDevelop.Ide.Gui
 			return WrapPad (content);
 		}
 		
+		public FileViewer[] GetFileViewers (string fileName)
+		{
+			ArrayList list = new ArrayList ();
+			
+			string mimeType = Gnome.Vfs.MimeType.GetMimeTypeForUri (fileName);
+
+			IDisplayBinding[] bindings = Services.DisplayBindings.GetBindingsForMimeType (mimeType);
+			foreach (IDisplayBinding bin in bindings)
+				list.Add (new FileViewer (bin));
+
+			foreach (DesktopApplication app in DesktopApplication.GetApplications (mimeType))
+				if (app.Command != "monodevelop")
+					list.Add (new FileViewer (app));
+				
+			return (FileViewer[]) list.ToArray (typeof(FileViewer));
+		}
+		
 		public Document OpenDocument (string fileName)
 		{
 			return OpenDocument (fileName, true);
 		}
 		
+		public Document OpenDocument (string fileName, string encoding)
+		{
+			return OpenDocument (fileName, -1, -1, true, encoding, null);
+		}
+		
 		public Document OpenDocument (string fileName, bool bringToFront)
 		{
 			return OpenDocument (fileName, -1, -1, bringToFront);
 		}
 		
 		public Document OpenDocument (string fileName, int line, int column, bool bringToFront)
+		{
+			return OpenDocument (fileName, line, column, bringToFront, null, null);
+		}
+		
+		public Document OpenDocument (string fileName, int line, int column, bool bringToFront, string encoding)
+		{
+			return OpenDocument (fileName, line, column, bringToFront, encoding, null);
+		}
+		
+		internal Document OpenDocument (string fileName, int line, int column, bool bringToFront, string encoding, IDisplayBinding binding)
 		{
 			foreach (Document doc in Documents) {
-				if (doc.FileName == fileName) {
-					if (bringToFront)
-						doc.Select ();
-					if (line != -1 && doc.Window.ViewContent is IPositionable) {
-						((IPositionable)doc.Window.ViewContent).JumpTo (line, column != -1 ? column : 0);
+				IBaseViewContent vcFound = null;
+				int vcIndex = 0;
+				
+				//search all ViewContents to see if they can "re-use" this filename
+				if (doc.Window.ViewContent.CanReuseView (fileName))
+					vcFound = doc.Window.ViewContent;
+				else if (doc.Window.SubViewContents != null) {
+					for (int i = 0; i < doc.Window.SubViewContents.Count; i++) {
+						ISecondaryViewContent vc = (ISecondaryViewContent) doc.Window.SubViewContents [i];
+						if (vc.CanReuseView (fileName)) {
+							vcIndex = i +1;
+							vcFound = vc;
+						}
 					}
+				}
+				
+				//old method as fallback
+				if ((vcFound == null) && (doc.FileName == fileName))
+					vcFound = doc.Window.ViewContent;
+				
+				//if found, select window and jump to line
+				if (vcFound != null) {
+					if (bringToFront) {
+						doc.Select ();
+						doc.Window.SwitchView (vcIndex);
+					}
+					if (line != -1 && vcFound is IPositionable) {
+						((IPositionable) vcFound).JumpTo (line, column != -1 ? column : 0);
+					}
+					
 					return doc;
 				}
 			}
@@ -250,6 +310,8 @@ namespace MonoDevelop.Ide.Gui
 			openFileInfo.BringToFront = bringToFront;
 			openFileInfo.Line = line;
 			openFileInfo.Column = column;
+			openFileInfo.DisplayBinding = binding;
+			openFileInfo.Encoding = encoding;
 			RealOpenFile (openFileInfo);
 			
 			if (!pm.AsyncOperation.Success)
@@ -431,7 +493,11 @@ namespace MonoDevelop.Ide.Gui
 					}
 				}
 				
-				IDisplayBinding binding = Services.DisplayBindings.GetBindingPerFileName(fileName);
+				IDisplayBinding binding;
+				if (oFileInfo.DisplayBinding != null)
+					binding = oFileInfo.DisplayBinding;
+				else
+					binding = Services.DisplayBindings.GetBindingPerFileName(fileName);
 				
 				if (binding != null) {
 					Project project = null;
@@ -516,7 +582,9 @@ namespace MonoDevelop.Ide.Gui
 		public bool BringToFront;
 		public int Line;
 		public int Column;
+		public IDisplayBinding DisplayBinding;
 		public IViewContent NewContent;
+		public string Encoding;
 	}
 	
 	class LoadFileWrapper
@@ -544,9 +612,21 @@ namespace MonoDevelop.Ide.Gui
 		
 		public void Invoke(string fileName)
 		{
-			newContent = binding.CreateContentForFile(fileName);
-			if (newContent == null)
+			try {
+				newContent = binding.CreateContentForFile (fileName);
+				if (newContent == null) {
+					fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), null);
+					return;
+				}
+
+				if (fileInfo.Encoding != null && newContent is IEncodedTextContent)
+					((IEncodedTextContent)newContent).Load (fileName, fileInfo.Encoding);
+				else
+					newContent.Load (fileName);
+			} catch (Exception ex) {
+				fileInfo.ProgressMonitor.ReportError (GettextCatalog.GetString ("The file '{0}' could not be opened.", fileName), ex);
 				return;
+			}
 
 			if (project != null)
 				newContent.Project = project;
