@@ -193,12 +193,13 @@ namespace MonoDevelop.Ide.Templates
 		
 		static FileTemplate()
 		{
-			LoadTemplates ((FileTemplateCodon[]) Runtime.AddInService.GetTreeItems ("/MonoDevelop/FileTemplates", typeof (FileTemplateCodon)));
+			Runtime.AddInService.RegisterExtensionItemListener ("/MonoDevelop/FileTemplates", OnExtensionChanged);
 		}
 
-		static void LoadTemplates (FileTemplateCodon[] codons)
+		static void OnExtensionChanged (ExtensionAction action, object item)
 		{
-			foreach (FileTemplateCodon codon in codons) {
+			if (action == ExtensionAction.Add) {
+				FileTemplateCodon codon = (FileTemplateCodon) item;
 				try {
 					FileTemplate t = LoadFileTemplate (codon.AddIn, codon.Resource);
 					t.id = codon.ID;
@@ -237,6 +238,16 @@ namespace MonoDevelop.Ide.Templates
 			}
 		}
 		
+		public virtual bool IsValidName (string name, string language)
+		{
+			bool valid = true;
+			foreach (FileDescriptionTemplate templ in Files)
+				if (!templ.IsValidName (name, language))
+					valid = false;
+			
+			return valid;
+		}
+		
 		protected virtual void CreateFile (FileDescriptionTemplate newfile, Project project, string directory, string language, string name)
 		{
 			if (project != null) {
