@@ -29,6 +29,7 @@
 using System;
 using System.IO;
 using System.Xml;
+using System.Collections;
 
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
@@ -39,13 +40,15 @@ namespace MonoDevelop.Ide.Templates
 	public class SingleFileDescriptionTemplate: FileDescriptionTemplate
 	{
 		string name;
+		string defaultName;
+		string defaultExtension;
 		string generatedFile;
 		
 		public override void Load (XmlElement filenode)
 		{
 			name = filenode.GetAttribute ("name");
-			if (name == "")
-				name = filenode.GetAttribute ("DefaultName") + filenode.GetAttribute ("DefaultExtension");
+			defaultName = filenode.GetAttribute ("DefaultName");
+			defaultExtension = filenode.GetAttribute ("DefaultExtension");
 		}
 		
 		public override string Name {
@@ -110,21 +113,30 @@ namespace MonoDevelop.Ide.Templates
 		
 		// Returns the name of the file that this template generates.
 		// All parameters are optional (can be null)
-		public string GetFileName (Project project, string language, string baseDirectory, string entryName)
+		public virtual string GetFileName (Project project, string language, string baseDirectory, string entryName)
 		{
-			string fileName = entryName;
-			string defaultName = name;
+			if (string.IsNullOrEmpty (entryName) && !string.IsNullOrEmpty (defaultName))
+				entryName = defaultName;
 			
-			if (language != null && language.Length > 0 && Path.GetExtension (defaultName).Length == 0) {
-				IDotNetLanguageBinding languageBinding = GetDotNetLanguageBinding (language);
-				defaultName = languageBinding.GetFileName (defaultName);
+			string fileName = entryName;
+			
+			//substitute tags
+			if ((name != null) && (name.Length > 0)) {
+				StringParserService sps = (StringParserService) ServiceManager.GetService (typeof (StringParserService));
+				Hashtable tags = new Hashtable ();
+				ModifyTags (project, language, entryName, null, ref tags);
+				fileName = sps.Parse (name, HashtableToStringArray (tags));
 			}
 			
-			if (fileName != null) {
-				if (Path.GetExtension (name) != Path.GetExtension (defaultName))
-					fileName = fileName + Path.GetExtension (defaultName);
-			} else {
-				fileName = defaultName;
+			//give it an extension if it didn't get one (compatibility with pre-substition behaviour)
+			if (Path.GetExtension (fileName).Length == 0) {
+				if (!string.IsNullOrEmpty  (defaultExtension)) {
+					fileName = fileName + defaultExtension;
+				}
+				else if (!string.IsNullOrEmpty  (language)) {
+					IDotNetLanguageBinding languageBinding = GetDotNetLanguageBinding (language);
+					fileName = languageBinding.GetFileName (fileName);
+				} 
 			}
 			
 			if (baseDirectory != null)
@@ -142,19 +154,11 @@ namespace MonoDevelop.Ide.Templates
 			if (project != null && project.IsFileInProject (fileName))
 				throw new UserException (GettextCatalog.GetString ("The file '{0}' already exists in the project.", Path.GetFileName (fileName)));
 			
-			string content = CreateContent (language);
+			Hashtable tags = new Hashtable ();
+			ModifyTags (project, language, null, fileName, ref tags);
 			
-			DotNetProject netProject = project as DotNetProject;
-			string ns = netProject != null ? netProject.GetDefaultNamespace (fileName) : "";
-			string cname = Path.GetFileNameWithoutExtension (fileName);
-			string[,] tags = { 
-				{"Name", cname}, 
-				{"Namespace", ns},
-				{"FullName", ns.Length > 0 ? ns + "." + cname : cname},
-				{"ProjectName", project != null ? project.Name : ""}
-			};
-				
-			content = sps.Parse (content, tags);
+			string content = CreateContent (language);
+			content = sps.Parse (content, HashtableToStringArray (tags));
 			
 			MemoryStream ms = new MemoryStream ();
 			byte[] data = System.Text.Encoding.UTF8.GetBytes (content);
@@ -165,11 +169,55 @@ namespace MonoDevelop.Ide.Templates
 		
 		// Creates the text content of the file
 		// The Language parameter is optional
+
 		public virtual string CreateContent (string language)
 		{
 			return "";
 		}
 		
+		// Can add tags for substitution based on project, language or filename.
+		// If overriding but still want base implementation's tags, should invoke base method.
+		// We supply defaults whenever it is possible, to avoid having unsubstituted tags. However,
+		// do not substitute blanks when a sensible default cannot be guessed, because they result
+		//in less obvious errors.
+		public virtual void ModifyTags (Project project, string language, string identifier, string fileName, ref Hashtable tags)
+		{
+			DotNetProject netProject = project as DotNetProject;
+			string languageExtension = "";
+			if (!string.IsNullOrEmpty (language)) {
+				IDotNetLanguageBinding binding = GetDotNetLanguageBinding (language);
+				if (binding != null)
+					languageExtension = Path.GetExtension (binding.GetFileName ("Default")).Remove (0, 1);
+			}
+			
+			//need a default namespace or if there is no project, substitutions can get very messed up
+			string ns = netProject != null ? netProject.GetDefaultNamespace (fileName) : "Application";
+			
+			//need an 'identifier' for tag substitution, e.g. class name or page name
+			//if not given an identifier, use fileName
+			if ((identifier == null) && (fileName != null))
+				identifier = Path.GetFileName (fileName);
+			 
+			 if (identifier != null) {
+			 	//remove all extensions
+				while (Path.GetExtension (identifier).Length > 0)
+					identifier = Path.GetFileNameWithoutExtension (identifier);
+			 	
+				tags ["Name"] = identifier;
+				tags ["FullName"] = ns.Length > 0 ? ns + "." + identifier : identifier;
+			}			
+			
+			if (ns.Length > 0)
+				tags ["Namespace"] = ns;
+			if (project != null)
+				tags ["ProjectName"] = project.Name;
+			if ((language != null) && (language.Length > 0))
+				tags ["Language"] = language;
+			if (languageExtension.Length > 0)
+				tags ["LanguageExtension"] = languageExtension;
+			tags ["FileName"] = fileName;
+		}
+		
 		protected IDotNetLanguageBinding GetDotNetLanguageBinding (string language)
 		{
 			IDotNetLanguageBinding binding = MonoDevelop.Projects.Services.Languages.GetBindingPerLanguageName (language) as IDotNetLanguageBinding;
@@ -177,5 +225,33 @@ namespace MonoDevelop.Ide.Templates
 				throw new InvalidOperationException ("Language '" + language + "' not found");
 			return binding;
 		}
+		
+		protected string[,] HashtableToStringArray (Hashtable tags)
+		{			
+			string[,] tagsArr = new string [tags.Count, 2];
+			int i = 0;
+			foreach (string key in tags.Keys) {
+				tagsArr [i, 0] = key;
+				tagsArr [i, 1] = (string) tags [key];
+				i++;
+			}
+			
+			return tagsArr;
+		}
+		
+		public override bool IsValidName (string name, string language)
+		{
+			if (name.Length > 0) {
+				if (language != null && language.Length > 0) {
+					IDotNetLanguageBinding binding = GetDotNetLanguageBinding (language);
+					System.CodeDom.Compiler.CodeDomProvider provider = binding.GetCodeDomProvider ();
+					if (provider != null)
+						return provider.IsValidIdentifier (name);
+				}
+				return name.IndexOfAny (Path.InvalidPathChars) == -1;
+			}
+			else
+				return false;
+		}
 	}
 }
