@@ -16,6 +16,7 @@ using MonoDevelop.Projects.Parser;
 using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Ambience;
+using Ambience_ = MonoDevelop.Projects.Ambience.Ambience;
 
 namespace MonoDevelop.Projects.Gui.Completion
 {
@@ -30,15 +31,6 @@ namespace MonoDevelop.Projects.Gui.Completion
 
 		bool convertedDocumentation = false;
 		
-		static IAmbience PangoAmbience
-		{
-			get {
-				IAmbience asvc = Services.Ambience.CurrentAmbience;
-				asvc.ConversionFlags |= ConversionFlags.IncludePangoMarkup;
-				return asvc;
-			}
-		}
-
 		public string CompletionString 
 		{
 			get 
@@ -144,57 +136,57 @@ namespace MonoDevelop.Projects.Gui.Completion
 			this.image = image;
 		}
 		
-		public CodeCompletionData (IClass c)
+		public CodeCompletionData (IClass c, Ambience_ ambience)
 		{
 			image = Services.Icons.GetIcon(c);
 			text = c.Name;
 			completionString = c.Name;
-			description = Services.Ambience.CurrentAmbience.Convert(c);
-			pango_description  = PangoAmbience.Convert(c);
+			description = ambience.Convert(c);
+			pango_description = ambience.Convert(c, ConversionFlags.StandardConversionFlags | ConversionFlags.IncludePangoMarkup);
 			documentation = c.Documentation;
 		}
 		
-		public CodeCompletionData (IMethod method)
+		public CodeCompletionData (IMethod method, Ambience_ ambience)
 		{
 			image  = Services.Icons.GetIcon(method);
 			text        = method.Name;
-			description = Services.Ambience.CurrentAmbience.Convert(method);
-			pango_description  = PangoAmbience.Convert (method);
+			description = ambience.Convert(method);
+			pango_description = ambience.Convert(method, ConversionFlags.StandardConversionFlags | ConversionFlags.IncludePangoMarkup);
 			completionString = method.Name;
 			documentation = method.Documentation;
 		}
 		
-		public CodeCompletionData (IField field)
+		public CodeCompletionData (IField field, Ambience_ ambience)
 		{
 			image  = Services.Icons.GetIcon(field);
 			text        = field.Name;
-			description = Services.Ambience.CurrentAmbience.Convert(field);
-			pango_description  = PangoAmbience.Convert (field);
+			description = ambience.Convert(field);
+			pango_description = ambience.Convert(field, ConversionFlags.StandardConversionFlags | ConversionFlags.IncludePangoMarkup);
 			completionString = field.Name;
 			documentation = field.Documentation;
 		}
 		
-		public CodeCompletionData (IProperty property)
+		public CodeCompletionData (IProperty property, Ambience_ ambience)
 		{
 			image  = Services.Icons.GetIcon(property);
 			text        = property.Name;
-			description = Services.Ambience.CurrentAmbience.Convert(property);
-			pango_description  = PangoAmbience.Convert (property);
+			description = ambience.Convert(property);
+			pango_description  = ambience.Convert(property, ConversionFlags.StandardConversionFlags | ConversionFlags.IncludePangoMarkup);
 			completionString = property.Name;
 			documentation = property.Documentation;
 		}
 		
-		public CodeCompletionData (IEvent e)
+		public CodeCompletionData (IEvent e, Ambience_ ambience)
 		{
 			image  = Services.Icons.GetIcon(e);
 			text        = e.Name;
-			description = Services.Ambience.CurrentAmbience.Convert(e);
-			pango_description  = PangoAmbience.Convert (e);
+			description = ambience.Convert(e);
+			pango_description  = ambience.Convert(e, ConversionFlags.StandardConversionFlags | ConversionFlags.IncludePangoMarkup);
 			completionString = e.Name;
 			documentation = e.Documentation;
 		}
 
-		public CodeCompletionData (IParameter o)
+		public CodeCompletionData (IParameter o, Ambience_ ambience)
 		{
 			image = MonoDevelop.Core.Gui.Stock.Field;
 			text  = o.Name;
