@@ -17,6 +17,7 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Gui;
 using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects;
+using Ambience_ = MonoDevelop.Projects.Ambience.Ambience;
 
 using Stock = MonoDevelop.Core.Gui.Stock;
 
@@ -27,7 +28,7 @@ namespace MonoDevelop.Projects.Gui.Completion
 	/// <summary>
 	/// Data provider for code completion.
 	/// </summary>
-	public class CodeCompletionDataProvider : ICompletionDataProvider
+	public class CodeCompletionDataProvider : IMutableCompletionDataProvider, IDisposable
 	{
 //		static AmbienceService          ambienceService = (AmbienceService)ServiceManager.Services.GetService(typeof(AmbienceService));
 		Hashtable insertedClasses = new Hashtable ();
@@ -35,28 +36,48 @@ namespace MonoDevelop.Projects.Gui.Completion
 		Hashtable insertedPropertiesElements = new Hashtable();
 		Hashtable insertedEventElements      = new Hashtable();
 		
+		Ambience_ ambience;
 		int caretLineNumber;
 		int caretColumn;
 		bool ctrlspace;
 		IParserContext parserContext;
 		string fileName;
+		EventHandler onStartedParsing;
+		EventHandler onFinishedParsing;
 
-		public CodeCompletionDataProvider (IParserContext parserContext, string fileName) : this (parserContext, fileName, false)
+		ArrayList completionData = null;
+		
+		public CodeCompletionDataProvider (IParserContext parserContext, Ambience_ ambience, string fileName) : this (parserContext, ambience, fileName, false)
 		{
 		}
 			
-		public CodeCompletionDataProvider (IParserContext parserContext, string fileName, bool ctrl) 
+		public CodeCompletionDataProvider (IParserContext parserContext, Ambience_ ambience, string fileName, bool ctrl) 
 		{
 			this.fileName = fileName;
 			this.parserContext = parserContext;
 			this.ctrlspace = ctrl;
+			this.ambience = ambience;
+			
+			onStartedParsing = (EventHandler) Services.DispatchService.GuiDispatch (new EventHandler (OnStartedParsing));
+			onFinishedParsing = (EventHandler) Services.DispatchService.GuiDispatch (new EventHandler (OnFinishedParsing));
+			
+			parserContext.ParserDatabase.ParseOperationStarted += onStartedParsing;
+			parserContext.ParserDatabase.ParseOperationFinished += onFinishedParsing;
 		}
 		
-		ArrayList completionData = null;
+		public virtual void Dispose ()
+		{
+			parserContext.ParserDatabase.ParseOperationStarted -= onStartedParsing;
+			parserContext.ParserDatabase.ParseOperationFinished -= onFinishedParsing;
+		}
 		
 		public ICompletionData[] GenerateCompletionData (ICompletionWidget widget, char charTyped)
 		{
 			completionData = new ArrayList();
+			insertedClasses.Clear ();
+			insertedElements.Clear ();
+			insertedPropertiesElements.Clear ();
+			insertedEventElements.Clear ();
 			
 			// the parser works with 1 based coordinates
 			caretLineNumber      = widget.TriggerLine + 1;
@@ -67,7 +88,7 @@ namespace MonoDevelop.Projects.Gui.Completion
 			IExpressionFinder expressionFinder = parserContext.GetExpressionFinder(fileName);
 			string expression    = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset(widget, widget.TriggerOffset) : expressionFinder.FindExpression(widget.GetText (0, widget.TriggerOffset), widget.TriggerOffset - 2).Expression;
 			if (expression == null) return null;
-			Console.WriteLine ("Expr: |{0}|", expression);
+
 			//FIXME: This chartyped check is a fucking *HACK*
 			if (expression == "is" || expression == "as") {
 				string expr = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset (widget, widget.TriggerOffset - 3) : expressionFinder.FindExpression (widget.GetText (0, widget.TriggerOffset), widget.TriggerOffset - 5).Expression;
@@ -106,20 +127,20 @@ namespace MonoDevelop.Projects.Gui.Completion
 				} else if (o is IClass) {
 					IClass iclass = (IClass) o;
 					if (iclass.Name != null && insertedClasses[iclass.Name] == null) {
-						completionData.Add(new CodeCompletionData(iclass));
+						completionData.Add(new CodeCompletionData(iclass, ambience));
 						insertedClasses[iclass.Name] = iclass;
 					}
 				} else if (o is IProperty) {
 					IProperty property = (IProperty)o;
 					if (property.Name != null && insertedPropertiesElements[property.Name] == null) {
-						completionData.Add(new CodeCompletionData(property));
+						completionData.Add(new CodeCompletionData(property, ambience));
 						insertedPropertiesElements[property.Name] = property;
 					}
 				} else if (o is IMethod) {
 					IMethod method = (IMethod)o;
 					
 					if (method.Name != null &&!method.IsConstructor) {
-						CodeCompletionData ccd = new CodeCompletionData(method);
+						CodeCompletionData ccd = new CodeCompletionData(method, ambience);
 						if (insertedElements[method.Name] == null) {
 							completionData.Add(ccd);
 							insertedElements[method.Name] = ccd;
@@ -130,15 +151,15 @@ namespace MonoDevelop.Projects.Gui.Completion
 						}
 					}
 				} else if (o is IField) {
-					completionData.Add(new CodeCompletionData((IField)o));
+					completionData.Add(new CodeCompletionData((IField)o, ambience));
 				} else if (o is IEvent) {
 					IEvent e = (IEvent)o;
 					if (e.Name != null && insertedEventElements[e.Name] == null) {
-						completionData.Add(new CodeCompletionData(e));
+						completionData.Add(new CodeCompletionData(e, ambience));
 						insertedEventElements[e.Name] = e;
 					}
 				} else if (o is IParameter) {
-					completionData.Add (new CodeCompletionData((IParameter)o));
+					completionData.Add (new CodeCompletionData((IParameter)o, ambience));
 				}
 			}
 		}
@@ -150,5 +171,24 @@ namespace MonoDevelop.Projects.Gui.Completion
 				AddResolveResults(results.Members);
 			}
 		}
+		
+		public bool IsChanging { 
+			get { return parserContext.ParserDatabase.IsParsing; } 
+		}
+		
+		void OnStartedParsing (object s, EventArgs args)
+		{
+			if (CompletionDataChanging != null)
+				CompletionDataChanging (this, EventArgs.Empty);
+		}
+		
+		void OnFinishedParsing (object s, EventArgs args)
+		{
+			if (CompletionDataChanged != null)
+				CompletionDataChanged (this, EventArgs.Empty);
+		}
+		
+		public event EventHandler CompletionDataChanging;
+		public event EventHandler CompletionDataChanged;
 	}
 }
