@@ -276,7 +276,7 @@ namespace MonoDevelop.Projects.CodeGeneration
 		// Returns a reparsed IClass instance that contains the generated code.
 		protected IClass GetGeneratedClass (RefactorerContext ctx, IEditableTextFile buffer, IClass cls)
 		{
-			IParseInformation pi = ctx.ParserContext.ParseFile (buffer.Name, buffer.Text);
+			IParseInformation pi = ctx.ParserContext.ParserDatabase.UpdateFile (buffer.Name, buffer.Text);
 			foreach (IClass rclass in ((ICompilationUnit)pi.BestCompilationUnit).Classes) {
 				if (cls.Name == rclass.Name)
 					return rclass;
@@ -409,7 +409,14 @@ namespace MonoDevelop.Projects.CodeGeneration
 			}
 			else {
 				IMethod m = cls.Methods [cls.Methods.Count - 1];
-				int pos = buffer.GetPositionFromLineColumn (m.BodyRegion.EndLine, m.BodyRegion.EndColumn);
+				
+				int pos;
+				if (m.BodyRegion != null && m.BodyRegion.EndLine > 0)
+					pos = buffer.GetPositionFromLineColumn (m.BodyRegion.EndLine, m.BodyRegion.EndColumn);
+				else
+					// Abstract or P/Inboke methods don't have a body
+					pos = buffer.GetPositionFromLineColumn (m.Region.EndLine, m.Region.EndColumn);
+
 				pos = GetNextLine (buffer, pos);
 				pos = GetNextLine (buffer, pos);
 				string ind = GetLineIndent (buffer, m.Region.EndLine);
