@@ -15,6 +15,8 @@ namespace MonoDevelop.Projects.Parser
 		protected int    pointerNestingLevel;
 		protected int[]  arrayDimensions;
 		protected object declaredin = null;
+		protected ReturnTypeList genericArguments;
+		protected bool   byRef;
 		string fname;
 		
 		public virtual string FullyQualifiedName {
@@ -74,6 +76,28 @@ namespace MonoDevelop.Projects.Parser
 				return arrayDimensions;
 			}
 		}
+		 		
+		/// <summary>
+		/// Indicates whether the return type is passed by reference.
+		/// </summary>
+		public virtual bool ByRef {
+			get {
+				return byRef;
+			}
+		}
+		 		
+		/// <summary>
+		/// Contains values (types) of actual parameters (arguments) to a
+		/// generic type.
+		/// </summary>
+		public virtual ReturnTypeList GenericArguments {
+			get {
+				return genericArguments;
+			}
+			set {
+				genericArguments = value;
+			}
+		}
 
 		public virtual int CompareTo (object ob) 
 		{
@@ -95,7 +119,14 @@ namespace MonoDevelop.Projects.Parser
 				return cmp;
 			}
 			
-			return DiffUtility.Compare(ArrayDimensions, value.ArrayDimensions);
+			cmp = DiffUtility.Compare(ArrayDimensions, value.ArrayDimensions);
+			if (cmp != 0)
+				return cmp;
+			
+			if (GenericArguments == value.GenericArguments)
+				return 0;
+			else
+				return DiffUtility.Compare(GenericArguments, value.GenericArguments);
 		}
 		
 		int IComparable.CompareTo(object value)
