@@ -47,7 +47,7 @@ namespace MonoDevelop.Projects.Parser
 	{
 		static readonly int MAX_ACTIVE_COUNT = 100;
 		static readonly int MIN_ACTIVE_COUNT = 50;
-		static protected readonly int FORMAT_VERSION = 6;
+		static protected readonly int FORMAT_VERSION = 13;
 		
 		NamespaceEntry rootNamespace;
 		protected ArrayList references;
@@ -59,6 +59,7 @@ namespace MonoDevelop.Projects.Parser
 		FileStream datafile;
 		int currentGetTime = 0;
 		bool modified;
+		bool disposed;
 		
 		string basePath;
 		string dataFile;
@@ -76,6 +77,7 @@ namespace MonoDevelop.Projects.Parser
 		
 		public virtual void Dispose ()
 		{
+			disposed = true;
 		}
 		
 		public string DataFile
@@ -83,6 +85,15 @@ namespace MonoDevelop.Projects.Parser
 			get { return dataFile; }
 		}
 		
+		public bool Modified {
+			get { return modified; }
+			set { modified = value; }
+		}
+		
+		public bool Disposed {
+			get { return disposed; }
+		}
+		
 		protected void SetLocation (string basePath, string name)
 		{
 			dataFile = Path.Combine (basePath, name + ".pidb");
@@ -121,6 +132,7 @@ namespace MonoDevelop.Projects.Parser
 					CloseReader ();
 					
 					Runtime.LoggingService.Debug ("Reading " + dataFile);
+					
 					ifile = new FileStream (dataFile, FileMode.Open, FileAccess.Read, FileShare.Read);
 					BinaryFormatter bf = new BinaryFormatter ();
 					
@@ -143,7 +155,6 @@ namespace MonoDevelop.Projects.Parser
 					DeserializeData (dataQueue);
 
 					ifile.Close ();
-					
 				}
 				catch (Exception ex)
 				{
@@ -172,6 +183,7 @@ namespace MonoDevelop.Projects.Parser
 			lock (rwlock)
 			{
 				if (!modified) return;
+				
 				modified = false;
 				headers["Version"] = FORMAT_VERSION;
 							
@@ -213,12 +225,19 @@ namespace MonoDevelop.Projects.Parser
 								}
 								datafile.Position = ce.Position;
 								len = datareader.ReadInt32 ();
+								
+								// Sanity check to avoid allocating huge byte arrays if something
+								// goes wrong when reading the file contents
+								if (len > 1024*1024*10 || len < 0)
+									throw new InvalidOperationException ("pidb file corrupted: " + dataFile);
+									
 								data = new byte[len];
 								datafile.Read (data, 0, len);
 							}
 							else {
 								buffer.Position = 0;
 								PersistentClass.WriteTo (c, bufWriter, parserDatabase.DefaultNameEncoder);
+								bufWriter.Flush ();
 								data = buffer.GetBuffer ();
 								len = (int)buffer.Position;
 							}
@@ -277,8 +296,13 @@ namespace MonoDevelop.Projects.Parser
 			return files [name] as FileEntry;
 		}
 				
-		void Flush ()
+		public void Flush ()
 		{
+			// Saves the database if it has too much information
+			// in memory. A parser database can't have more
+			// MAX_ACTIVE_COUNT classes loaded in memory at the
+			// same time.
+
 			int activeCount = 0;
 			
 			foreach (FileEntry fe in files.Values) {
@@ -381,35 +405,64 @@ namespace MonoDevelop.Projects.Parser
 			
 			ce.Class = ReadClass (ce);
 			return ce.Class;
-		}		
+		}
+		
+		public void UpdateDatabase ()
+		{
+			ArrayList list = GetModifiedFileEntries ();
+			foreach (FileEntry file in list)
+				ParseFile (file.FileName, null);
+		}
 		
 		public virtual void CheckModifiedFiles ()
 		{
+			ArrayList list = GetModifiedFileEntries ();
+			foreach (FileEntry file in list)
+				QueueParseJob (file);
+		}
+		
+		protected ArrayList GetModifiedFileEntries ()
+		{
+			ArrayList list = new ArrayList ();
 			lock (rwlock)
 			{
-				foreach (FileEntry file in files.Values)
-				{
-					if (!File.Exists (file.FileName)) continue;
-					FileInfo fi = new FileInfo (file.FileName);
-					if ((fi.LastWriteTime > file.LastParseTime || file.ParseErrorRetries > 0) && !file.DisableParse)
-						QueueParseJob (file);
+				foreach (FileEntry file in files.Values) {
+					if (IsFileModified (file))
+						list.Add (file);
 				}
 			}
+			return list;
+		}
+		
+		protected virtual bool IsFileModified (FileEntry file)
+		{
+			if (!File.Exists (file.FileName))
+				return false;
+			FileInfo fi = new FileInfo (file.FileName);
+			return ((fi.LastWriteTime > file.LastParseTime || file.ParseErrorRetries > 0) && !file.DisableParse);
 		}
 		
 		protected void QueueParseJob (FileEntry file)
 		{
-			// Change date now, to avoid reparsing if CheckModifiedFiles is called again
-			// before the parse job is executed
-			
-			FileInfo fi = new FileInfo (file.FileName);
-			file.LastParseTime = fi.LastWriteTime;
-			parserDatabase.QueueParseJob (new JobCallback (ParseCallback), file.FileName);
+			if (file.InParseQueue)
+				return;
+
+			file.InParseQueue = true;
+			parserDatabase.QueueParseJob (this, new JobCallback (ParseCallback), file.FileName);
 		}
 		
 		void ParseCallback (object ob, IProgressMonitor monitor)
 		{
-			ParseFile ((string)ob, monitor);
+			string fileName = (string) ob;
+			ParseFile (fileName, monitor);
+			lock (rwlock) {
+				FileEntry file = GetFile (fileName);
+				if (file != null) {
+					file.InParseQueue = false;
+					FileInfo fi = new FileInfo (fileName);
+					file.LastParseTime = fi.LastWriteTime;
+				}
+			}
 		}
 		
 		protected virtual void ParseFile (string fileName, IProgressMonitor monitor)
@@ -564,7 +617,6 @@ namespace MonoDevelop.Projects.Parser
 				rootNamespace.Clean ();
 				fe.LastParseTime = DateTime.Now;
 				modified = true;
-				Flush ();
 				
 				return res;
 			}
