@@ -114,6 +114,15 @@ namespace MonoDevelop.Projects.Parser
 			this.db = db;
 		}
 		
+		public IParserDatabase ParserDatabase { 
+			get { return pdb; }
+		}
+		
+		public void UpdateDatabase ()
+		{
+			db.UpdateDatabase ();
+		}
+		
 		public IExpressionFinder GetExpressionFinder (string fileName)
 		{
 			return pdb.GetExpressionFinder (fileName);
@@ -330,6 +339,7 @@ namespace MonoDevelop.Projects.Parser
 		bool threadRunning;
 		bool trackingFileChanges;
 		IProgressMonitorFactory parseProgressMonitorFactory;
+		int parseStatus;
 		
 		// Only keeps track of explicitely loaded assemblies, not the ones
 		// referenced by projects.
@@ -356,6 +366,7 @@ namespace MonoDevelop.Projects.Parser
 		{
 			public object Data;
 			public JobCallback ParseCallback;
+			public CodeCompletionDatabase Database;
 		}
 
 		class CompilationUnitTypeResolver: ITypeResolver
@@ -398,6 +409,7 @@ namespace MonoDevelop.Projects.Parser
 		CombineEntryEventHandler combineEntryRemovedHandler;
 
 		Queue parseQueue = new Queue();
+		object parseQueueLock = new object ();
 		AutoResetEvent parseEvent = new AutoResetEvent (false);
 		
 		string codeCompletionPath;
@@ -453,7 +465,10 @@ namespace MonoDevelop.Projects.Parser
 		
 		public IProjectParserContext GetProjectParserContext (Project project)
 		{
-			return new ProjectParserContext (parserService, this, GetProjectDatabase (project));
+			CodeCompletionDatabase pdb = GetProjectDatabase (project);
+			if (pdb == null)
+				throw new InvalidOperationException ("Project not found in parser database");
+			return new ProjectParserContext (parserService, this, pdb);
 		}
 		
 		public IFileParserContext GetFileParserContext (string file)
@@ -471,6 +486,9 @@ namespace MonoDevelop.Projects.Parser
 			set { parseProgressMonitorFactory = value; }
 		}
 		
+		public bool IsParsing {
+			get { return parseStatus > 0; }
+		}
 		
 		public bool TrackFileChanges {
 			get {
@@ -551,10 +569,13 @@ namespace MonoDevelop.Projects.Parser
 		
 		internal IProgressMonitor GetParseProgressMonitor ()
 		{
+			IProgressMonitor mon;
 			if (parseProgressMonitorFactory != null)
-				return parseProgressMonitorFactory.CreateProgressMonitor ();
+				mon = parseProgressMonitorFactory.CreateProgressMonitor ();
 			else
-				return new NullProgressMonitor ();
+				mon = new NullProgressMonitor ();
+			
+			return new AggregatedProgressMonitor (mon, new InternalProgressMonitor (this));
 		}
 			
 		internal CodeCompletionDatabase GetDatabase (string uri)
@@ -605,7 +626,7 @@ namespace MonoDevelop.Projects.Parser
 						databases [realUri] = adb;
 						if (uri != realUri)
 							databases [uri] = adb;
-						
+							
 						// Load referenced databases
 						foreach (ReferenceEntry re in db.References)
 							GetDatabase (baseDir, re.Uri);
@@ -726,12 +747,22 @@ namespace MonoDevelop.Projects.Parser
 			lock (databases)
 			{
 				db = databases [uri] as CodeCompletionDatabase;
-				if (db != null)
+				if (db != null) {
 					databases.Remove (uri);
+					lock (parseQueueLock) {
+						// Delete all pending parse jobs for this database
+						Queue newQueue = new Queue ();
+						foreach (ParsingJob pj in parseQueue)
+							if (pj.Database != db)
+								newQueue.Enqueue (pj);
+						parseQueue = newQueue;
+					}
+				}
 			}
 			if (db != null) {
 				db.Write ();
-				db.Dispose ();
+				if (!db.Disposed)
+					db.Dispose ();
 			}
 		}
 		
@@ -749,12 +780,9 @@ namespace MonoDevelop.Projects.Parser
 			lock (databases)
 			{
 				Hashtable references = new Hashtable ();
-				foreach (CodeCompletionDatabase db in databases.Values)
-				{
-					if (db is ProjectCodeCompletionDatabase) {
-						foreach (ReferenceEntry re in ((ProjectCodeCompletionDatabase)db).References)
-							references [re.Uri] = null;
-					}
+				foreach (CodeCompletionDatabase db in databases.Values) {
+					if (db is ProjectCodeCompletionDatabase)
+						CollectAssemblyReferences (db, references);
 				}
 				
 				ArrayList todel = new ArrayList ();
@@ -762,9 +790,11 @@ namespace MonoDevelop.Projects.Parser
 				{
 					if (!(en.Value is ProjectCodeCompletionDatabase) &&
 						!references.Contains (en.Key) &&
-						!loadedAssemblies.Contains (en.Key)
-					)
+						!loadedAssemblies.Contains (en.Key) &&
+						((string)en.Key) != CoreDB
+					) {
 						todel.Add (en.Key);
+					}
 				}
 				
 				foreach (string uri in todel)
@@ -772,25 +802,36 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
+		void CollectAssemblyReferences (CodeCompletionDatabase db, Hashtable references)
+		{
+			foreach (ReferenceEntry re in db.References) {
+				if (!references.Contains (re.Uri)) {
+					references.Add (re.Uri, null);
+					CodeCompletionDatabase dbc = GetDatabase (re.Uri);
+					if (dbc != null && !(dbc is ProjectCodeCompletionDatabase))
+						CollectAssemblyReferences (dbc, references);
+				}
+			}
+		}
+		
 		void LoadCombineDatabases (Combine combine)
 		{
-			CombineEntryCollection projects = combine.GetAllProjects();
-			foreach (Project entry in projects) {
-				LoadProjectDatabase (entry);
-			}
+			foreach (CombineEntry entry in combine.Entries)
+				Load (entry);
+
 			combine.EntryAdded += combineEntryAddedHandler;
 			combine.EntryRemoved += combineEntryRemovedHandler;
 		}
 		
 		void UnloadCombineDatabases (Combine combine)
 		{
-			CombineEntryCollection projects = combine.GetAllProjects();
-			foreach (Project entry in projects) {
-				UnloadProjectDatabase (entry);
-			}
-			CleanUnusedDatabases ();
+			foreach (CombineEntry entry in combine.Entries)
+				Unload (entry);
+
 			combine.EntryAdded -= combineEntryAddedHandler;
 			combine.EntryRemoved -= combineEntryRemovedHandler;
+
+			CleanUnusedDatabases ();
 		}
 		
 		void OnProjectRenamed (object sender, CombineEntryRenamedEventArgs args)
@@ -810,19 +851,12 @@ namespace MonoDevelop.Projects.Parser
 		
 		void OnCombineEntryAdded (object sender, CombineEntryEventArgs args)
 		{
-			if (args.CombineEntry is Project)
-				LoadProjectDatabase ((Project)args.CombineEntry);
-			else if (args.CombineEntry is Combine)
-				LoadCombineDatabases ((Combine)args.CombineEntry);
+			Load (args.CombineEntry);
 		}
 		
 		void OnCombineEntryRemoved (object sender, CombineEntryEventArgs args)
 		{
-			if (args.CombineEntry is Project)
-				UnloadProjectDatabase ((Project) args.CombineEntry);
-			else if (args.CombineEntry is Combine)
-				UnloadCombineDatabases ((Combine) args.CombineEntry);
-			CleanUnusedDatabases ();
+			Unload (args.CombineEntry);
 		}
 		
 		internal void NotifyReferencesChanged (CodeCompletionDatabase db)
@@ -856,12 +890,13 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
-		internal void QueueParseJob (JobCallback callback, object data)
+		internal void QueueParseJob (CodeCompletionDatabase db, JobCallback callback, object data)
 		{
 			ParsingJob job = new ParsingJob ();
 			job.ParseCallback = callback;
 			job.Data = data;
-			lock (parseQueue)
+			job.Database = db;
+			lock (parseQueueLock)
 			{
 				parseQueue.Enqueue (job);
 				parseEvent.Set ();
@@ -977,7 +1012,7 @@ namespace MonoDevelop.Projects.Parser
 					}
 					
 					ParsingJob job = null;
-					lock (parseQueue)
+					lock (parseQueueLock)
 					{
 						if (parseQueue.Count > 0)
 							job = (ParsingJob) parseQueue.Dequeue ();
@@ -993,7 +1028,7 @@ namespace MonoDevelop.Projects.Parser
 						}
 					}
 					
-					lock (parseQueue)
+					lock (parseQueueLock)
 						pending = parseQueue.Count;
 					
 				}
@@ -1003,6 +1038,41 @@ namespace MonoDevelop.Projects.Parser
 			}
 		}
 		
+		internal void StartParseOperation ()
+		{
+			if ((parseStatus++) == 0) {
+				if (ParseOperationStarted != null)
+					ParseOperationStarted (this, EventArgs.Empty);
+			}
+		}
+		
+		internal void EndParseOperation ()
+		{
+			if (parseStatus == 0)
+				return;
+
+			if (--parseStatus == 0) {
+				if (ParseOperationFinished != null)
+					ParseOperationFinished (this, EventArgs.Empty);
+			}
+		}
+		
+		public IParseInformation UpdateFile (string fileName, string fileContent)
+		{
+			Project project = null;
+			
+			lock (databases) {
+				foreach (object ob in databases.Values) {
+					ProjectCodeCompletionDatabase db = ob as ProjectCodeCompletionDatabase;
+					if (db != null) {
+						if (db.Project.IsFileInProject (fileName))
+							project = db.Project;
+					}
+				}
+			}
+			return UpdateFile (project, fileName, fileContent);
+		}
+		
 		public IParseInformation UpdateFile (Project project, string fileName, string fileContent)
 		{
 			try {
@@ -1555,6 +1625,8 @@ namespace MonoDevelop.Projects.Parser
 		public event ParseInformationEventHandler ParseInformationChanged;
 		public event ClassInformationEventHandler ClassInformationChanged;
 		public event AssemblyInformationEventHandler AssemblyInformationChanged;
+		public event EventHandler ParseOperationStarted;
+		public event EventHandler ParseOperationFinished;
 	}
 	
 	[Serializable]
@@ -1607,8 +1679,10 @@ namespace MonoDevelop.Projects.Parser
 
 		void PutBaseClassesOnStack(IClass c)
 		{
-			foreach (string baseTypeName in c.BaseTypes)
-				baseTypeQueue.Enqueue(baseTypeName);
+			foreach (IReturnType baseType in c.BaseTypes)
+			{
+				baseTypeQueue.Enqueue(baseType.FullyQualifiedName);
+			}
 		}
 
 		public IClass Current {
@@ -1658,7 +1732,24 @@ namespace MonoDevelop.Projects.Parser
 			PutBaseClassesOnStack(topLevelClass);
 			baseTypeQueue.Enqueue("System.Object");
 		}
-	}	
+	}
+	
+	class InternalProgressMonitor: NullProgressMonitor
+	{
+		ParserDatabase db;
+		
+		public InternalProgressMonitor (ParserDatabase db)
+		{
+			this.db = db;
+			db.StartParseOperation ();
+		}
+		
+		public override void Dispose ()
+		{
+			base.Dispose ();
+			db.EndParseOperation ();
+		}
+	}
 	
 	public class ClassUpdateInformation
 	{
