@@ -45,6 +45,9 @@ namespace MonoDevelop.Projects.Parser
 		[NonSerialized]
 		bool disableParse;
 		
+		[NonSerialized]
+		bool inParseQueue;
+		
 		public FileEntry (string path)
 		{
 			filePath = path;
@@ -61,6 +64,11 @@ namespace MonoDevelop.Projects.Parser
 			set { disableParse = value; }
 		}
 		
+		public bool InParseQueue {
+			get { return inParseQueue; }
+			set { inParseQueue = value; }
+		}
+		
 		public DateTime LastParseTime
 		{
 			get { return parseTime; }
