@@ -9,6 +9,7 @@ using System;
 using System.Collections;
 using System.Collections.Specialized;
 using System.Reflection;
+using MonoDevelop.Projects.Parser;
 
 namespace MonoDevelop.Projects.Parser
 {
@@ -38,7 +39,20 @@ namespace MonoDevelop.Projects.Parser
 			get;
 		}
 		
-		StringCollection BaseTypes {
+		/* Reasoning behind the 'null' isGeneric indication: since most classes
+		   are not generic, it is best not to create string collections to hold
+		   type parameters and use the 'null' value to indicate that a type is
+		   not generic */
+		/// <summary>
+		/// Contains a set of formal parameters to a generic type. 
+		/// <p>If this property returns null or an empty collection, the type is
+		/// not generic.</p>
+		/// </summary>
+		GenericParameterList GenericParameters {
+			get;
+		}
+		
+		ReturnTypeList BaseTypes {
 			get;
 		}
 		
