@@ -7,7 +7,6 @@
 
 using System.Collections;
 using System.Collections.Specialized;
-using ICSharpCode.SharpRefactory.Parser;
 
 namespace MonoDevelop.Projects.Parser
 {
@@ -28,4 +27,8 @@ namespace MonoDevelop.Projects.Parser
 			set;
 		}
 	}
+	
+	public class ErrorInfo
+	{
+	}
 }
