@@ -53,15 +53,20 @@ namespace MonoDevelop.Projects.Parser
 		IFileParserContext GetFileParserContext (string file);
 		IAssemblyParserContext GetAssemblyParserContext (string assemblyFile);
 
+		IParseInformation UpdateFile (string fileName, string fileContent);
 		IParseInformation UpdateFile (Project project, string fileName, string fileContent);
 		
 		bool TrackFileChanges { get; set; }
+		bool IsParsing { get; }
 		
 		IProgressMonitorFactory ParseProgressMonitorFactory { get; set; }
 
 		event ParseInformationEventHandler ParseInformationChanged;
 		event ClassInformationEventHandler ClassInformationChanged;
 		event AssemblyInformationEventHandler AssemblyInformationChanged;
+
+		event EventHandler ParseOperationStarted;
+		event EventHandler ParseOperationFinished;
 	}
 	
 	public interface IFileParserContext: IParserContext
@@ -79,10 +84,15 @@ namespace MonoDevelop.Projects.Parser
 
 	public interface IParserContext
 	{
+		IParserDatabase ParserDatabase { get; }
+		
 		IParseInformation ParseFile (string fileName);
 		IParseInformation ParseFile (string fileName, string fileContent);
 		IParseInformation ParseFile (ITextFile file);
 		
+		// Makes sure that all parser information is up to date.
+		void UpdateDatabase ();
+		
 		IExpressionFinder GetExpressionFinder(string fileName);
 		
 		IParseInformation GetParseInformation (string fileName);
