@@ -35,8 +35,10 @@ namespace MonoDevelop.Projects.Parser
 			cls.modifiers          = sclass.Modifiers;
 			cls.classType          = sclass.ClassType;
 
-			foreach (string t in sclass.BaseTypes)
-				cls.baseTypes.Add (typeResolver.Resolve (t));
+			foreach (IReturnType t in sclass.BaseTypes)
+			{
+				cls.baseTypes.Add (PersistentReturnType.Resolve(t, typeResolver));
+			}
 			
 			foreach (IClass c in sclass.InnerClasses) {
 				PersistentClass pc = PersistentClass.Resolve (c, typeResolver);
@@ -74,6 +76,13 @@ namespace MonoDevelop.Projects.Parser
 				cls.indexer.Add (pi);
 			}
 			
+			if (sclass.GenericParameters != null && sclass.GenericParameters.Count > 0) {
+				cls.GenericParameters = new GenericParameterList();
+				foreach (GenericParameter gp in sclass.GenericParameters) {
+					cls.GenericParameters.Add(PersistentGenericParamater.Resolve(gp, typeResolver));
+				}
+			}
+			
 			cls.region = sclass.Region;
 			cls.bodyRegion = sclass.BodyRegion;
 			cls.attributes = PersistentAttributeSectionCollection.Resolve (sclass.Attributes, typeResolver);
@@ -92,7 +101,7 @@ namespace MonoDevelop.Projects.Parser
 
 			uint count = reader.ReadUInt32();
 			for (uint i = 0; i < count; ++i) {
-				cls.baseTypes.Add (PersistentHelper.ReadString (reader, nameTable));
+				cls.baseTypes.Add (PersistentReturnType.Read (reader, nameTable));
 			}
 			
 			count = reader.ReadUInt32();
@@ -137,6 +146,17 @@ namespace MonoDevelop.Projects.Parser
 				cls.indexer.Add (ind);
 			}
 			
+			// Read the generic parameters
+			count = reader.ReadUInt32();
+			if (count > 0) {
+				cls.GenericParameters = new GenericParameterList();
+				// Add the generic parameters one by one
+				for (uint i = 0; i < count; ++i) {
+					cls.GenericParameters.Add(PersistentGenericParamater.Read(reader, nameTable));
+				}
+				// All the generic parameters have been added...
+			}
+			
 			cls.region = PersistentRegion.Read (reader, nameTable);
 			cls.bodyRegion = PersistentRegion.Read (reader, nameTable);
 			cls.attributes = PersistentAttributeSectionCollection.Read (reader, nameTable);
@@ -150,10 +170,10 @@ namespace MonoDevelop.Projects.Parser
 				
 			writer.Write((uint)cls.Modifiers);
 			writer.Write((short)cls.ClassType);
-
+				
 			writer.Write((uint)(cls.BaseTypes.Count));
-			foreach (string baseType in cls.BaseTypes)
-				PersistentHelper.WriteString (baseType, writer, nameTable);
+			foreach (IReturnType baseType in cls.BaseTypes)
+				PersistentReturnType.WriteTo(baseType, writer, nameTable);
 
 			writer.Write((uint)cls.InnerClasses.Count);
 			foreach (IClass innerClass in cls.InnerClasses) {
@@ -185,6 +205,17 @@ namespace MonoDevelop.Projects.Parser
 				PersistentIndexer.WriteTo (ind, writer, nameTable);
 			}
 			
+			// Write the generic parameters to the database file
+			if (cls.GenericParameters == null || cls.GenericParameters.Count < 1)
+				writer.Write((uint)0);
+			else {
+				writer.Write((uint)cls.GenericParameters.Count);
+				foreach (GenericParameter gp in cls.GenericParameters) {
+					PersistentGenericParamater.WriteTo(gp, writer, nameTable);
+				}
+			}
+			
+			
 			PersistentRegion.WriteTo (cls.Region, writer, nameTable);
 			PersistentRegion.WriteTo (cls.BodyRegion, writer, nameTable);
 			PersistentAttributeSectionCollection.WriteTo (cls.Attributes, writer, nameTable);
