@@ -44,8 +44,6 @@ namespace MonoDevelop.Projects
 		[ItemProperty ("StartMode/Execute", ValueType = typeof(CombineExecuteDefinition))]
 		ArrayList combineExecuteDefinitions = new ArrayList();
 		
-		bool   eventsAllowed = true;
-		
 		[ProjectPathItemProperty ("outputpath")]
 		string outputdir     = null;
 		
@@ -157,6 +155,13 @@ namespace MonoDevelop.Projects
 			entrySavedHandler = new CombineEntryEventHandler (NotifyEntrySaved);
 		}
 		
+		public override void Dispose()
+		{
+			base.Dispose ();
+			foreach (CombineEntry e in Entries)
+				e.Dispose ();
+		}
+		
 		public override IConfiguration CreateConfiguration (string name)
 		{
 			CombineConfiguration cc = new CombineConfiguration ();
@@ -187,6 +192,9 @@ namespace MonoDevelop.Projects
 			if (StartupEntry == null)
 				StartupEntry = entry;
 			
+			// If the combine has no configurations, copy them from the
+			// new child entry
+			
 			if (Configurations.Count == 0) {
 				foreach (IConfiguration pconf in entry.Configurations) {
 					if (pconf.Name == null)
@@ -198,19 +206,27 @@ namespace MonoDevelop.Projects
 				}
 			}
 		
-			foreach (CombineConfiguration conf in Configurations)
-				conf.AddEntry (entry);						
+			foreach (CombineConfiguration conf in Configurations) {
+				// Register the new entry in every combine configuration
+				CombineConfigurationEntry centry = conf.AddEntry (entry);
+				
+				// Look for a valid configuration in the child entry to be bound
+				// to the combine configuration. Look first for a configuration
+				// with the same name. Use the active configuration if not found.
+				if (entry.GetConfiguration (conf.Name) != null)
+					centry.ConfigurationName = conf.Name;
+				else if (entry.ActiveConfiguration != null)
+					centry.ConfigurationName = entry.ActiveConfiguration.Name;
+			}
+			
+			// If the child is a combine and has no configurations, copy them
+			// from the parent combine
 			
 			if (entry.Configurations.Count == 0 && entry is Combine)			
 				foreach (CombineConfiguration conf in Configurations)					
 					entry.Configurations.Add(new CombineConfiguration(conf.Name));			
 			
 			combineExecuteDefinitions.Add (new CombineExecuteDefinition (entry, EntryExecuteType.None));
-			
-			if (eventsAllowed) {
-				NotifyModified ();
-				OnEntryAdded (new CombineEntryEventArgs (entry));
-			}
 
 			if (entry is Project)
 			{
@@ -236,6 +252,9 @@ namespace MonoDevelop.Projects
 			}
 			entry.Modified += entryModifiedHandler;
 			entry.Saved += entrySavedHandler;
+			
+			NotifyModified ();
+			OnEntryAdded (new CombineEntryEventArgs (entry));
 		}
 		
 		public override DataCollection Serialize (ITypeSerializer handler)
@@ -490,6 +509,15 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		public CombineEntryCollection GetAllBuildableEntries (string configuration, bool topologicalSort)
+		{
+			CombineEntryCollection list = GetAllBuildableEntries (configuration);
+			if (topologicalSort)
+				return TopologicalSort (list);
+			else
+				return list;
+		}
+		
 		public CombineEntryCollection GetAllBuildableEntries (string configuration)
 		{
 			CombineEntryCollection list = new CombineEntryCollection();
@@ -513,7 +541,7 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
-		public Project GetProjectEntryContaining (string fileName) 
+		public Project GetProjectContainingFile (string fileName) 
 		{
 			CombineEntryCollection projects = GetAllProjects ();
 			foreach (Project projectEntry in projects) {
@@ -608,9 +636,10 @@ namespace MonoDevelop.Projects
 
 					ICompilerResult res = entry.Build (monitor, false);
 					builds++;
-					cres.Errors.AddRange (res.CompilerResults.Errors);
+					if (res != null)
+						cres.Errors.AddRange (res.CompilerResults.Errors);
 					monitor.Step (1);
-					if (res.ErrorCount > 0) {
+					if (res != null && res.ErrorCount > 0) {
 						failedBuilds++;
 						break;
 					}
@@ -702,93 +731,6 @@ namespace MonoDevelop.Projects
 		}
 
 		
-		public void GenerateMakefiles ()
-		{
-			GenerateMakefiles (null);
-		}
-
-		public override void GenerateMakefiles (Combine parentCombine)
-		{
-			CombineEntryCollection allProjects = TopologicalSort (GetAllProjects ());
-			ArrayList projects = new ArrayList ();
-			foreach (CombineEntry entry in allProjects) {
-				if (entry is Project) {
-					entry.GenerateMakefiles (this);
-					projects.Add ((Project)entry);
-				}
-				else
-					Runtime.LoggingService.ErrorFormat ("Dont know how to generate makefiles for {0}", entry);
-			}
-			
-			string rel_outputdir = Runtime.FileUtilityService.AbsoluteToRelativePath (BaseDirectory, outputdir);
-			
-			StreamWriter buildstream = new StreamWriter (Path.Combine (BaseDirectory, "make.sh"));
-			buildstream.WriteLine ("#!/bin/sh");
-			buildstream.WriteLine ("# This file is autogenerated by MonoDevelop");
-			buildstream.WriteLine ("# Do not edit it.");
-			buildstream.WriteLine ();
-			buildstream.WriteLine ("make -f Makefile.solution.{0} \"$@\"", Name.Replace (" ",""));
-			buildstream.Flush ();
-			buildstream.Close ();
-			
-			Syscall.chmod (Path.Combine (BaseDirectory, "make.sh"), FilePermissions.S_IRUSR | FilePermissions.S_IWUSR | FilePermissions.S_IXUSR | FilePermissions.S_IRGRP | FilePermissions.S_IWGRP | FilePermissions.S_IROTH);
-
-			StreamWriter stream = new StreamWriter (Path.Combine (BaseDirectory, "Makefile.solution." + Name.Replace (" ", "")));
-			stream.WriteLine ("# This file is autogenerated by MonoDevelop");
-			stream.WriteLine ("# Do not edit it.");
-			stream.WriteLine ();
-
-			stream.WriteLine ("RUNTIME := mono");
-			stream.WriteLine ();
-			stream.WriteLine ("OUTPUTDIR := {0}", rel_outputdir);
-			stream.WriteLine ();
-			stream.Write ("all: depcheck __init ");
-			foreach (Project proj in projects) {
-				stream.Write ("Makefile.{0}.all ", proj.Name.Replace (" ",""));
-			}
-			stream.WriteLine ();
-			stream.WriteLine ();
-
-			stream.WriteLine ("__init:");
-			stream.WriteLine ("\tmkdir -p $(OUTPUTDIR)");
-			stream.WriteLine ();
-
-			stream.Write ("clean: ");
-			foreach (Project proj in projects) {
-				stream.Write ("Makefile.{0}.clean ", proj.Name.Replace (" ", ""));
-			}
-			stream.WriteLine ();
-			stream.WriteLine ();
-
-			stream.Write ("depcheck: ");
-			foreach (Project proj in projects) {
-				stream.Write ("Makefile.{0}.depcheck ", proj.Name.Replace (" ", ""));
-			}
-			stream.WriteLine ();
-			stream.WriteLine ();
-
-			stream.WriteLine ("run: all");
-			if (!SingleStartupProject) {
-				stream.WriteLine ("\t@echo `run'ning multiple startup projects is not yet support");
-			} else {
-				if (StartupEntry != null)
-					stream.WriteLine ("\tcd $(OUTPUTDIR) && $(RUNTIME) {0}", ((Project)StartupEntry).GetOutputFileName ());
-				else
-					stream.WriteLine ("\t@echo No startup project defined");
-			}
-			stream.WriteLine ();
-
-			foreach (Project proj in projects) {
-				string relativeLocation = Runtime.FileUtilityService.AbsoluteToRelativePath (BaseDirectory, proj.BaseDirectory);
-				stream.WriteLine ("Makefile.{0}.%:", proj.Name.Replace (" ", ""));
-				stream.WriteLine ("\t@cd {0} && $(MAKE) -f $(subst .$*,,$@) $*", relativeLocation);
-				stream.WriteLine ();
-			}
-
-			stream.Flush ();
-			stream.Close ();
-		}
-		
 		internal void NotifyFileRemovedFromProject (object sender, ProjectFileEventArgs e)
 		{
 			OnFileRemovedFromProject (e);
@@ -929,53 +871,4 @@ namespace MonoDevelop.Projects
 		public event CombineEntryEventHandler EntryModified;
 		public event CombineEntryEventHandler EntrySaved;
 	}
-	
-	public class CombineActiveConfigurationTypeConverter : TypeConverter
-	{
-		public override bool CanConvertFrom(ITypeDescriptorContext context,Type sourceType)
-		{
-			return true;
-		}
-		
-		public override  bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
-		{
-			return true;
-		}
-		
-		public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture,  object value)
-		{
-			Combine combine = (Combine)context.Instance;
-			foreach (IConfiguration configuration in combine.Configurations) {
-				if (configuration.Name == value.ToString()) {
-					return configuration;
-				}
-			}
-			return null;
-		}
-		
-		public override object ConvertTo(ITypeDescriptorContext context, CultureInfo culture, object value, Type destinationType)
-		{
-			IConfiguration config = value as IConfiguration;
-			Debug.Assert(config != null, String.Format("Tried to convert {0} to IConfiguration", config));
-			if (config != null) {
-				return config.Name;
-			}
-			return String.Empty;
-		}
-		
-		public override bool GetStandardValuesSupported(ITypeDescriptorContext context)
-		{
-			return true;
-		}
-		
-		public override bool GetStandardValuesExclusive(ITypeDescriptorContext context)
-		{
-			return true;
-		}
-		
-		public override System.ComponentModel.TypeConverter.StandardValuesCollection GetStandardValues(System.ComponentModel.ITypeDescriptorContext context)
-		{
-			return new TypeConverter.StandardValuesCollection(((Combine)context.Instance).Configurations);
-		}
-	}
 }
