@@ -17,9 +17,11 @@ using MonoDevelop.Core.Properties;
 using MonoDevelop.Core;
 using MonoDevelop.Projects;
 using MonoDevelop.Projects.Serialization;
+using MonoDevelop.Projects.Deployment;
 
 namespace MonoDevelop.Projects
 {
+	[DataItem (FallbackType = typeof(UnknownCombineEntry))]
 	public abstract class CombineEntry : ICustomDataItem, IDisposable, IExtendedDataItem
 	{
 		ConfigurationCollection configurations;
@@ -33,6 +35,11 @@ namespace MonoDevelop.Projects
 		
 		IFileFormat fileFormat;
 		
+		[ItemProperty]
+		string defaultDeployTarget;
+		
+		DeployTargetCollection deployTargets;
+		
 		public CombineEntry ()
 		{
 			configurations = new ConfigurationCollection ();
@@ -45,7 +52,7 @@ namespace MonoDevelop.Projects
 		{
 		}
 		
-		IDictionary IExtendedDataItem.ExtendedProperties {
+		public IDictionary ExtendedProperties {
 			get {
 				if (extendedProperties == null)
 					extendedProperties = new Hashtable ();
@@ -53,6 +60,18 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		[ItemProperty ("releaseversion", DefaultValue = "0.1")]
+		string release_version;
+
+		public string Version {
+			get {
+				return release_version;
+			}
+			set {
+				release_version = value;
+			}
+		}
+		
 		[ItemProperty ("name")]
 		public virtual string Name {
 			get {
@@ -163,6 +182,34 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		public DeployTarget DefaultDeployTarget {
+			get {
+				if (defaultDeployTarget == null)
+					return null;
+				if (deployTargets == null)
+					return null;
+				foreach (DeployTarget dt in deployTargets)
+					if (dt.Name == defaultDeployTarget)
+						return dt;
+				return null;
+			}
+			set {
+				if (value != null)
+					defaultDeployTarget = value.Name;
+				else
+					defaultDeployTarget = null;
+			}
+		}
+		
+		[ItemProperty]
+		public DeployTargetCollection DeployTargets {
+			get { 
+				if (deployTargets == null)
+					deployTargets = new DeployTargetCollection ();
+				return deployTargets;
+			}
+		}
+		
 		public virtual DataCollection Serialize (ITypeSerializer handler)
 		{
 			DataCollection data = handler.Serialize (this);
@@ -185,6 +232,10 @@ namespace MonoDevelop.Projects
 			handler.Deserialize (this, data);
 			if (ac != null)
 				activeConfiguration = GetConfiguration (ac.Value);
+				
+			if (deployTargets != null)
+				foreach (DeployTarget target in deployTargets)
+					target.SetCombineEntry (this);
 		}
 		
 		public abstract IConfiguration CreateConfiguration (string name);
@@ -297,10 +348,6 @@ namespace MonoDevelop.Projects
 		public abstract void Execute (IProgressMonitor monitor, ExecutionContext context);
 		public abstract bool NeedsBuilding { get; set; }
 		
-		public virtual void GenerateMakefiles (Combine parentCombine)
-		{
-		}
-		
 		public event CombineEntryRenamedEventHandler NameChanged;
 		public event ConfigurationEventHandler ActiveConfigurationChanged;
 		public event ConfigurationEventHandler ConfigurationAdded;
