@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.Collections;
 using System.IO;
 using MonoDevelop.Projects.Serialization;
 
@@ -59,7 +60,7 @@ namespace MonoDevelop.Projects
 		[ItemProperty ("Build/target")]
 		CompileTarget compiletarget = CompileTarget.Exe;
 		
-		[ItemProperty ("CodeGeneration")]
+		[ItemProperty ("CodeGeneration", FallbackType=typeof(UnknownCompilationParameters))]
 		ICloneable compilationParameters;
 		
 		string sourcePath;
@@ -74,7 +75,7 @@ namespace MonoDevelop.Projects
 			set { netRuntime = value; }
 		}
 
-		public CompileTarget CompileTarget {
+		public virtual CompileTarget CompileTarget {
 			get { return compiletarget; }
 			set { compiletarget = value; }
 		}
@@ -111,4 +112,18 @@ namespace MonoDevelop.Projects
 			compilationParameters = conf.compilationParameters != null ? (ICloneable)conf.compilationParameters.Clone () : null;
 		}
 	}
+	
+	public class UnknownCompilationParameters: ICloneable, IExtendedDataItem
+	{
+		Hashtable table = new Hashtable ();
+		
+		public IDictionary ExtendedProperties { 
+			get { return table; }
+		}
+		
+		public object Clone ()
+		{
+			return MemberwiseClone (); 
+		}
+	}
 }
