@@ -19,6 +19,7 @@ using MonoDevelop.Core;
 using MonoDevelop.Core.Execution;
 using MonoDevelop.Core.Properties;
 using MonoDevelop.Core.AddIns;
+using MonoDevelop.Projects.Ambience;
 using MonoDevelop.Projects.Serialization;
 
 namespace MonoDevelop.Projects
@@ -32,8 +33,8 @@ namespace MonoDevelop.Projects
 	/// <summary>
 	/// External language bindings must extend this class
 	/// </summary>
-	[DataItemAttribute ("Project")]
 	[DataInclude (typeof(ProjectFile))]
+	[DataItem (FallbackType=typeof(UnknownProject))]
 	public abstract class Project : CombineEntry
 	{
 		[ItemProperty ("Description", DefaultValue="")]
@@ -49,13 +50,7 @@ namespace MonoDevelop.Projects
 
 		protected ProjectReferenceCollection projectReferences;
 		
-		[ItemProperty ("DeploymentInformation")]
-		protected DeployInformation deployInformation = new DeployInformation();
-		
-		IBuildStep[] buildPipeline;
-		
 		bool isDirty = false;
-		bool filesChecked;
 		
 		private FileSystemWatcher projectFileWatcher;
 		
@@ -125,10 +120,14 @@ namespace MonoDevelop.Projects
 			get;
 		}
 		
+		public virtual MonoDevelop.Projects.Ambience.Ambience Ambience {
+			get { return Services.Ambience.GenericAmbience; }
+		}
+		
 		[Browsable(false)]
-		public DeployInformation DeployInformation {
+		public virtual string [] SupportedLanguages {
 			get {
-				return deployInformation;
+				return new String [] { "" };
 			}
 		}
 
@@ -203,16 +202,17 @@ namespace MonoDevelop.Projects
 		{
 			foreach (ProjectReference projectReference in ProjectReferences) {
 				if ((projectReference.LocalCopy || force) && projectReference.ReferenceType != ReferenceType.Gac) {
-					string referenceFileName   = projectReference.GetReferencedFileName();
-					string destinationFileName = Path.Combine (destPath, Path.GetFileName (referenceFileName));
-					try {
-						if (destinationFileName != referenceFileName) {
-							File.Copy(referenceFileName, destinationFileName, true);
-							if (File.Exists (referenceFileName + ".mdb"))
-								File.Copy (referenceFileName + ".mdb", destinationFileName + ".mdb", true);
+					foreach (string referenceFileName in projectReference.GetReferencedFileNames ()) {
+						string destinationFileName = Path.Combine (destPath, Path.GetFileName (referenceFileName));
+						try {
+							if (destinationFileName != referenceFileName) {
+								File.Copy(referenceFileName, destinationFileName, true);
+								if (File.Exists (referenceFileName + ".mdb"))
+									File.Copy (referenceFileName + ".mdb", destinationFileName + ".mdb", true);
+							}
+						} catch (Exception e) {
+							Runtime.LoggingService.ErrorFormat ("Can't copy reference file from {0} to {1}: {2}", referenceFileName, destinationFileName, e);
 						}
-					} catch (Exception e) {
-						Runtime.LoggingService.ErrorFormat ("Can't copy reference file from {0} to {1} reason {2}", referenceFileName, destinationFileName, e);
 					}
 				}
 				if (projectReference.ReferenceType == ReferenceType.Project && RootCombine != null) {
@@ -222,6 +222,30 @@ namespace MonoDevelop.Projects
 			}
 		}
 		
+		void CleanReferencesInOutputPath (string destPath)
+		{
+			foreach (ProjectReference projectReference in ProjectReferences) {
+				if (projectReference.ReferenceType != ReferenceType.Gac) {
+					foreach (string referenceFileName in projectReference.GetReferencedFileNames ()) {
+						string destinationFileName = Path.Combine (destPath, Path.GetFileName (referenceFileName));
+						try {
+							if (destinationFileName != referenceFileName) {
+								File.Delete (destinationFileName);
+								if (File.Exists (destinationFileName + ".mdb"))
+									File.Delete (destinationFileName + ".mdb");
+							}
+						} catch (Exception e) {
+							Runtime.LoggingService.ErrorFormat ("Can't delete reference file {0}: {2}", destinationFileName, e);
+						}
+					}
+				}
+				if (projectReference.ReferenceType == ReferenceType.Project && RootCombine != null) {
+					Project p = RootCombine.FindProject (projectReference.Reference);
+					p.CleanReferencesInOutputPath (destPath);
+				}
+			}
+		}
+		
 		public override void Dispose()
 		{
 			base.Dispose ();
@@ -262,9 +286,20 @@ namespace MonoDevelop.Projects
 		public override void Clean ()
 		{
 			isDirty = true;
+			
+			// Delete the generated assembly
 			string file = GetOutputFileName ();
-			if (file != null && File.Exists (file))
-				File.Delete (file);
+			if (file != null) {
+				if (File.Exists (file))
+					File.Delete (file);
+				if (File.Exists (file + ".mdb"))
+					File.Delete (file + ".mdb");
+			}
+
+			// Delete referenced assemblies
+			AbstractProjectConfiguration config = ActiveConfiguration as AbstractProjectConfiguration;
+			if (config != null)
+				CleanReferencesInOutputPath (config.OutputDirectory);
 		}
 		
 		public override ICompilerResult Build (IProgressMonitor monitor)
@@ -314,7 +349,7 @@ namespace MonoDevelop.Projects
 				
 				ICompilerResult res = null;
 				
-				foreach (IBuildStep step in BuildPipeline) {
+				foreach (IBuildStep step in steps) {
 					ICompilerResult sres = step.Build (monitor, this);
 					if (sres != null) {
 						if (res != null) {
@@ -330,10 +365,12 @@ namespace MonoDevelop.Projects
 				
 				isDirty = false;
 				
-				string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", res.ErrorCount, res.ErrorCount);
-				string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", res.WarningCount, res.WarningCount);
+				if (res != null) {
+					string errorString = GettextCatalog.GetPluralString("{0} error", "{0} errors", res.ErrorCount, res.ErrorCount);
+					string warningString = GettextCatalog.GetPluralString("{0} warning", "{0} warnings", res.WarningCount, res.WarningCount);
 				
-				monitor.Log.WriteLine(GettextCatalog.GetString("Build complete -- ") + errorString + ", " + warningString);
+					monitor.Log.WriteLine(GettextCatalog.GetString("Build complete -- ") + errorString + ", " + warningString);
+				}
 				
 				return res;
 			} finally {
@@ -343,9 +380,7 @@ namespace MonoDevelop.Projects
 		
 		protected virtual IBuildStep[] BuildPipeline {
 			get {
-				if (buildPipeline == null)
-					buildPipeline = (IBuildStep[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/BuildPipeline", typeof(IBuildStep));
-				return buildPipeline;
+				return (IBuildStep[]) Runtime.AddInService.GetTreeItems ("/SharpDevelop/Workbench/BuildPipeline", typeof(IBuildStep));
 			}
 		}
 		
@@ -481,19 +516,15 @@ namespace MonoDevelop.Projects
 				return;
 			}
 			
-			if (!filesChecked) {
-				foreach (ProjectFile file in ProjectFiles) {
-					if (file.BuildAction == BuildAction.Exclude) continue;
-					FileInfo finfo = new FileInfo (file.FilePath);
-					if (finfo.Exists && finfo.LastWriteTime > tim) {
-						isDirty = true;
-						return;
-					}
+			foreach (ProjectFile file in ProjectFiles) {
+				if (file.BuildAction == BuildAction.Exclude) continue;
+				FileInfo finfo = new FileInfo (file.FilePath);
+				if (finfo.Exists && finfo.LastWriteTime > tim) {
+					isDirty = true;
+					return;
 				}
-				
-				filesChecked = true;
 			}
-
+			
 			foreach (ProjectReference pref in ProjectReferences) {
 				if (pref.ReferenceType == ReferenceType.Project && RootCombine != null) {
 					Project rp = RootCombine.FindProject (pref.Reference);
@@ -503,7 +534,6 @@ namespace MonoDevelop.Projects
 					}
 				}
 			}
-			
 		}
 		
 		protected virtual DateTime GetLastBuildTime ()
@@ -526,7 +556,13 @@ namespace MonoDevelop.Projects
 			ProjectFile file = GetProjectFile (e.FullPath);
 			if (file != null) {
 				isDirty = true;
-				NotifyFileChangedInProject (file);
+				try {
+					NotifyFileChangedInProject (file);
+				} catch {
+					// Workaround Mono bug. The watcher seems to
+					// stop watching if an exception is thrown in
+					// the event handler
+				}
 			}
 
 		}
@@ -677,4 +713,16 @@ namespace MonoDevelop.Projects
 			}
 		}
 	}
+	
+	public class UnknownProject: Project
+	{
+		public override string ProjectType {
+			get { return ""; }
+		}
+
+		public override IConfiguration CreateConfiguration (string name)
+		{
+			return null;
+		}
+	}
 }
