@@ -17,19 +17,14 @@ namespace MonoDevelop.Projects
 {
 	public enum Subtype {
 		Code,
-		Directory,
-		WinForm,
-		WebForm,
-		XmlForm,
-		WebService,		
-		WebReferences,
-		Dataset
+		Directory
 	}
 	
 	public enum BuildAction {
 		Nothing,
 		Compile,
 		EmbedAsResource,
+		FileCopy,
 		Exclude		
 	}
 	
