@@ -2,6 +2,7 @@ using MonoDevelop.Core;
 using MonoDevelop.Projects.Ambience;
 using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects.Documentation;
+using MonoDevelop.Projects.Deployment;
 
 namespace MonoDevelop.Projects
 {
@@ -12,7 +13,16 @@ namespace MonoDevelop.Projects
 		static LanguageBindingService languageBindingService;
 		static IParserService parserService;
 		static IDocumentationService documentationService;
+		static DeployService deployService;
 
+		public static DeployService DeployService {
+			get {
+				if (deployService == null)
+					deployService = (DeployService) ServiceManager.GetService (typeof(DeployService));
+				return deployService;
+			}
+		}
+	
 		public static AmbienceService Ambience {
 			get {
 				if (ambienceService == null)
