@@ -11,7 +11,12 @@ namespace MonoDevelop.Startup
 		public static int Main (string[] args)
 		{
 			Runtime.Initialize ();
-			return Runtime.AddInService.StartApplication ("IDE", args);
+			Runtime.AddInService.CheckAssemblyLoadConflicts = true;
+			try {
+				return Runtime.AddInService.StartApplication ("IDE", args);
+			} finally {
+				Runtime.Shutdown ();
+			}
 		}
 	}
 }
