@@ -3,15 +3,15 @@
 using System;
 using System.Collections;
 
-using ICSharpCode.SharpRefactory.Parser;
-using ICSharpCode.SharpRefactory.Parser.AST;
+using ICSharpCode.NRefactory.Parser;
+using ICSharpCode.NRefactory.Parser.AST;
 using CSharpBinding.Parser.SharpDevelopTree;
 
 using MonoDevelop.Projects.Parser;
 
 namespace CSharpBinding.Parser
 {
-	internal class LanguageItemVisitor : AbstractASTVisitor
+	internal class LanguageItemVisitor : AbstractAstVisitor
 	{
 		Resolver resolver;
 		
@@ -61,7 +61,8 @@ namespace CSharpBinding.Parser
 				ArrayList methods = resolver.SearchMethod(type, id);
 				resolver.ShowStatic = false;
 				if (methods.Count <= 0) {
-					return null;
+					// It may be a call to a constructor
+					return resolver.SearchType (id, resolver.CompilationUnit);
 				}
 				// TODO: Find the right method
 				return methods[0];
@@ -72,7 +73,7 @@ namespace CSharpBinding.Parser
 				return null;
 			}
 			IClass c = resolver.SearchType(t.FullyQualifiedName, resolver.CompilationUnit);
-			if (c.ClassType == ClassType.Delegate) {
+			if (c.ClassType == MonoDevelop.Projects.Parser.ClassType.Delegate) {
 				ArrayList methods = resolver.SearchMethod(t, "invoke");
 				if (methods.Count <= 0) {
 					return null;
@@ -118,7 +119,7 @@ namespace CSharpBinding.Parser
 		
 		public override object Visit(PointerReferenceExpression pointerReferenceExpression, object data)
 		{
-			ReturnType type = pointerReferenceExpression.Expression.AcceptVisitor(this, data) as ReturnType;
+			ReturnType type = pointerReferenceExpression.TargetObject.AcceptVisitor(this, data) as ReturnType;
 			if (type == null) {
 				return null;
 			}
@@ -194,20 +195,26 @@ namespace CSharpBinding.Parser
 		
 		public override object Visit(IndexerExpression indexerExpression, object data)
 		{
-			IReturnType type = (IReturnType)indexerExpression.TargetObject.AcceptVisitor(this, data);
-			if (type == null) {
+			object ob = indexerExpression.TargetObject.AcceptVisitor(this, data);
+			IReturnType type = ob as IReturnType;
+			if (type != null) {
+				if (type.ArrayDimensions == null || type.ArrayDimensions.Length == 0) {
+					if (indexerExpression.TargetObject is ThisReferenceExpression) {
+						return null;
+					}
+					ArrayList indexer = resolver.SearchIndexer(type);
+					if (indexer.Count == 0) {
+						return null;
+					}
+					// TODO: get the right indexer
+					return indexer[0];
+				}
 				return null;
 			}
-			if (type.ArrayDimensions == null || type.ArrayDimensions.Length == 0) {
-				if (indexerExpression.TargetObject is ThisReferenceExpression) {
-					return null;
-				}
-				ArrayList indexer = resolver.SearchIndexer(type);
-				if (indexer.Count == 0) {
-					return null;
-				}
-				// TODO: get the right indexer
-				return indexer[0];
+			
+			IClass cls = ob as IClass;
+			if (cls != null) {
+				return ReturnType.Convert (cls);
 			}
 			
 			return null;
@@ -232,12 +239,12 @@ namespace CSharpBinding.Parser
 		
 		public override object Visit(ObjectCreateExpression objectCreateExpression, object data)
 		{
-			return null;
+			return resolver.SearchType (objectCreateExpression.CreateType.SystemType, resolver.CompilationUnit);
 		}
 		
 		public override object Visit(ArrayCreateExpression arrayCreateExpression, object data)
 		{
-			return null;
+			return resolver.SearchType (arrayCreateExpression.CreateType.SystemType, resolver.CompilationUnit);
 		}
 		
 		public override object Visit(DirectionExpression directionExpression, object data)
