@@ -6,6 +6,7 @@
 // </file>
 
 using System;
+using System.IO;
 using System.Collections;
 using System.Drawing;
 
@@ -13,15 +14,17 @@ using MonoDevelop.Core;
 using MonoDevelop.Projects.Parser;
 using MonoDevelop.Projects;
 using CSharpBinding.Parser.SharpDevelopTree;
-using ICSharpCode.SharpRefactory.Parser.AST;
-using ICSharpCode.SharpRefactory.Parser;
+using ICSharpCode.NRefactory.Parser.AST;
+using ICSharpCode.NRefactory.Parser;
+using ClassType = MonoDevelop.Projects.Parser.ClassType;
 
 namespace CSharpBinding.Parser
 {
 	class Resolver
 	{
 		IParserContext parserContext;
-		ICompilationUnit cu;
+		ICompilationUnit currentUnit;
+		string currentFile;
 		IClass callingClass;
 		LookupTableVisitor lookupTableVisitor;
 		
@@ -38,7 +41,7 @@ namespace CSharpBinding.Parser
 		
 		public ICompilationUnit CompilationUnit {
 			get {
-				return cu;
+				return currentUnit;
 			}
 		}
 		
@@ -79,9 +82,9 @@ namespace CSharpBinding.Parser
 			this.caretColumn   = caretColumn;
 			
 			IParseInformation parseInfo = parserContext.GetParseInformation(fileName);
-			ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+			ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
 			if (fileCompilationUnit == null) {
-//				ICSharpCode.SharpRefactory.Parser.Parser fileParser = new ICSharpCode.SharpRefactory.Parser.Parser();
+//				ICSharpCode.NRefactory.Parser.Parser fileParser = new ICSharpCode.NRefactory.Parser.Parser();
 //				fileParser.Parse(new Lexer(new StringReader(fileContent)));
 				Console.WriteLine("Warning: no parse information!");
 				return null;
@@ -92,7 +95,7 @@ namespace CSharpBinding.Parser
 			!!is so!! don't change things that work
 			Expression expr=null;	// tentative expression
 			Lexer l=null;
-			ICSharpCode.SharpRefactory.Parser.Parser p = new ICSharpCode.SharpRefactory.Parser.Parser();
+			ICSharpCode.NRefactory.Parser.Parser p = new ICSharpCode.NRefactory.Parser.Parser();
 			while (expression.Length > 0) {
 				l = new Lexer(new StringReader(expression));
 				expr = p.ParseExpression(l);
@@ -112,20 +115,21 @@ namespace CSharpBinding.Parser
 			//// here last subexpression should be fixed in expr
 			if it should be changed in expressionfinder don't fix it here
 			*/
-			ICSharpCode.SharpRefactory.Parser.Parser p = new ICSharpCode.SharpRefactory.Parser.Parser();
-			Lexer l = new Lexer(new StringReader(expression));
-			Expression expr = p.ParseExpression(l);
+			ICSharpCode.NRefactory.Parser.IParser p = ICSharpCode.NRefactory.Parser.ParserFactory.CreateParser (SupportedLanguage.CSharp, new StringReader(expression));
+			Expression expr = p.ParseExpression();
 			if (expr == null) {
 				return null;
 			}
-			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor = new LookupTableVisitor (StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit(fileCompilationUnit, null);
 			
 			TypeVisitor typeVisitor = new TypeVisitor(this);
 			
 			CSharpVisitor cSharpVisitor = new CSharpVisitor();
-			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
-			if (cu != null) {
+			currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			currentFile = fileName;
+			
+			if (currentUnit != null) {
 				callingClass = GetInnermostClass();
 //				Console.WriteLine("CallingClass is " + callingClass == null ? "null" : callingClass.Name);
 			}
@@ -141,10 +145,10 @@ namespace CSharpBinding.Parser
 				}
 				//// when type is null might be file needs to be reparsed - some vars were lost
 				fileCompilationUnit = parserContext.ParseFile (fileName, fileContent).MostRecentCompilationUnit.Tag 
-					as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+					as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
 				lookupTableVisitor.Visit(fileCompilationUnit,null);
-				cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
-				if (cu != null) {
+				currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+				if (currentUnit != null) {
 					callingClass = GetInnermostClass();
 				}
 				type=expr.AcceptVisitor(typeVisitor,null) as IReturnType;
@@ -157,22 +161,23 @@ namespace CSharpBinding.Parser
 			return type;
 		}
 		
-		public IClass ResolveExpressionType (ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit, Expression expr, int line, int col)
+		public IClass ResolveExpressionType (ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit, Expression expr, int line, int col)
 		{
 			CSharpVisitor cSharpVisitor = new CSharpVisitor();
-			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			currentFile = null;
 			
 			this.caretLine = line;
 			this.caretColumn = col;
 			
 			callingClass = GetInnermostClass();
 			
-			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor = new LookupTableVisitor (StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit (fileCompilationUnit, null);
 			TypeVisitor typeVisitor = new TypeVisitor (this);
 			IReturnType type = expr.AcceptVisitor (typeVisitor, null) as IReturnType;
 			if (type != null)
-				return SearchType (type.FullyQualifiedName, cu);
+				return SearchType (type.FullyQualifiedName, currentUnit);
 			else
 				return null;
 		}
@@ -180,25 +185,29 @@ namespace CSharpBinding.Parser
 		public ILanguageItem ResolveIdentifier (IParserContext parserContext, string id, int line, int col, string fileName, string fileContent)
 		{
 			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
-			ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+			ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
+			currentFile = fileName;
+			if (fileCompilationUnit == null)
+				return null;
 			return ResolveIdentifier (fileCompilationUnit, id, line, col);
 		}
 		
-		public ILanguageItem ResolveIdentifier (ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit, string id, int line, int col)
+		public ILanguageItem ResolveIdentifier (ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit, string id, int line, int col)
 		{
-			ICSharpCode.SharpRefactory.Parser.Parser p = new ICSharpCode.SharpRefactory.Parser.Parser();
-			Lexer l = new Lexer(new StringReader(id));
-			Expression expr = p.ParseExpression(l);
+			ICSharpCode.NRefactory.Parser.IParser p = ICSharpCode.NRefactory.Parser.ParserFactory.CreateParser (SupportedLanguage.CSharp, new StringReader(id));
+			Expression expr = p.ParseExpression ();
+			if (expr == null)
+				return null;
 			
 			CSharpVisitor cSharpVisitor = new CSharpVisitor();
-			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
 			
 			this.caretLine = line;
 			this.caretColumn = col;
 			
 			callingClass = GetInnermostClass();
 			
-			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor = new LookupTableVisitor(StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit (fileCompilationUnit, null);
 			
 			LanguageItemVisitor itemVisitor = new LanguageItemVisitor (this);
@@ -255,10 +264,13 @@ namespace CSharpBinding.Parser
 			
 			//Console.WriteLine("Not in Using");
 			IReturnType type = internalResolve (expression, caretLineNumber, caretColumn, fileName, fileContent);
-			IClass returnClass = SearchType (type.FullyQualifiedName, cu);
+			if (type == null)
+				return null;
+
+			IClass returnClass = SearchType (type.FullyQualifiedName, currentUnit);
 			if (returnClass == null) {
 				// Try if type is Namespace:
-				string n = SearchNamespace(type.FullyQualifiedName, cu);
+				string n = SearchNamespace(type.FullyQualifiedName, currentUnit);
 				if (n == null) {
 					return null;
 				}
@@ -330,8 +342,8 @@ namespace CSharpBinding.Parser
 			}
 //			Console.WriteLine("ClassType = " + curType.ClassType);
 			if (curType.ClassType == ClassType.Interface && !showStatic) {
-				foreach (string s in curType.BaseTypes) {
-					IClass baseClass = parserContext.GetClass (s, true, true);
+				foreach (IReturnType s in curType.BaseTypes) {
+					IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, true, true);
 					if (baseClass != null && baseClass.ClassType == ClassType.Interface) {
 						ListMembers(members, baseClass);
 					}
@@ -349,8 +361,8 @@ namespace CSharpBinding.Parser
 		
 		public IClass BaseClass(IClass curClass)
 		{
-			foreach (string s in curClass.BaseTypes) {
-				IClass baseClass = parserContext.GetClass (s, true, true);
+			foreach (IReturnType s in curClass.BaseTypes) {
+				IClass baseClass = parserContext.GetClass (s.FullyQualifiedName, true, true);
 				if (baseClass != null && baseClass.ClassType != ClassType.Interface) {
 					return baseClass;
 				}
@@ -507,7 +519,7 @@ namespace CSharpBinding.Parser
 			if (type == null || memberName == null || memberName == "")
 				return false;
 			
-			curType = SearchType (type.FullyQualifiedName, cu);
+			curType = SearchType (type.FullyQualifiedName, currentUnit);
 			if (curType == null)
 				return false;
 
@@ -564,8 +576,8 @@ namespace CSharpBinding.Parser
 					}
 				}
 			}
-			foreach (string baseType in curType.BaseTypes) {
-				IClass c = parserContext.GetClass (baseType, true, true);
+			foreach (IReturnType baseType in curType.BaseTypes) {
+				IClass c = parserContext.GetClass (baseType.FullyQualifiedName, true, true);
 				if (c != null)
 					return SearchClassMember (new ReturnType(c.FullyQualifiedName), memberName, includeMethods, out curType, out member);
 			}
@@ -596,37 +608,22 @@ namespace CSharpBinding.Parser
 			return between.Y < end.Y || between.X <= end.X;
 		}
 		
-		ReturnType SearchVariable(string name)
+		LocalVariable SearchVariable (string name)
 		{
-//			Console.WriteLine("Searching Variable");
-//			
-//			Console.WriteLine("LookUpTable has {0} entries", lookupTableVisitor.variables.Count);
-//			Console.WriteLine("Listing Variables:");
-//			IDictionaryEnumerator enumerator = lookupTableVisitor.variables.GetEnumerator();
-//			while (enumerator.MoveNext()) {
-//				Console.WriteLine(enumerator.Key);
-//			}
-//			Console.WriteLine("end listing");
-			ArrayList variables = (ArrayList)lookupTableVisitor.variables[name];
-			if (variables == null || variables.Count <= 0) {
-//				Console.WriteLine(name + " not in LookUpTable");
+			System.Collections.Generic.List<ICSharpCode.NRefactory.Parser.LocalLookupVariable> variables;
+			if (!lookupTableVisitor.Variables.TryGetValue (name, out variables) || variables.Count <= 0) {
 				return null;
 			}
 			
-			ReturnType found = null;
 			foreach (LocalLookupVariable v in variables) {
-//				Console.WriteLine("Position: ({0}/{1})", v.StartPos, v.EndPos);
 				if (IsInside(new Point(caretColumn, caretLine), v.StartPos, v.EndPos)) {
-					found = new ReturnType(v.TypeRef);
-//					Console.WriteLine("Variable found");
-					break;
+					IClass c = SearchType (v.TypeRef.SystemType, CompilationUnit);
+					DefaultRegion reg = new DefaultRegion (v.StartPos, v.EndPos);
+					reg.FileName = currentFile;
+					return new LocalVariable (name, new ReturnType (v.TypeRef, c), "", reg);
 				}
 			}
-			if (found == null) {
-//				Console.WriteLine("No Variable found");
-				return null;
-			}
-			return found;
+			return null;
 		}
 		
 		/// <remarks>
@@ -635,9 +632,9 @@ namespace CSharpBinding.Parser
 		public ILanguageItem IdentifierLookup (string id)
 		{
 			// try if it exists a variable named id
-			ReturnType variable = SearchVariable (id);
+			LocalVariable variable = SearchVariable (id);
 			if (variable != null) {
-				return new LocalVariable (id, variable, "");
+				return variable;
 			}
 			
 			if (callingClass == null) {
@@ -699,10 +696,10 @@ namespace CSharpBinding.Parser
 //			Console.WriteLine("name == " + typeName);
 			
 			// try if it exists a variable named typeName
-			ReturnType variable = SearchVariable(typeName);
+			LocalVariable variable = SearchVariable (typeName);
 			if (variable != null) {
 				showStatic = false;
-				return variable;
+				return variable.ReturnType;
 			}
 //			Console.WriteLine("No Variable found");
 			
@@ -851,9 +848,7 @@ namespace CSharpBinding.Parser
 //				Console.WriteLine("Found!");
 				return c;
 			}
-			Console.WriteLine("No FullName");
 			if (unit != null) {
-				Console.WriteLine(unit.Usings.Count + " Usings");
 				foreach (IUsing u in unit.Usings) {
 					if (u != null && (u.Region == null || u.Region.IsInside(caretLine, caretColumn))) {
 //						Console.WriteLine("In UsingRegion");
@@ -897,8 +892,8 @@ namespace CSharpBinding.Parser
 			if (possibleBaseClass.FullyQualifiedName == c.FullyQualifiedName) {
 				return true;
 			}
-			foreach (string baseClass in c.BaseTypes) {
-				IClass bc = parserContext.GetClass (baseClass, true, true);
+			foreach (IReturnType baseClass in c.BaseTypes) {
+				IClass bc = parserContext.GetClass (baseClass.FullyQualifiedName, true, true);
 				if (IsClassInInheritanceTree(possibleBaseClass, bc)) {
 					return true;
 				}
@@ -912,8 +907,8 @@ namespace CSharpBinding.Parser
 		/// </remarks>
 		IClass GetInnermostClass()
 		{
-			if (cu != null) {
-				foreach (IClass c in cu.Classes) {
+			if (currentUnit != null) {
+				foreach (IClass c in currentUnit.Classes) {
 					if (c != null && ((c.Region != null && c.Region.IsInside(caretLine, caretColumn)) ||
 						              (c.BodyRegion != null && c.BodyRegion.IsInside(caretLine, caretColumn))))
 					{
@@ -949,8 +944,8 @@ namespace CSharpBinding.Parser
 		ClassCollection GetOuterClasses()
 		{
 			ClassCollection classes = new ClassCollection();
-			if (cu != null) {
-				foreach (IClass c in cu.Classes) {
+			if (currentUnit != null) {
+				foreach (IClass c in currentUnit.Classes) {
 					if (c != null && c.Region != null && c.BodyRegion.IsInside(caretLine, caretColumn)) {
 						if (c != GetInnermostClass()) {
 							GetOuterClasses(classes, c);
@@ -992,33 +987,33 @@ namespace CSharpBinding.Parser
 			this.caretColumn = caretColumn;
 			
 			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
-			ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fcu = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+			ICSharpCode.NRefactory.Parser.AST.CompilationUnit fcu = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
 			if (fcu == null)
 				return null;
-			ICSharpCode.SharpRefactory.Parser.Parser p = new ICSharpCode.SharpRefactory.Parser.Parser ();
-			Lexer l = new Lexer (new StringReader (expression));
-			Expression expr = p.ParseExpression(l);
+			ICSharpCode.NRefactory.Parser.IParser p = ICSharpCode.NRefactory.Parser.ParserFactory.CreateParser (SupportedLanguage.CSharp, new StringReader (expression));
+			Expression expr = p.ParseExpression ();
 			if (expr == null)
 				return null;
 
-			lookupTableVisitor = new LookupTableVisitor ();
+			lookupTableVisitor = new LookupTableVisitor (StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit (fcu, null);
 
 			TypeVisitor typeVisitor = new TypeVisitor (this);
 
 			CSharpVisitor csharpVisitor = new CSharpVisitor ();
-			cu = (ICompilationUnit)csharpVisitor.Visit (fcu, null);
-			if (cu != null) {
+			currentUnit = (ICompilationUnit)csharpVisitor.Visit (fcu, null);
+			currentFile = fileName;
+			if (currentUnit != null) {
 				callingClass = GetInnermostClass ();
 			}
 
 			IReturnType type = expr.AcceptVisitor (typeVisitor, null) as IReturnType;
 			if (type == null || type.PointerNestingLevel != 0) {
-				fcu = parserContext.ParseFile (fileName, fileContent).MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+				fcu = parserContext.ParseFile (fileName, fileContent).MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
 				lookupTableVisitor.Visit (fcu, null);
-				cu = (ICompilationUnit)csharpVisitor.Visit (fcu, null);
+				currentUnit = (ICompilationUnit)csharpVisitor.Visit (fcu, null);
 
-				if (cu != null) {
+				if (currentUnit != null) {
 					callingClass = GetInnermostClass ();
 				}
 				type = expr.AcceptVisitor (typeVisitor, null) as IReturnType;
@@ -1028,8 +1023,8 @@ namespace CSharpBinding.Parser
 			if (type.ArrayDimensions != null && type.ArrayDimensions.Length > 0)
 				type = new ReturnType ("System.Array");
 
-			IClass returnClass = SearchType (type.FullyQualifiedName, cu);
-//			IClass returnClass = parserContext.SearchType (type.FullyQualifiedName, null, cu);
+			IClass returnClass = SearchType (type.FullyQualifiedName, currentUnit);
+//			IClass returnClass = parserContext.SearchType (type.FullyQualifiedName, null, currentUnit);
 			if (returnClass == null)
 				return null;
 
@@ -1043,27 +1038,28 @@ namespace CSharpBinding.Parser
 		public LanguageItemCollection CtrlSpace (int caretLine, int caretColumn, string fileName)
 		{
 			LanguageItemCollection result = new LanguageItemCollection ();
-			foreach (string pt in TypeReference.PrimitiveTypes)
-				result.Add (new Namespace (pt));
+			foreach (System.Collections.Generic.KeyValuePair<string, string> pt in TypeReference.GetPrimitiveTypesCSharp ())
+				result.Add (new Namespace (pt.Key));
 
 			this.caretLine = caretLine;
 			this.caretColumn = caretColumn;
 			IParseInformation parseInfo = parserContext.GetParseInformation (fileName);
-			ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.SharpRefactory.Parser.AST.CompilationUnit;
+			ICSharpCode.NRefactory.Parser.AST.CompilationUnit fileCompilationUnit = parseInfo.MostRecentCompilationUnit.Tag as ICSharpCode.NRefactory.Parser.AST.CompilationUnit;
 			if (fileCompilationUnit == null) {
 				Console.WriteLine("!Warning: no parseinformation!");
 				return null;
 			}
-			lookupTableVisitor = new LookupTableVisitor();
+			lookupTableVisitor = new LookupTableVisitor(StringComparer.InvariantCulture);
 			lookupTableVisitor.Visit(fileCompilationUnit, null);
 			CSharpVisitor cSharpVisitor = new CSharpVisitor();
-			cu = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
-			if (cu != null) {
+			currentUnit = (ICompilationUnit)cSharpVisitor.Visit(fileCompilationUnit, null);
+			currentFile = fileName;
+			if (currentUnit != null) {
 				callingClass = GetInnermostClass();
 				Console.WriteLine("CallingClass is " + (callingClass == null ? "null" : callingClass.Name));
 			}
-			foreach (string name in lookupTableVisitor.variables.Keys) {
-				ArrayList variables = (ArrayList)lookupTableVisitor.variables[name];
+			foreach (string name in lookupTableVisitor.Variables.Keys) {
+				ICollection variables = lookupTableVisitor.Variables[name];
 				if (variables != null && variables.Count > 0) {
 					foreach (LocalLookupVariable v in variables) {
 						if (IsInside(new Point(caretColumn, caretLine), v.StartPos, v.EndPos)) {
@@ -1078,7 +1074,7 @@ namespace CSharpBinding.Parser
 			}
 			string n = "";
 			result.AddRange(parserContext.GetNamespaceContents (n, true));
-			foreach (IUsing u in cu.Usings) {
+			foreach (IUsing u in currentUnit.Usings) {
 				if (u != null && (u.Region == null || u.Region.IsInside(caretLine, caretColumn))) {
 					foreach (string name in u.Usings) {
 						result.AddRange(parserContext.GetNamespaceContents (name, true));
