@@ -4,7 +4,8 @@ using System;
 using System.Diagnostics;
 
 using MonoDevelop.Projects.Parser;
-using ICSharpCode.SharpRefactory.Parser;
+using ICSharpCode.NRefactory.Parser;
+using ModifierFlags = ICSharpCode.NRefactory.Parser.AST.Modifier;
 
 namespace CSharpBinding.Parser.SharpDevelopTree
 {
