@@ -1,6 +1,7 @@
 using System;
 using MonoDevelop.Projects.Parser;
-using ICSharpCode.SharpRefactory.Parser;
+using ICSharpCode.NRefactory.Parser;
+using ModifierFlags = ICSharpCode.NRefactory.Parser.AST.Modifier;
 
 namespace CSharpBinding.Parser.SharpDevelopTree
 {
