@@ -51,6 +51,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		string fileName;
 		internal bool UpdatingWindow;
 		bool hasError;
+		bool needsUpdate = true;
 		
 		public event WindowEventHandler WindowAdded;
 		public event WindowEventHandler WindowRemoved;
@@ -68,6 +69,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 			 
 			this.project = project;
+			gproject.ResourceProvider = GtkCoreService.GetGtkInfo (project).ResourceProvider;
+			
 			gproject.WidgetAdded += new Stetic.Wrapper.WidgetEventHandler (OnAddWidget);
 			gproject.WidgetRemoved += new Stetic.Wrapper.WidgetEventHandler (OnRemoveWidget);
 			
@@ -145,12 +148,17 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			gproject.RemoveWidget ((Gtk.Container) win.RootWidget.Wrapped);
 		}
 	
+		public void Remove (Stetic.Wrapper.ActionGroup group)
+		{
+			gproject.ActionGroups.Remove (group);
+		}
+	
 		void OnAddWidget (object s, Stetic.Wrapper.WidgetEventArgs args)
 		{
 			if (UpdatingWindow)
 				return;
-			if (args.Widget.ParentWrapper == null) {
-				GuiBuilderWindow win = RegisterWindow (args.Widget);
+			if (args.WidgetWrapper.ParentWrapper == null) {
+				GuiBuilderWindow win = RegisterWindow (args.WidgetWrapper);
 				if (WindowAdded != null)
 					WindowAdded (this, new WindowEventArgs (win));
 			}
@@ -158,10 +166,10 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		void OnRemoveWidget (object s, Stetic.Wrapper.WidgetEventArgs args)
 		{
-			if (UpdatingWindow)
+			if (UpdatingWindow || args.WidgetWrapper == null)
 				return;
-			if (args.Widget.ParentWrapper == null) {
-				GuiBuilderWindow win = GetWindowForWidget (args.Widget);
+			if (args.WidgetWrapper.ParentWrapper == null) {
+				GuiBuilderWindow win = GetWindowForWidget (args.WidgetWrapper);
 				if (win != null) {
 					UnregisterWindow (win);
 					if (WindowRemoved != null)
@@ -210,20 +218,66 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			return null;
 		}
 		
-		public GuiBuilderWindow GetWindowForClass (IClass cls)
-		{
-			return GetWindowForClass (cls.FullyQualifiedName);
-		}
-		
 		public GuiBuilderWindow GetWindowForClass (string className)
 		{
 			foreach (GuiBuilderWindow form in formInfos) {
-				IClass fc = form.GetClass ();
-				if (fc != null && fc.FullyQualifiedName == className)
+				if (CodeBinder.GetObjectName (form.RootWidget) == className)
 					return form;
 			}
 			return null;
 		}
+		
+		public GuiBuilderWindow GetWindowForFile (string fileName)
+		{
+			foreach (GuiBuilderWindow win in Windows) {
+				if (fileName == win.SourceCodeFile)
+					return win;
+			}
+			return null;
+		}
+		
+		public Stetic.Wrapper.ActionGroup GetActionGroupForFile (string fileName)
+		{
+			foreach (Stetic.Wrapper.ActionGroup group in SteticProject.ActionGroups) {
+				if (fileName == GetSourceCodeFile (group))
+					return group;
+			}
+			return null;
+		}
+		
+		public string GetSourceCodeFile (object obj)
+		{
+			IClass cls = GetClass (obj);
+			if (cls != null) return cls.Region.FileName;
+			else return null;
+		}
+		
+		IClass GetClass (object obj)
+		{
+			string name = CodeBinder.GetClassName (obj);
+			return FindClass (name);
+		}
+		
+		public IClass FindClass (string className)
+		{
+			IParserContext ctx = GetParserContext ();
+			IClass[] classes = ctx.GetProjectContents ();
+			foreach (IClass cls in classes) {
+				if (cls.FullyQualifiedName == className)
+					return cls;
+			}
+			return null;
+		}
+		
+		public IParserContext GetParserContext ()
+		{
+			IParserContext ctx = IdeApp.ProjectOperations.ParserDatabase.GetProjectParserContext (Project);
+			if (needsUpdate) {
+				needsUpdate = false;
+				ctx.UpdateDatabase ();
+			}
+			return ctx;
+		}
 	}
 	
 	public delegate void WindowEventHandler (object s, WindowEventArgs args);
