@@ -75,30 +75,71 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			set { widgetTreePad = value; }
 		}
 		
+		public static GuiBuilderProject GetGuiBuilderProject (Project project)
+		{
+			GtkDesignInfo info = GtkCoreService.GetGtkInfo (project);
+			if (info != null)
+				return info.GuiBuilderProject;
+			else
+				return null;
+		}
+		
+		public static ActionGroupView OpenActionGroup (Project project, Stetic.Wrapper.ActionGroup group)
+		{
+			GuiBuilderProject p = GetGuiBuilderProject (project);
+			string file = p != null ? p.GetSourceCodeFile (group) : null;
+			if (file == null) {
+				file = ActionGroupDisplayBinding.BindToClass (project, group);
+			}
+			
+			Document doc = IdeApp.Workbench.OpenDocument (file, true);
+			if (doc != null) {
+				ActionGroupView view = doc.Content as ActionGroupView;
+				if (view != null) {
+					view.ShowDesignerView ();
+					return view;
+				}
+			}
+			return null;
+		}
+		
 		static void OnActiveDocumentChanged (object s, EventArgs args)
 		{
-			NotifyWidgetLibraryChange ();
-
 			if (IdeApp.Workbench.ActiveDocument == null) {
-				ActiveProject = EmptyProject;
-				if (widgetTreePad != null)
-					widgetTreePad.Fill (null);
-				RestoreLayout ();
+				if (ActiveProject != null) {
+					NotifyWidgetLibraryChange ();
+					ActiveProject = null;
+					if (widgetTreePad != null)
+						widgetTreePad.Fill (null);
+					RestoreLayout ();
+				}
 				return;
 			}
 
 			GuiBuilderView view = IdeApp.Workbench.ActiveDocument.Content as GuiBuilderView;
 			if (view != null) {
+				NotifyWidgetLibraryChange ();
 				ActiveProject = view.EditSession.SteticProject;
 				if (widgetTreePad != null)
 					widgetTreePad.Fill (view.EditSession.SteticProject);
 				SetDesignerLayout ();
 			}
-			else {
-				ActiveProject = EmptyProject;
-				if (widgetTreePad != null)
-					widgetTreePad.Fill (null);
-				RestoreLayout ();
+			else if (IdeApp.Workbench.ActiveDocument.Content is ActionGroupView) {
+				if (ActiveProject != null) {
+					NotifyWidgetLibraryChange ();
+					ActiveProject = null;
+					if (widgetTreePad != null)
+						widgetTreePad.Fill (null);
+					SetDesignerLayout ();
+				}
+			} else {
+				if (ActiveProject != null) {
+					NotifyWidgetLibraryChange ();
+					ActiveProject = null;
+					if (widgetTreePad != null)
+						widgetTreePad.Fill (null);
+					RestoreLayout ();
+				}
 			}
 		}
 		
@@ -252,8 +293,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			get { return activeProject; }
 			set {
 				activeProject = value;
-				if (activeProject == null)
-					activeProject = EmptyProject;
+//				if (activeProject == null)
+//					activeProject = EmptyProject;
 
 				if (ActiveProjectChanged != null)
 					ActiveProjectChanged (null, null);
@@ -264,7 +305,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			get {
 				Document doc = IdeApp.Workbench.ActiveDocument;
 				if (doc != null && IdeApp.ProjectOperations.CurrentOpenCombine != null) {
-					Project p = IdeApp.ProjectOperations.CurrentOpenCombine.GetProjectEntryContaining (doc.FileName);
+					Project p = IdeApp.ProjectOperations.CurrentOpenCombine.GetProjectContainingFile (doc.FileName);
 					GtkDesignInfo info = GtkCoreService.GetGtkInfo (p);
 					if (info != null)
 						return info.GetReferencedWidgetLibraries ();
@@ -273,6 +314,19 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			}
 		}
 		
+		public static Project GetProjectFromDesign (Stetic.Project project)
+		{
+			if (IdeApp.ProjectOperations.CurrentOpenCombine == null)
+				return null;
+				
+			foreach (Project prj in IdeApp.ProjectOperations.CurrentOpenCombine.GetAllProjects ()) {
+				GtkDesignInfo info = GtkCoreService.GetGtkInfo (prj);
+				if (info != null && info.GuiBuilderProject != null && info.GuiBuilderProject.SteticProject == project)
+					return prj;
+			}
+			return null;
+		}
+		
 		public static void NotifyWidgetLibraryChange ()
 		{
 			if (WidgetLibrariesChanged != null)
@@ -291,7 +345,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		internal static void JumpToSignalHandler (Stetic.Signal signal)
 		{
 			if (IdeApp.Workbench.ActiveDocument != null) {
-				GuiBuilderView view = IdeApp.Workbench.ActiveDocument.Content as GuiBuilderView;
+				CombinedDesignView view = IdeApp.Workbench.ActiveDocument.Content as CombinedDesignView;
 				if (view != null)
 					view.JumpToSignalHandler (signal);
 			}
