@@ -53,6 +53,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		GuiBuilderProject fproject;
 		
 		public event WindowEventHandler NameChanged;
+		public event WindowEventHandler Changed;
 		
 		internal GuiBuilderWindow (GuiBuilderProject fproject, Stetic.Project gproject, Stetic.Wrapper.Widget rootWidget)
 		{
@@ -61,6 +62,8 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			this.gproject = gproject;
 			gproject.WidgetNameChanged += new Stetic.Wrapper.WidgetNameChangedHandler (OnWidgetNameChanged);
 			gproject.ProjectReloaded += new EventHandler (OnProjectReloaded);
+			rootWidget.LocalActionGroups.ActionGroupAdded += new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
+			rootWidget.LocalActionGroups.ActionGroupRemoved += new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
 		}
 		
 		void OnProjectReloaded (object s, EventArgs a)
@@ -88,11 +91,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		}
 		
 		public string SourceCodeFile {
-			get {
-				IClass cls = GetClass ();
-				if (cls != null) return cls.Region.FileName;
-				else return null;
-			}
+			get { return fproject.GetSourceCodeFile (rootWidget); }
 		}
 		
 		public GuiBuilderEditSession CreateEditSession (ITextFileProvider textFileProvider)
@@ -104,11 +103,15 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		{
 			fproject.UpdatingWindow = true;
 			string oldName = rootWidget.Wrapped.Name;
+			rootWidget.LocalActionGroups.ActionGroupAdded -= new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
+			rootWidget.LocalActionGroups.ActionGroupRemoved -= new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
 			
 			try {
 				rootWidget.Delete ();
 				Gtk.Widget w = Stetic.WidgetUtils.ImportWidget (gproject, data);
 				rootWidget = Stetic.Wrapper.Container.Lookup (w);
+				rootWidget.LocalActionGroups.ActionGroupAdded += new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
+				rootWidget.LocalActionGroups.ActionGroupRemoved += new Stetic.Wrapper.ActionGroupEventHandler (OnGroupsChanged);
 				gproject.AddWidget ((Gtk.Widget) rootWidget.Wrapped);
 			} finally {
 				fproject.UpdatingWindow = false;
@@ -131,27 +134,24 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		
 		void OnWidgetNameChanged (object s, Stetic.Wrapper.WidgetNameChangedArgs args)
 		{
-			if (!InsideWindow (args.Widget))
+			if (!InsideWindow (args.WidgetWrapper))
 				return;
 			
-			if (args.Widget == rootWidget && NameChanged != null)
+			if (args.WidgetWrapper == rootWidget && NameChanged != null) {
 				NameChanged (this, new WindowEventArgs (this));
-		}
-		
-		public IClass GetClass ()
-		{
-			return GetClass (rootWidget.Wrapped.Name);
-		}
-		
-		internal IClass GetClass (string name)
-		{
-			IParserContext ctx = IdeApp.ProjectOperations.ParserDatabase.GetProjectParserContext (fproject.Project);
-			IClass[] classes = ctx.GetProjectContents ();
-			foreach (IClass cls in classes) {
-				if (cls.FullyQualifiedName == name)
-					return cls;
 			}
-			return null;
+			OnChanged ();
+		}
+		
+		void OnGroupsChanged (object s, Stetic.Wrapper.ActionGroupEventArgs args)
+		{
+			OnChanged ();
+		}
+		
+		protected virtual void OnChanged ()
+		{
+			if (Changed != null)
+				Changed (this, new WindowEventArgs (this));
 		}
 		
 		public bool InsideWindow (Stetic.Wrapper.Widget widget)
@@ -169,25 +169,30 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 			// Find the classes that could be bound to this design
 			
 			ArrayList list = new ArrayList ();
-			IParserContext ctx = IdeApp.ProjectOperations.ParserDatabase.GetProjectParserContext (Project.Project);
+			IParserContext ctx = fproject.GetParserContext ();
 			foreach (IClass cls in ctx.GetProjectContents ())
 				if (IsValidClass (ctx, cls))
 					list.Add (cls.FullyQualifiedName);
 		
 			// Ask what to do
-			
-			using (BindDesignDialog dialog = new BindDesignDialog (Name, list, Project.Project.BaseDirectory)) {
-				if (!dialog.Run ())
-					return false;
-				
-				if (dialog.CreateNew)
-					CreateClass (dialog.ClassName, dialog.Namespace, dialog.Folder);
 
-				string fullName = dialog.Namespace.Length > 0 ? dialog.Namespace + "." + dialog.ClassName : dialog.ClassName;
-				rootWidget.Wrapped.Name = fullName;
-				Save ();
+			try {
+				using (BindDesignDialog dialog = new BindDesignDialog (Name, list, Project.Project.BaseDirectory)) {
+					if (!dialog.Run ())
+						return false;
+					
+					if (dialog.CreateNew)
+						CreateClass (dialog.ClassName, dialog.Namespace, dialog.Folder);
+
+					string fullName = dialog.Namespace.Length > 0 ? dialog.Namespace + "." + dialog.ClassName : dialog.ClassName;
+					rootWidget.Wrapped.Name = fullName;
+					Save ();
+				}
+				return true;
+			} catch (Exception ex) {
+				IdeApp.Services.MessageService.ShowError (ex);
+				return false;
 			}
-			return true;
 		}
 		
 		void CreateClass (string name, string namspace, string folder)
@@ -210,7 +215,7 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 				if (prop.PropertyType.IsValueType)
 					ctor.BaseConstructorArgs.Add (new CodePrimitiveExpression (Activator.CreateInstance (prop.PropertyType)));
 				else
-					ctor.BaseConstructorArgs.Add (null);
+					ctor.BaseConstructorArgs.Add (new CodePrimitiveExpression (null));
 			}
 			
 			CodeMethodInvokeExpression call = new CodeMethodInvokeExpression (
@@ -254,11 +259,11 @@ namespace MonoDevelop.GtkCore.GuiBuilder
 		internal bool IsValidClass (IParserContext ctx, IClass cls)
 		{
 			if (cls.BaseTypes != null) {
-				foreach (string bt in cls.BaseTypes) {
-					if (bt == rootWidget.Wrapped.GetType().FullName)
+				foreach (IReturnType bt in cls.BaseTypes) {
+					if (bt.FullyQualifiedName == rootWidget.Wrapped.GetType().FullName)
 						return true;
 					
-					IClass baseCls = ctx.GetClass (bt, true, true);
+					IClass baseCls = ctx.GetClass (bt.FullyQualifiedName, true, true);
 					if (IsValidClass (ctx, baseCls))
 						return true;
 				}
