@@ -32,6 +32,10 @@ namespace MonoDevelop.SourceEditor.Gui
 			GtkSourceViewManager.Init ();
 		}
 
+		public string DisplayName {
+			get { return "Source Code Editor"; }
+		}
+		
 		public virtual bool CanCreateContentForFile (string fileName)
 		{
 			return false;
@@ -49,14 +53,18 @@ namespace MonoDevelop.SourceEditor.Gui
 				return true;
 			if (mimetype == "application/x-aspx")
 				return true;
+
+			// If gedit can open the file, this editor also can do it
+			foreach (DesktopApplication app in DesktopApplication.GetApplications (mimetype))
+				if (app.Command == "gedit")
+					return true;
+				
 			return false;
 		}
 		
 		public virtual IViewContent CreateContentForFile (string fileName)
 		{
 			SourceEditorDisplayBindingWrapper w = new SourceEditorDisplayBindingWrapper ();
-			
-			w.Load (fileName);
 			return w;
 		}
 		
@@ -74,7 +82,7 @@ namespace MonoDevelop.SourceEditor.Gui
 	
 	public class SourceEditorDisplayBindingWrapper : AbstractViewContent,
 		IEditableTextBuffer, IPositionable, IBookmarkBuffer, IDebuggableEditor, ICodeStyleOperations,
-		IDocumentInformation
+		IDocumentInformation, IEncodedTextContent
 	{
 		VBox mainBox;
 		HBox editorBar;
@@ -229,6 +237,7 @@ namespace MonoDevelop.SourceEditor.Gui
 			se.Dispose ();
 			fsw.Dispose ();
 			se = null;
+			base.Dispose ();
 		}
 		
 		void OnModifiedChanged (object o, EventArgs e)
@@ -253,6 +262,11 @@ namespace MonoDevelop.SourceEditor.Gui
 		}
 		
 		public override void Save (string fileName)
+		{
+			Save (fileName, null);
+		}
+		
+		public void Save (string fileName, string encoding)
 		{
 			if (warnOverwrite) {
 				if (fileName == ContentName) {
@@ -265,14 +279,21 @@ namespace MonoDevelop.SourceEditor.Gui
 			}
 
 			lock (fileSaveLock) {
-				se.Buffer.Save (fileName);
+				se.Buffer.Save (fileName, encoding);
 				lastSaveTime = File.GetLastWriteTime (fileName);
 			}
+			if (encoding != null)
+				se.Buffer.SourceEncoding = encoding;
 			ContentName = fileName;
 			InitializeFormatter ();
 		}
 		
 		public override void Load (string fileName)
+		{
+			Load (fileName, null);
+		}
+		
+		public void Load (string fileName, string encoding)
 		{
 			if (warnOverwrite) {
 				warnOverwrite = false;
@@ -283,7 +304,7 @@ namespace MonoDevelop.SourceEditor.Gui
 			vfsname = vfsname.Replace ("%", "%25");
 			vfsname = vfsname.Replace ("#", "%23");
 			vfsname = vfsname.Replace ("?", "%3F");
-			se.Buffer.LoadFile (fileName, Gnome.Vfs.MimeType.GetMimeTypeForUri (vfsname));
+			se.Buffer.LoadFile (fileName, Gnome.Vfs.MimeType.GetMimeTypeForUri (vfsname), encoding);
 			ContentName = fileName;
 			InitializeFormatter ();
 			
@@ -417,10 +438,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		public void LoadString (string mime, string val)
 		{
-			if (mime != null)
-				se.Buffer.LoadText (val, mime);
-			else
-				se.Buffer.LoadText (val);
+			se.Buffer.LoadText (val, mime);
 		}
 		
 #region IEditableTextBuffer
@@ -515,6 +533,10 @@ namespace MonoDevelop.SourceEditor.Gui
 			remove { se.Buffer.Changed -= value; }
 		}
 		
+		public string SourceEncoding {
+			get { return se.Buffer.SourceEncoding; }
+		}
+		
 #endregion
 
 #region Status Bar Handling
