@@ -36,6 +36,14 @@ namespace MonoDevelop.SourceEditor.Gui
 		bool autoHideCompletionWindow = true;
 		bool autoInsertTemplates;
 		EditActionCollection editactions = new EditActionCollection ();
+		LanguageItemWindow languageItemWindow;
+		CodeCompletionDataProvider currentCompletionProvider;
+		
+		const int LanguageItemTipTimer = 800;
+		ILanguageItem tipItem;
+		bool showTipScheduled;
+		int langTipX, langTipY;
+		uint tipTimeoutId;
 
 		public bool EnableCodeCompletion {
 			get { return codeCompleteEnabled; }
@@ -65,15 +73,107 @@ namespace MonoDevelop.SourceEditor.Gui
 			buf.MarkSet += new MarkSetHandler (BufferMarkSet);
 			buf.Changed += new EventHandler (BufferChanged);
 			LoadEditActions ();
+			this.Events = this.Events | EventMask.PointerMotionMask | EventMask.LeaveNotifyMask;
 		}
 		
 		public new void Dispose ()
 		{
+			if (currentCompletionProvider != null) {
+				currentCompletionProvider.Dispose ();
+				currentCompletionProvider = null;
+			}
+			HideLanguageItemWindow ();
 			buf.MarkSet -= new MarkSetHandler (BufferMarkSet);
 			buf.Changed -= new EventHandler (BufferChanged);
 			base.Dispose ();
 		}
 		
+		protected override bool OnMotionNotifyEvent (Gdk.EventMotion evnt)
+		{
+			bool res = base.OnMotionNotifyEvent (evnt);
+			UpdateLanguageItemWindow ();
+			return res;
+		}
+		
+		void UpdateLanguageItemWindow ()
+		{
+			if (languageItemWindow != null) {
+				// Tip already being shown. Update it.
+				ShowTooltip ();
+			}
+			else if (showTipScheduled) {
+				// Tip already scheduled. Reset the timer.
+				GLib.Source.Remove (tipTimeoutId);
+				tipTimeoutId = GLib.Timeout.Add (LanguageItemTipTimer, ShowTooltip);
+			}
+			else {
+				// Start a timer to show the tip
+				showTipScheduled = true;
+				tipTimeoutId = GLib.Timeout.Add (LanguageItemTipTimer, ShowTooltip);
+			}
+		}
+		
+		bool ShowTooltip ()
+		{
+			ModifierType mask; // ignored
+			int xloc, yloc;
+
+			showTipScheduled = false;
+				
+			this.GetWindow (TextWindowType.Text).GetPointer (out xloc, out yloc, out mask);
+
+			TextIter ti = this.GetIterAtLocation (xloc + this.VisibleRect.X, yloc + this.VisibleRect.Y);
+			ILanguageItem item = GetLanguageItem (ti);
+			
+			if (item != null) {
+				// Tip already being shown for this language item?
+				if (languageItemWindow != null && tipItem != null && tipItem.Equals (item))
+					return false;
+				
+				langTipX = xloc;
+				langTipY = yloc;
+				tipItem = item;
+
+				HideLanguageItemWindow ();
+
+				languageItemWindow = new LanguageItemWindow (tipItem, GetParserContext (), GetAmbience ());
+				
+				int ox, oy;
+				this.GetWindow (TextWindowType.Text).GetOrigin (out ox, out oy);
+				int w = languageItemWindow.Child.SizeRequest().Width;
+				languageItemWindow.Move (langTipX + ox - (w/2), langTipY + oy + 20);
+				languageItemWindow.ShowAll ();
+			} else
+				HideLanguageItemWindow ();
+			
+			return false;
+		}
+		
+
+		protected override bool OnLeaveNotifyEvent (Gdk.EventCrossing evnt)		
+		{
+			HideLanguageItemWindow ();
+			return base.OnLeaveNotifyEvent (evnt);
+		}
+		
+		protected override bool OnScrollEvent (Gdk.EventScroll evnt)
+		{
+			HideLanguageItemWindow ();
+			return base.OnScrollEvent (evnt);
+		}
+		
+		public void HideLanguageItemWindow ()
+		{
+			if (showTipScheduled) {
+				GLib.Source.Remove (tipTimeoutId);
+				showTipScheduled = false;
+			}
+			if (languageItemWindow != null) {
+				languageItemWindow.Destroy ();
+				languageItemWindow = null;
+			}
+		}
+		
 		void BufferMarkSet (object s, MarkSetArgs a)
 		{
 			if (a.Mark.Name == "insert") {
@@ -110,6 +210,7 @@ namespace MonoDevelop.SourceEditor.Gui
 		protected override bool OnButtonPressEvent (Gdk.EventButton e)
 		{
 			CompletionListWindow.HideWindow ();
+			HideLanguageItemWindow ();
 			
 			if (!ShowLineMarkers)
 				return base.OnButtonPressEvent (e);
@@ -147,6 +248,8 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		protected override void OnPopulatePopup (Menu menu)
 		{
+			HideLanguageItemWindow ();
+			
 			CommandEntrySet cset = IdeApp.CommandService.CreateCommandEntrySet ("/SharpDevelop/ViewContent/DefaultTextEditor/ContextMenu");
 			if (cset.Count > 0) {
 				cset.AddItem (Command.Separator);
@@ -252,12 +355,26 @@ namespace MonoDevelop.SourceEditor.Gui
 			else
 				return pdb.GetFileParserContext (file);
 		}
+		
+		MonoDevelop.Projects.Ambience.Ambience GetAmbience ()
+		{
+			Project project = ParentEditor.DisplayBinding.Project;
+			if (project != null)
+				return project.Ambience;
+			else {
+				string file = ParentEditor.DisplayBinding.IsUntitled ? ParentEditor.DisplayBinding.UntitledName : ParentEditor.DisplayBinding.ContentName;
+				return MonoDevelop.Projects.Services.Ambience.GetAmbienceForFile (file);
+			}
+		}
 
 		CodeCompletionDataProvider GetCodeCompletionDataProvider (bool ctrl)
 		{
+			if (currentCompletionProvider != null)
+				currentCompletionProvider.Dispose ();
 			IParserContext ctx = GetParserContext ();
 			string file = ParentEditor.DisplayBinding.IsUntitled ? ParentEditor.DisplayBinding.UntitledName : ParentEditor.DisplayBinding.ContentName;
-			return new CodeCompletionDataProvider (ctx, file, ctrl);
+			currentCompletionProvider = new CodeCompletionDataProvider (ctx, GetAmbience (), file, ctrl);
+			return currentCompletionProvider;
 		}
 			
 		internal bool MonodocResolver ()
@@ -286,14 +403,7 @@ namespace MonoDevelop.SourceEditor.Gui
 			} catch {
 				return false;
 			}
-			insertIter = triggerIter;
-			string fileName = ParentEditor.DisplayBinding.ContentName;
-			IExpressionFinder expressionFinder = GetParserContext ().GetExpressionFinder(fileName);
-			string expression    = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset(this, insertIter.Offset) : expressionFinder.FindExpression(buf.GetText(buf.StartIter, insertIter, true), insertIter.Offset - 2).Expression;
-			if (expression == null) return false;
-			Console.WriteLine ("Expression: {" + expression + "}");
-
-			ILanguageItem languageItem = GetParserContext ().ResolveIdentifier(expression, insertIter.Line + 1, insertIter.LineOffset + 1, fileName, buf.Text);
+			ILanguageItem languageItem = GetLanguageItem (triggerIter);
 			
 			if (languageItem == null)
 				return false;
@@ -302,6 +412,25 @@ namespace MonoDevelop.SourceEditor.Gui
 			
 			return true;
 		}
+		
+		ILanguageItem GetLanguageItem (TextIter ti)
+		{
+			string txt = buf.Text;
+			string fileName = ParentEditor.DisplayBinding.ContentName;
+			if (fileName == null)
+				fileName = ParentEditor.DisplayBinding.UntitledName;
+
+			IParserContext ctx = GetParserContext ();
+			IExpressionFinder expressionFinder = null;
+			if (fileName != null)
+				expressionFinder = ctx.GetExpressionFinder (fileName);
+
+			string expression = expressionFinder == null ? TextUtilities.GetExpressionBeforeOffset (this, ti.Offset) : expressionFinder.FindFullExpression (txt, ti.Offset).Expression;
+			if (expression == null)
+				return null;
+
+			return ctx.ResolveIdentifier (expression, ti.Line + 1, ti.LineOffset + 1, fileName, txt);
+		}
 
 		internal void ScrollUp ()
 		{
@@ -327,6 +456,8 @@ namespace MonoDevelop.SourceEditor.Gui
 		
 		protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
 		{
+			HideLanguageItemWindow ();
+			
 			if (CompletionListWindow.ProcessKeyEvent (evnt))
 				return true;
 			
@@ -704,10 +835,11 @@ namespace MonoDevelop.SourceEditor.Gui
 		void ICompletionWidget.SetCompletionText (string partial_word, string complete_word)
 		{
 			TextIter offsetIter = buf.GetIterAtMark(triggerMark);
-                        TextIter endIter = buf.GetIterAtOffset (offsetIter.Offset + partial_word.Length);
-                        buf.MoveMark (buf.InsertMark, offsetIter);
-                        buf.Delete (ref offsetIter, ref endIter);
-                        buf.InsertAtCursor (complete_word);
+			TextIter endIter = buf.GetIterAtOffset (offsetIter.Offset + partial_word.Length);
+			buf.MoveMark (buf.InsertMark, offsetIter);
+			buf.Delete (ref offsetIter, ref endIter);
+			buf.InsertAtCursor (complete_word);
+			ScrollMarkOnscreen (buf.InsertMark);
 		}
 
 		void ICompletionWidget.InsertAtCursor (string text)
