@@ -44,7 +44,8 @@ namespace MonoDeveloper
 		string outFile;
 		ArrayList refNames = new ArrayList ();
 		bool loading;
-		MonoTestSuite testSuite;
+		string testFileBase;
+		object unitTest;
 		
 		public override string ProjectType {
 			get { return "MonoMakefile"; }
@@ -55,6 +56,12 @@ namespace MonoDeveloper
 			Read (mkfile);
 		}
 		
+		public override string [] SupportedLanguages {
+			get {
+				return new string [] { "C#", "" };
+			}
+		}
+		
 		void Read (MonoMakefile mkfile)
 		{
 			loading = true;
@@ -122,8 +129,7 @@ namespace MonoDeveloper
 			
 			if (mkfile.GetVariable ("NO_TEST") != "yes") {
 				string tname = Path.GetFileNameWithoutExtension (aname) + "_test_";
-				string testFileBase = Path.Combine (basePath, tname);
-				testSuite = new MonoTestSuite (this, Name, testFileBase);
+				testFileBase = Path.Combine (basePath, tname);
 			}
 			
 			MonoProjectConfiguration conf = new MonoProjectConfiguration ("default", "default");
@@ -318,19 +324,20 @@ namespace MonoDeveloper
 			return found;
 		}
 		
-		public override void GenerateMakefiles (Combine parentCombine)
-		{
-		}
-		
 		public override void Dispose ()
 		{
 			base.Dispose ();
 			IdeApp.ProjectOperations.CombineOpened -= new CombineEventHandler (CombineOpened);
 		}
 		
-		internal MonoTestSuite GetTestSuite ()
+		public string GetTestFileBase ()
 		{
-			return testSuite;
+			return testFileBase;
+		}
+		
+		public object UnitTest {
+			get { return unitTest; }
+			set { unitTest = value; }
 		}
 	}
 }
