@@ -52,13 +52,34 @@ namespace MonoDevelop.NUnit
 		
 		public override void InitializeService ()
 		{
-			RegisterTestProvider (new SystemTestProvider ());
 			IdeApp.ProjectOperations.CombineOpened += (CombineEventHandler) MonoDevelop.Core.Gui.Services.DispatchService.GuiDispatch (new CombineEventHandler (OnOpenCombine));
 			IdeApp.ProjectOperations.CombineClosed += (CombineEventHandler) MonoDevelop.Core.Gui.Services.DispatchService.GuiDispatch (new CombineEventHandler (OnCloseCombine));
 			
 			IProjectService ps = MonoDevelop.Projects.Services.ProjectService;
 			ps.DataContext.IncludeType (typeof(UnitTestOptionsSet));
 			ps.DataContext.RegisterProperty (typeof(AbstractConfiguration), "UnitTestInformation", typeof(UnitTestOptionsSet));
+			
+			Runtime.AddInService.RegisterExtensionItemListener ("/Services/NUnit/TestProviders", OnExtensionChange);
+		}
+		
+		void OnExtensionChange (ExtensionAction action, object item)
+		{
+			if (action == ExtensionAction.Add) {
+				IProjectService ps = MonoDevelop.Projects.Services.ProjectService;
+				ITestProvider provider = item as ITestProvider;
+				providers.Add (provider);
+				
+				Type[] types = provider.GetOptionTypes ();
+				if (types != null) {
+					foreach (Type t in types) {
+						if (!typeof(ICloneable).IsAssignableFrom (t)) {
+							Console.WriteLine ("Option types must implement ICloneable: " + t);
+							continue;
+						}
+						ps.DataContext.IncludeType (t);
+					}
+				}
+			}
 		}
 		
 		public IAsyncOperation RunTest (UnitTest test)
@@ -131,20 +152,6 @@ namespace MonoDevelop.NUnit
 			get { return rootTest; }
 		}
 		
-		public void RegisterTestProvider (ITestProvider provider)
-		{
-			providers.Add (provider);
-			Type[] types = provider.GetOptionTypes ();
-			if (types != null) {
-				foreach (Type t in types) {
-					if (!typeof(ICloneable).IsAssignableFrom (t))
-						throw new InvalidOperationException ("Option types must implement ICloneable: " + t);
-					IProjectService ps = MonoDevelop.Projects.Services.ProjectService;
-					ps.DataContext.IncludeType (t);
-				}
-			}
-		}
-		
 		public static void ShowOptionsDialog (UnitTest test)
 		{
 			UnitTestOptionsDialog optionsDialog = new UnitTestOptionsDialog (IdeApp.Workbench.RootWindow, test);
