@@ -37,10 +37,10 @@ namespace VBBinding.Parser.SharpDevelopTree
 			this.pointerNestingLevel = pointerNestingLevel;
 		}
 		
-		public ReturnType(ICSharpCode.SharpRefactory.Parser.AST.VB.TypeReference type)
+		public ReturnType(ICSharpCode.NRefactory.Parser.AST.TypeReference type)
 		{
 			base.FullyQualifiedName  = type.SystemType == null ? type.Type : type.SystemType;
-			base.arrayDimensions     = type.RankSpecifier == null ? new int[] { } : (int[])type.RankSpecifier.ToArray(typeof(int));
+			base.arrayDimensions     = type.RankSpecifier == null ? new int[] { } : type.RankSpecifier;
 			base.pointerNestingLevel = 0;
 		}
 		
