@@ -240,7 +240,11 @@ namespace VersionControlPlugin {
 		
 		protected virtual void SaveAs(string fileName) {
 		}
-
+		
+		string IViewContent.StockIconId {
+			get { return null; }
+		}
+		
 		void IViewContent.Load(string fileName) {
 			throw new InvalidOperationException();
 		}
@@ -282,7 +286,7 @@ namespace VersionControlPlugin {
 			set { }
 		}
 		
-		string IViewContent.TabPageLabel {
+		string IBaseViewContent.TabPageLabel {
 			get { return name; }
 		}
 		
