@@ -24,6 +24,7 @@ OTHER DEALINGS IN THE SOFTWARE.
 */
 using System;
 using System.Text;
+using System.Text.RegularExpressions;
 
 using Gtk;
 
@@ -48,12 +49,20 @@ namespace MonoDevelop.WelcomePage
 		protected Frame control;
 		protected MozillaControl htmlControl;
 		
+		string datadir;
+		
 		public override Gtk.Widget Control {
 			get {
 				return control;
 			}
 		}
 		
+		public override string StockIconId {
+			get {
+				return Gtk.Stock.Home;
+			}
+		}
+		
 		public override void Load(string fileName) 
 		{
 			//Initialize(null);
@@ -61,7 +70,7 @@ namespace MonoDevelop.WelcomePage
 
 		public WelcomePageView() : base()
 		{
-			this.ContentName = "Welcome";
+			this.ContentName = GettextCatalog.GetString("Welcome");
 			
 			control = new Frame();
 			control.Show();
@@ -72,13 +81,17 @@ namespace MonoDevelop.WelcomePage
 			htmlControl.OpenUri += new OpenUriHandler (CatchUri);
 			htmlControl.LinkMsg += new EventHandler (LinkMessage);
 			
-			//START HERE: Move To Thread?
-			
-			string datadir = Path.GetDirectoryName (typeof(ShowWelcomePageHandler).Assembly.Location) + "/";
-			//Console.WriteLine(datadir);
-			
-			//string datadir = Runtime.Properties.DataDirectory;
+			datadir = Path.GetDirectoryName (typeof(ShowWelcomePageHandler).Assembly.Location) + "/";
 
+			this.IsViewOnly = true;
+
+			LoadContent ();
+
+			IdeApp.Workbench.RecentOpen.RecentProjectChanged += RecentChangesHandler;
+		}
+		
+		void LoadContent ()
+		{
 			// build simple xml which XSLT will process into XHTML
 			string myxml = 	"<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>" +
 							"<WelcomePage>" +
@@ -94,15 +107,20 @@ namespace MonoDevelop.WelcomePage
 			StringWriter fs = new StringWriter();
 			xslt.Transform(inxml, null, fs, null);
 
-			//Console.WriteLine(fs.ToString());
 			htmlControl.Html = fs.ToString();
 			//Initialize(null);
 		}
+
+		void RecentChangesHandler ( object sender, EventArgs e )
+		{
+			LoadContent ();
+			Initialize (null);
+		}
 		
 		void LinkMessage (object sender, EventArgs e)
 		{
-			if (htmlControl.LinkMessage.IndexOf ("monodevelop://") != -1) return;
-			if (htmlControl.LinkMessage == null || htmlControl.LinkMessage == String.Empty)
+			if (htmlControl.LinkMessage == null || htmlControl.LinkMessage == String.Empty
+				|| htmlControl.LinkMessage.IndexOf ("monodevelop://") != -1)
 			{
 				IdeApp.Workbench.StatusBar.SetMessage (null);
 			} else
@@ -118,10 +136,12 @@ namespace MonoDevelop.WelcomePage
 			e.RetVal = true;
 	
 			string URI = e.AURI;
+
 			if (URI.StartsWith("project://"))
 			{
-				string projectUri = URI.Substring(10);
-				IdeApp.ProjectOperations.OpenCombine(projectUri);
+				string projectUri = URI.Substring(10);			
+				Uri fileuri = new Uri ( projectUri );
+				IdeApp.ProjectOperations.OpenCombine ( fileuri.LocalPath );
 			}
 			else if (URI.StartsWith("monodevelop://"))
 			{
@@ -175,40 +195,37 @@ namespace MonoDevelop.WelcomePage
 		public void Initialize(object obj)
 		{
 			htmlControl.DelayedInitialize();
-			//object t = new object();
-			//htmlControl.Navigate("http://www.google.com", ref t, ref t,ref t,ref t); 
 		}
 
+		public override void Dispose ()
+		{
+			base.Dispose ();
+
+			IdeApp.Workbench.RecentOpen.RecentProjectChanged -= RecentChangesHandler;
+			htmlControl.Dispose ();
+		}
+		
 		public static string TimeSinceEdited(int timestamp)
 		{
 			DateTime prjtime = (new DateTime (1970, 1, 1, 0, 0, 0, 0)).AddSeconds(timestamp);
 			TimeSpan sincelast = DateTime.UtcNow - prjtime;
 
-			if (sincelast.Days > 1)
+			if (sincelast.Days >= 1)
 			{
-				return sincelast.Days + " days";
+				return GettextCatalog.GetPluralString("{0} day", "{0} days", sincelast.Days, sincelast.Days);
 			}
-			else if (sincelast.Days == 1)
+			else if (sincelast.Hours >= 1)
 			{
-				return "One day";
-			}
-			else if (sincelast.Hours > 1)
-			{
-				return sincelast.Hours + " hours";
-			}
-			else if (sincelast.Hours == 1)
-			{
-				return "One hour";
+				return GettextCatalog.GetPluralString("{0} hour", "{0} hours", sincelast.Hours, sincelast.Hours);
 			}
 			else if (sincelast.Minutes > 0)
 			{
-				return sincelast.Minutes + " minutes";
+				return GettextCatalog.GetPluralString("{0} minute", "{0} minutes", sincelast.Minutes, sincelast.Minutes);
 			}
 			else
 			{
-				return "Less than a minute";
+				return GettextCatalog.GetString("Less than a minute");
 			}
 		}
-
 	}
 }
